/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2016, 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dojo/string", "dijit/layout/ContentPane", "dijit/tree/ForestStoreModel", "dojo/data/ItemFileReadStore", "dojox/layout/TableContainer",
		"dojox/grid/LazyTreeGrid", "dijit/form/Button", "./gLinesPane", "dijit/Tooltip", 
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",		
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function (string, ContentPane, ForestStoreModel, ItemFileReadStore, TableContainer, LazyTreeGrid, Button, gLinesPane, Tooltip, PacbaseEditor, typeDict ) {
	
	function getValue ( value ) {		
		return value.toString().replace("_","").replace("None","");
	}
	
	function getNetworkType ( item ) {
		if (item.networkRecordType != undefined && item.networkRecordType != "") {
			var type = getValue(item.networkRecordType[0].value);
			if (type == "STAR") {
				type = "*"
			}
			return "<span class='display_icon pacDCLine'></span>&nbsp" + type;
		}
		return "";
	}
	
	function getNetworkName ( item ) {	
		if (item.dataBaseObjectName != undefined) 
			return item.dataBaseObjectName;
		return "";
	}
	
	function getParentSegment ( item , project, streamId) {
		if (item.segment != undefined && item.segment != "") {			
			return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", item.segment[0], false, streamId, project, "");
		}
		return "";
	}
	
	function getChildSegment ( item , project, streamId) {
		if (item.child != undefined && item.child != "") {			
			return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", item.child[0], false, streamId, project, "");
		}
		return "";		
	}
	
	function getFromSegment ( item ) {	
		if (item.fromSegment != undefined && item.fromSegment != "")		
			return item.fromSegment[0].name;
		return "";
	}
	
	function getMethod ( item ) {	
		if (item.methodCode != undefined) 
			return item.methodCode;
		return "";
	}
	
	function getOccurence ( item ) {	
		if (item.numberOccurrencesSet != undefined && item.numberOccurrencesSet != "0")		
			return item.numberOccurrencesSet;
		return "";
	}	
	
	function getAreaName ( item ) {	
		if (item.commentOrName != undefined) 
			return item.commentOrName;
		return "";
	}
	
	return {		
		getDetailDCLines : function ( data, type, project, streamId, compentId, fileItemId ) {
			var DCLines = data.DCLines
			var cpDetailDCLines = new ContentPane( {
				content : "<strong>" + typeDict[ type ] + " " + data.name + " : " + data.label + "</strong><P>"
			    + "<strong>" + PacbaseEditor.DCLINE_TABLE_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
						+ string.substitute( PacbaseEditor.DCLINE_TABLE_SECTION_DESCRIPTION, [
							typeDict[ type ]
						] ) + "</p>",
						style : {
							'overflow' : 'auto',
						},
			} );
			// prepare lazy tree grid
			var layoutDCLines = [
					{
						name : PacbaseEditor.BLOCKBASE_DCLINE_COLUMN_NETWORK_TYPE,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getNetworkType( item );
						},
					}, {
						name : PacbaseEditor.BLOCKBASE_DCLINE_COLUMN_NETWORK_NAME,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getNetworkName( item );
						},
					}, {
						name : PacbaseEditor.BLOCKBASE_DCLINE_COLUMN_PARENT_SEG,
						width : "70px",
						field : "_item",
						formatter : function ( item , project, streamId) {
							return getParentSegment( item );
						},
					}, {
						name : PacbaseEditor.BLOCKBASE_DCLINE_COLUMN_CHILD_SEG,
						width : "70px",
						field : "_item",
						formatter : function ( item , project, streamId) {
							return getChildSegment( item );
						},
					}, {
						name : PacbaseEditor.BLOCKBASE_DCLINE_COLUMN_FROM_SEG,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getFromSegment( item );
						},
					}, {
						name : PacbaseEditor.BLOCKBASE_DCLINE_COLUMN_METHOD,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getMethod( item );
						},
					}, {
						name : PacbaseEditor.BLOCKBASE_DCLINE_COLUMN_OCC,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getOccurence( item );
						},
					}, {
						name : PacbaseEditor.BLOCKBASE_DCLINE_COLUMN_AREA_NAME,
						width : "300px",
						field : "_item",
						formatter : function ( item ) {
							return getAreaName( item );
						},
					}, {
						name : PacbaseEditor.BLOCKBASE_LINE_COLUMN_COMMENT,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							if ( item.GGLines == undefined || item.GGLines[0] == undefined )
								return "";							
							var hasComment = false;
							for(var i = 0; i < item.GGLines.length; i++) {							
								  if (item.GGLines[i]["class"] != "PacGenerationElementVirtual" && item.GGLines[i]["class"] != "PacGenerationElementGuide") {
									  hasComment = true;
									  break;
								  }
								}							
							if ( hasComment == false )
								return "";
							var w = new dijit.form.Button({
								label: PacbaseEditor.GLINE_TABTEXT,
								showLabel: false,
								iconClass:'arrrow',
								onClick: function(){
									// -G Lines
									var cpGLines = new ContentPane( {
										title : PacbaseEditor.GLINE_TABTEXT,
										style:"width:1000px;height:500px;"
									} );
									cpGLines.addChild( gLinesPane.getGLines ( item, type, false, false, true, false, streamId, project ) );
								    dlg = new dijit.Dialog({
								        title:PacbaseEditor.GLINE_TABTEXT,
								        content:cpGLines.domNode
								      });
								    dlg.show();
						        }								
							});
							w._destroyOnRemove = true;
							return w;
						}
					}, 
			];
			
			// prepare lazy tree grid
			var data = {
				items : DCLines
			};
			var store = new ItemFileReadStore( {
				data : data,
			} );
			var model = new ForestStoreModel( {
				store : store,
				
				getParent : function ( item ) {
										
					var arr = this.store._arrayOfTopLevelItems[0];
					return arr; 
				}
			} );
			
			var hearderTooltips =  [PacbaseEditor.BLOCKBASE_DCLINE_COLUMN_TOOL_TIP_NETWORK_TYPE,
					                PacbaseEditor.BLOCKBASE_DCLINE_COLUMN_TOOL_TIP_NETWORK_NAME,
					                PacbaseEditor.BLOCKBASE_DCLINE_COLUMN_TOOL_TIP_PARENT_SEG,
					                PacbaseEditor.BLOCKBASE_DCLINE_COLUMN_TOOL_TIP_CHILD_SEG,
					                PacbaseEditor.BLOCKBASE_DCLINE_COLUMN_TOOL_TIP_FROM_SEG,
					                PacbaseEditor.BLOCKBASE_DCLINE_COLUMN_TOOL_TIP_METHOD,
					                PacbaseEditor.BLOCKBASE_DCLINE_COLUMN_TOOL_TIP_OCC,
					                PacbaseEditor.BLOCKBASE_DCLINE_COLUMN_TOOL_TIP_AREA_NAME,
					                PacbaseEditor.BLOCKBASE_LINE_COLUMN_TOOL_TIP_COMMENT];
			var grid = new LazyTreeGrid( {
				treeModel : model,
				structure : layoutDCLines,
				height : ( rppCurrentPaneGridHeight - 70 ) + "px",
				autoWidth : true,
				selectable : true,
				canSort : function () {
					return false;
				},
				
				onHeaderCellMouseOver  : function (e) {
					dijit.showTooltip(hearderTooltips[e.cellIndex], e.cellNode);
				},
				onHeaderCellMouseOut : function (e) {
					dijit.hideTooltip(e.cellNode);
				} 				
			
			} );

			cpDetailDCLines.addChild( grid );
			return cpDetailDCLines;
		}
	}
} );