/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2016, 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
	"dojo/string", "dijit/layout/ContentPane", "dijit/tree/ForestStoreModel", "dojo/data/ItemFileReadStore", "dojox/layout/TableContainer",
	"dojox/grid/LazyTreeGrid", "dijit/form/Button", "./gLinesPane", "dijit/Tooltip", 
	"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",		
	"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function (string, ContentPane, ForestStoreModel, ItemFileReadStore, TableContainer, LazyTreeGrid, Button, gLinesPane, Tooltip, PacbaseEditor, typeDict ) {
	
	function getDatabase ( item ) {
		if (item.psbOrPcb != undefined && item.psbOrPcb != "") {
			var type = item.psbOrPcb[0].name;
			return "<span class='display_icon pacDHLine'></span>&nbsp" + type;
		}
		return "<span class='display_icon pacDHLine'></span>&nbsp";
	}	
	
	function getParentSegment ( item , project, streamId) {
		if (item.parent != undefined && item.parent != "") {			
			return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", item.parent[0], false, streamId, project, "");
		}
		return "";
	}	
	
	function getChildSegment ( item , project, streamId) {
		if (item.segment != undefined && item.segment != "") {
			return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", item.segment[0], false, streamId, project, "");		
		}		
		return "";			
	}
	
	function getRelation ( item ) {	
		if (item.relationCode != undefined)		
			return item.relationCode;
		return "";
	}
	
	function getIndicOption ( item ) {	
		if (item.keyIndicatorOrOption != undefined) 
			return item.keyIndicatorOrOption;
		return "";
	}
	
	function getOccurence ( item ) {	
		if (item.estimatedNumberOfLinks != undefined && item.estimatedNumberOfLinks != "0")		
			return item.estimatedNumberOfLinks;
		return "";
	}	
	
	function getComments ( item ) {	
		if (item.commentRelatKeyLength != undefined) 
			return item.commentRelatKeyLength;
		return "";
	}
	
	
	
	return {		
		getDetailDHLines : function ( data, type, project, streamId, compentId, fileItemId ) {
			var DHLines = data.DHLines
			var cpDetailDHLines = new ContentPane( {
				content : "<strong>" + typeDict[ type ] + " " + data.name + " : " + data.label + "</strong><P>"
			    + "<strong>" + PacbaseEditor.DHLINE_TABLE_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
						+ string.substitute( PacbaseEditor.DHLINE_TABLE_SECTION_DESCRIPTION, [
							typeDict[ type ]
						] ) + "</p>",
						style : {
							'overflow' : 'auto',
						},
			} );
			// prepare lazy tree grid
			var layoutDHLines = [
					{
						name : PacbaseEditor.BLOCKBASE_DHLINE_COLUMN_DATABASE,
						width : "120px",
						field : "_item",
						formatter : function ( item ) {
							return getDatabase( item );
						}, 
					}, {
						name : PacbaseEditor.BLOCKBASE_DHLINE_COLUMN_CHILD_SEG,
						width : "70px",
						field : "_item",
						formatter : function ( item , project, streamId) {
							return getChildSegment( item );
						},
					},{
						name : PacbaseEditor.BLOCKBASE_DHLINE_COLUMN_PARENT_SEG,
						width : "70px",
						field : "_item",
						formatter : function ( item , project, streamId) {
							return getParentSegment( item );
						},
					},  {
						name : PacbaseEditor.BLOCKBASE_DHLINE_COLUMN_RELATION,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getRelation( item );
						},
					}, {
						name : PacbaseEditor.BLOCKBASE_DHLINE_COLUMN_INDIC,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getIndicOption( item );
						},
					}, {
						name : PacbaseEditor.BLOCKBASE_DHLINE_COLUMN_OCC,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getOccurence( item );
						},
					}, {
						name : PacbaseEditor.BLOCKBASE_DHLINE_COLUMN_COMMENT,
						width : "300px",
						field : "_item",
						formatter : function ( item ) {
							return getComments( item );
						},
					},  {
						name : PacbaseEditor.BLOCKBASE_LINE_COLUMN_COMMENT,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							if ( item.GGLines == undefined || item.GGLines[0] == undefined )
								return "";							
							var hasComment = false;
							for(var i = 0; i < item.GGLines.length; i++) {							
								  if (item.GGLines[i]["class"] != "PacGenerationElementVirtual" && item.GGLines[i]["class"] != "PacGenerationElementGuide") {
									  hasComment = true;
									  break;
								  }
								}							
							if ( hasComment == false )
								return "";
							var w = new dijit.form.Button({
								label: PacbaseEditor.GLINE_TABTEXT,
								showLabel: false,
								iconClass:'arrrow',
								onClick: function(){
									// -G Lines
									var cpGLines = new ContentPane( {
										title : PacbaseEditor.GLINE_TABTEXT,
										style:"width:"+rppCurrentPaneWidth+"px;height:"+rppCurrentPaneHeight+"px;"
									} );
									cpGLines.addChild( gLinesPane.getGLines ( item, type, false, false, true, false, streamId, project ) );
								    dlg = new dijit.Dialog({
								        title:PacbaseEditor.GLINE_TABTEXT,
								        content:cpGLines.domNode
								      });
								    dlg.show();
						        }								
							});
							w._destroyOnRemove = true;
							return w;
						}
					}, 
			];
			
			// prepare lazy tree grid
			var data = {
				items : DHLines
			};
			var store = new ItemFileReadStore( {
				data : data,
			} );
			var model = new ForestStoreModel( {
				store : store,
				
				getParent : function ( item ) {
										
					var arr = this.store._arrayOfTopLevelItems[0];
					return arr; 
				}
			} );
			
			var hearderTooltips =  [PacbaseEditor.BLOCKBASE_DHLINE_COLUMN_TOOL_TIP_DATABASE,
					                PacbaseEditor.BLOCKBASE_DHLINE_COLUMN_TOOL_TIP_CHILD_SEG,
					                PacbaseEditor.BLOCKBASE_DHLINE_COLUMN_TOOL_TIP_PARENT_SEG,			                
					                PacbaseEditor.BLOCKBASE_DHLINE_COLUMN_TOOL_TIP_RELATION,
					                PacbaseEditor.BLOCKBASE_DHLINE_COLUMN_TOOL_TIP_INDIC,
					                PacbaseEditor.BLOCKBASE_DCLINE_COLUMN_TOOL_TIP_OCC,
					                PacbaseEditor.BLOCKBASE_DHLINE_COLUMN_TOOL_TIP_COMMENT,
					                PacbaseEditor.BLOCKBASE_LINE_COLUMN_TOOL_TIP_COMMENT];	
			
			var grid = new LazyTreeGrid( {
				treeModel : model,
				structure : layoutDHLines,
				height : ( rppCurrentPaneGridHeight - 70 ) + "px",
				autoWidth : true,
				selectable : true,
				canSort : function () {
					return false;
				},
				onHeaderCellMouseOver  : function (e) {
					dijit.showTooltip(hearderTooltips[e.cellIndex], e.cellNode);
				},
				onHeaderCellMouseOut : function (e) {
					dijit.hideTooltip(e.cellNode);
				} 				
			
			} );

			cpDetailDHLines.addChild( grid );
			return cpDetailDHLines;
		}
	}
} );