/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2016, 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dojo/string", "dojox/layout/TableContainer", "dijit/layout/ContentPane", "dijit/tree/ForestStoreModel",
		"dojo/data/ItemFileReadStore", "dojox/grid/LazyTreeGrid", "dijit/form/Button", "./gLinesPane", "dijit/Tooltip", 
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function ( string, TableContainer, ContentPane, ForestStoreModel, ItemFileReadStore, LazyTreeGrid, Button, gLinesPane, Tooltip, 	PacbaseEditor, typeDict ) {	
	
	function getValue ( value ) {		
		return value.toString().replace("_","").replace("None","");
	}
	
	function getSQLRecordType ( item ) {
		if (item.sqlRecordType != undefined && item.sqlRecordType != "") {
			var type = getValue(item.sqlRecordType[0].value);
			return "<span class='display_icon pacDRLine'></span>&nbsp" + type;
		}
		return "";
	}	
	
	function getExternalName ( item ) {	
		if (item.dataBaseObjectExternalName != undefined) 
			return item.dataBaseObjectExternalName;
		return "";
	}	
	
	function getReferencedTable ( item ) {
		if (item.referencedTable != undefined && item.referencedTable != "")		
			return item.referencedTable[0].name;
		return "";
	}
	
	function getTableOrViewCode ( item , project, streamId) {
		if (item.segment != undefined && item.segment != "") {
			return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", item.segment[0], false, streamId, project, "");		
		}		
		return "";
	}
	
	function getKeyType ( item ) {
		if (item.keyType != undefined) 
			return getValue(item.keyType[0].value);
		return "";
	}
	
	function getCommandGenerationType ( item ) {
		if (item.commandGenerationType != undefined) 
			return getValue(item.commandGenerationType[0].value);
		return "";
	}
	
	function getOrder ( item ) {		
		if (item.order != undefined) {
			return getValue(item.order[0].value);
		}
		return "";
	}
	
	function getDataElement ( item , project, streamId) {
		if (item.dataElement != undefined && item.dataElement != "") {
			return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", item.dataElement[0], false, streamId, project, "");		
		}		
		return "";			
	}
	
	function getComment ( item ) {
		return "";
	}
	
	

	return {		
		getDetailDRLines : function ( data, type, project, streamId, compentId, fileItemId ) {
			var DRLines = data.DRLines
			var cpDetailDRLines = new ContentPane( {
				content : "<strong>" + typeDict[ type ] + " " + data.name + " : " + data.label + "</strong><P>"
			    + "<strong>" + PacbaseEditor.DRLINE_TABLE_SECTION_HEADER + + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
						+ string.substitute( PacbaseEditor.DRLINE_TABLE_SECTION_DESCRIPTION, [
							typeDict[ type ]
						] ) + "</p>",
						style : {
							'overflow' : 'auto',
						},
			} );
			// prepare lazy tree grid
			var layoutDRLines = [
					{
						name : PacbaseEditor.BLOCKBASE_DRLINE_COLUMN_RECORD_SQL,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getSQLRecordType( item );
						},
					}, {
						name : PacbaseEditor.BLOCKBASE_DRLINE_COLUMN_EXTERNAL_NAME,
						width : "120px",
						field : "_item",
						formatter : function ( item ) {
							return getExternalName( item );
						},
					}, {
						name : PacbaseEditor.BLOCKBASE_DRLINE_COLUMN_REF_TABLE,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getReferencedTable( item );
						},
					}, {
						name : PacbaseEditor.BLOCKBASE_DRLINE_COLUMN_TABLE_VIEW,
						width : "70px",
						field : "_item",
						formatter : function ( item , project, streamId) {
							return getTableOrViewCode( item );
						},
					}, {
						name : PacbaseEditor.BLOCKBASE_DRLINE_COLUMN_KEY,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getKeyType( item );
						},
					}, {
						name : PacbaseEditor.BLOCKBASE_DRLINE_COLUMN_GEN_MVT,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getCommandGenerationType( item );
						},
					}, {
						name : PacbaseEditor.BLOCKBASE_DRLINE_COLUMN_ORDER,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getOrder( item );
						},
					}, {
						name : PacbaseEditor.BLOCKBASE_DRLINE_COLUMN_DATA_ELEMENT,
						width : "70px",
						field : "_item",
						formatter : function ( item , project, streamId) {
							return getDataElement( item );						
						},
					}, {
						name : PacbaseEditor.BLOCKBASE_LINE_COLUMN_COMMENT,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							if ( item.GGLines == undefined || item.GGLines[0] == undefined )
								return "";							
							var hasComment = false;
							for(var i = 0; i < item.GGLines.length; i++) {							
								  if (item.GGLines[i]["class"] != "PacGenerationElementVirtual" && item.GGLines[i]["class"] != "PacGenerationElementGuide") {
									  hasComment = true;
									  break;
								  }
								}							
							if ( hasComment == false )
								return "";
							var w = new dijit.form.Button({
								label: PacbaseEditor.GLINE_TABTEXT,
								showLabel: false,
								iconClass:'arrrow',
								onClick: function(){
									// -G Lines
									var cpGLines = new ContentPane( {
										title : PacbaseEditor.GLINE_TABTEXT,
										style:"width:"+rppCurrentPaneWidth+"px;height:"+rppCurrentPaneHeight+"px;"
									} );
									cpGLines.addChild( gLinesPane.getGLines ( item, type, false, false, true, false, streamId, project ) );
								    dlg = new dijit.Dialog({
								        title:PacbaseEditor.GLINE_TABTEXT,
								        content:cpGLines.domNode
								      });
								    dlg.show();
						        }								
							});
							w._destroyOnRemove = true;
							return w;
						}
					}, 
			];
			
			// prepare lazy tree grid
			var data = {
				items : DRLines
			};
			var store = new ItemFileReadStore( {
				data : data,
			} );
			var model = new ForestStoreModel( {
				store : store,
				mayHaveChildren : function ( item ) {
					if ( item.KLines != undefined && item.KLines.length > 0 ) {						
						return true;
					}					
					return false;
				},
				getChildren : function ( item, onComplete, onError ) {
					if ( item.KLines != undefined && item.KLines.length > 0) {
						onComplete(item.KLines);
					}					
					 
				},				
				getParent : function ( item ) {
										
					var arr = this.store._arrayOfTopLevelItems[0];
					return arr; 
				}
			} );
			
			var hearderTooltips =  [PacbaseEditor.BLOCKBASE_DRLINE_COLUMN_TOOL_TIP_RECORD_SQL,	
				      				PacbaseEditor.BLOCKBASE_DRLINE_COLUMN_TOOL_TIP_EXTERNAL_NAME,
				      				PacbaseEditor.BLOCKBASE_DRLINE_COLUMN_TOOL_TIP_REF_TABLE,
				      				PacbaseEditor.BLOCKBASE_DRLINE_COLUMN_TOOL_TIP_TABLE_VIEW,
				      				PacbaseEditor.BLOCKBASE_DRLINE_COLUMN_TOOL_TIP_KEY,
				      				PacbaseEditor.BLOCKBASE_DRLINE_COLUMN_TOOL_TIP_GEN_MVT,
				      				PacbaseEditor.BLOCKBASE_DRLINE_COLUMN_TOOL_TIP_ORDER,
				      				PacbaseEditor.BLOCKBASE_DRLINE_COLUMN_TOOL_TIP_DATA_ELEMENT,
				      				PacbaseEditor.BLOCKBASE_LINE_COLUMN_TOOL_TIP_COMMENT];	  
			
			var grid = new LazyTreeGrid( {
				treeModel : model,
				structure : layoutDRLines,
				height : ( rppCurrentPaneGridHeight - 70 ) + "px",
				autoWidth : true,
				selectable : true,
				canSort : function () {
					return false;
				},
				onHeaderCellMouseOver  : function (e) {
					dijit.showTooltip(hearderTooltips[e.cellIndex], e.cellNode);
				},
				onHeaderCellMouseOut : function (e) {
					dijit.hideTooltip(e.cellNode);
				} 				
			
			} );

			cpDetailDRLines.addChild( grid );
			return cpDetailDRLines;
		}
	}
} );