/*
 * IBM Confidential
 * Licensed Materials - Property of IBM
 *
 * IBM Rational Developer Traveler
 *
 * (C) Copyright IBM Corporation 2013, 2014.
 *
 * The source code for this program is not published or otherwise divested of its trade secrets, 
 * irrespective of what has been deposited with the U.S. Copyright Office.
 */
define(['./cobolScanner', './styleParser'], function(mCobolScanner, mStyleParser) {
	
	var cobolStyles = {
		singleCommentStyle: {styleClass: "comment"},
		multiCommentStyle: {styleClass: "comment"},
		tasktagStyle: {styleClass: "token_task_tag"},
		stringStyle: {styleClass: "string"},
		numberStyle: {styleClass: "number"},
		keywordStyle: {styleClass: "keyword"},
		spaceStyle: {styleClass: "token_space"},
		tabStyle: {styleClass: "token_tab"},
		caretLineStyle: {styleClass: "annotation.currentLine"},
		linkStyle: {styleClass: "token_link"},
		generatedStyle: {styleClass: "generated"},
		macroStyle: {styleClass: "macro"},
		mpStyle: {styleClass: "mp"},
		mpdetailStyle: {styleClass: "detailmp"}
	};
	
	
	function CobolStyler(filename) {
		this.whitespacesVisible = false;
		this.brackets = "()";
		
		this.scanner = new mCobolScanner.CobolScanner(this.whitespacesVisible);
		this.firstScanner = new mCobolScanner.FirstScanner();
		this.commentScanner = new mCobolScanner.CommentScanner(this.whitespacesVisible);
		this.whitespaceScanner = new mCobolScanner.WhitespaceScanner();
		
		this._qualifier = "";
		this._computeQualifier(filename);
	}
	
	CobolStyler.prototype = {
		
		getStyleForToken: function(token) {
			switch(token) {
				case this.scanner.KEYWORD: return cobolStyles.keywordStyle;
				case this.scanner.NUMBER: return cobolStyles.numberStyle;
				case this.scanner.STRING: return cobolStyles.stringStyle;
				case this.scanner.MULTILINE_STRING: return cobolStyles.stringStyle;
				case this.scanner.SINGLELINE_COMMENT: return cobolStyles.singleCommentStyle;
				case this.scanner.MULTILINE_COMMENT: return cobolStyles.multiCommentStyle;
				case this.scanner.WHITE_TAB: return cobolStyles.tabStyle;
				case this.scanner.WHITE_SPACE: return cobolStyles.spaceStyle;
				case this.scanner.TASK_TAG: return cobolStyles.tasktagStyle;
				case this.scanner.GENERATED: return cobolStyles.generatedStyle;
				case this.scanner.MACRO: return cobolStyles.macroStyle;
				case this.scanner.MP: return cobolStyles.mpStyle;
				case this.scanner.DETAILMP: return cobolStyles.mpdetailStyle;
				default: return null;
			}
		},
		
		setWhitespacesVisible: function(visible) {
			this.whitespacesVisible = visible;
			this.scanner.whitespacesVisible = visible;
			this.commentScanner.whitespacesVisible = visible;
		},
			
		_computeQualifier: function(filename){
			var match = /^.*(MVSFiles\/)([\w\.]+)(\/.*)$/.exec(filename);  // Find the qualifier
			if(match){
				this._qualifier = match[2];
			}
		},
	
		createLink: function(baseModel, tokenText, lineText, offset, s) {
			return null;
		}
		
	};
	
	return {CobolStyler:CobolStyler};
});