/****************************
 * IBM Confidential
 * Licensed Materials - Property of IBM
 *
 * IBM Rational Developer Traveler
 *
 * (C) Copyright IBM Corporation 2013, 2014.
 *
 * The source code for this program is not published or otherwise divested of its trade secrets,
 * irrespective of what has been deposited with the U.S. Copyright Office.
 */
/*******************************************************************************
 * @license
 * Copyright (c) 2009, 2011 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License v1.0
 * (http://www.eclipse.org/legal/epl-v10.html), and the Eclipse Distribution
 * License v1.0 (http://www.eclipse.org/org/documents/edl-v10.html).
 *
 * Contributors: IBM Corporation - initial API and implementation
 *******************************************************************************/
/*global define window document navigator*/

define([/*'util/util'*/], function(/*mUtil*/) {

	/**
	 * Store JCL in the RDT Property Store by Job ID
	 * @param jobID
	 * @param jclInfo
	 * @returns
	 */
	function updateJCLJobMap(jobMap, jobID, jclInfo){

		// If the job map doesn't exist, create it
		if(jobMap === null || jobMap === undefined){
			jobMap = {
					jobIDs: [],
					jobInfo: {}
			};
		}else{
			var existingInfo = jobMap.jobInfo[jobID];

			if(existingInfo !== undefined){
				// re-used Job ID - remove the old info
				jobMap.jobIDs.splice(existingInfo.index, 1);
				delete jobMap.jobInfo[jobID];
			}else{
				// Remove the least recent value if the map is too big (100 max)
				if(jobMap.jobIDs.length === 100){
					var removedID = jobMap.jobIDs.splice(0, 1);
					delete jobMap.jobInfo[removedID[0]];
				}
			}
		}

		// Add the new job
		jobMap.jobIDs.push(jobID);
		jobMap.jobInfo[jobID] = {index: jobMap.jobIDs.length - 1, jclInfo: jclInfo};

		return jobMap;
	}

	/**
	 * This class contains static utility methods. It is not intended to be instantiated.
	 * @class This class contains static utility methods.
	 * @name orion.util
	 */

	function removeSequenceNumbers(sequenceNumbers, lineLength, contents, delimiter){
		var result = {
				mainContent: contents,
				leftSequenceNumbers: [],
				rightSequenceNumbers: []
		};
		if(sequenceNumbers && lineLength){
			var contentRange = [0, lineLength];
			
			for(var i = 0; i < sequenceNumbers.length; i++){
				if(sequenceNumbers[i].start === 0){
					contentRange[0] = sequenceNumbers[i].end;
				}else if((sequenceNumbers[i].end) === lineLength){
					contentRange[1] = sequenceNumbers[i].start;
				}
			}
			var lines = this.splitFile(contents);
			var newContent = "";
			for(var i = 0; i < lines.length; i++){
				result.leftSequenceNumbers.push(lines[i].substring(0, contentRange[0]));
				result.rightSequenceNumbers.push(lines[i].substring(contentRange[1], lineLength));
				newContent += lines[i].substring(contentRange[0], contentRange[1]);

				if(i + 1 < lines.length){
					newContent += delimiter;
				}
			}
			result.mainContent = newContent;
		}

		return result;
	}

	/**
	 * Visit each line in 'contents' and attempt to fit the line within 'lineLength'.  Including:
	 *  - Pad out lines with blanks if necessary
	 *  - Remove trailing blanks if possible
	 *  
	 * Returns a string
	 */
	function adjustLineLengths(lineLength, contents, delimiter, padWithBlanks){
		var result = "";
		if(lineLength){
			// Only adjust line lengths if we have a length
			var lines = this.splitFile(contents);
			var newContent = "";
			for(var i = 0; i < lines.length; i++){
				var line = lines[i];
	
				newContent += this.adjustLineLength(lineLength, line, padWithBlanks);
	
				if(i + 1 < lines.length){
					newContent += delimiter;
				}
			}
			result = newContent;
		}else{
			result = contents;
		}
		return result;
	}

	/**
	 * See adjustLineLengths
	 */
	function adjustLineLength(lineLength, line, padWithBlanks){
		if(padWithBlanks){
			if(line.length < lineLength){
				var numBlanks = lineLength - line.length;
				line += (new Array(numBlanks + 1)).join(" ");
			}
		}
		if(line.length > lineLength){
			var canTrim = true;
			for(var j = line.length - 1; j >= lineLength; j--){
				if(line.charAt(j) != " "){
					canTrim = false;
					break;
				}
			}
			if(canTrim){
				line = line.substring(0, lineLength);
			}
		}
		return line;
	}

	function splitMVSResourceName(resourcePath){
		
		var pathSegments = resourcePath.getSegments();
		
		var result = {
			member :  pathSegments[pathSegments.length - 1],
			qualifier : resourcePath.getSubPath(pathSegments.length - 1).toString()
		};
		
		return result;
	}

	/**
	 * Copied from Orion searchUtils.js
	 *
	 * Split file contents into lines. It also handles the mixed line endings with "\n", "\r" and "\r\n".
	 *
	 * @param {String} text The file contents.
	 * @returns {Array} Split file lines.
	 * @name splitFile
	 * @function
	 */
	function splitFile(text) {
		var cr = 0, lf = 0, index = 0, start = 0;
		var splitLines = [];
		if(text.length > 0){
			while (true) {
				if (cr !== -1 && cr <= index) {
					cr = text.indexOf("\r", index);  //$NON-NLS-0$
				}
				if (lf !== -1 && lf <= index) {
					lf = text.indexOf("\n", index);  //$NON-NLS-0$
				}
				if (lf === -1 && cr === -1) {
					splitLines.push(text.substring(start));
					break;
				}
				var offset = 1;
				if (cr !== -1 && lf !== -1) {
					if (cr + 1 === lf) {
						offset = 2;
						index = lf + 1;
					} else {
						index = (cr < lf ? cr : lf) + 1;
					}
				} else if (cr !== -1) {
					index = cr + 1;
				} else {
					index = lf + 1;
				}
				splitLines.push(text.substring(start, index - offset));
				start = index;
			}
		}
		return splitLines;
	}
	
	function splitInTwoFiles(text) {
		var indexEndOfCobol = text.indexOf("#EndOfCobol#");
		var splitLines = [];
		if (indexEndOfCobol == -1) {
			splitLines.push(text);
		} else {		
			splitLines.push(text.substring(0, indexEndOfCobol));
			splitLines.push(text.substring(indexEndOfCobol + 12));
		}
		return splitLines;
	}

	function clone(obj) {
		if (!obj) { return obj; }
		var newObj = {};
		for (var p in obj) {
			if (obj.hasOwnProperty(p)) {
				var value = obj[p];
				newObj[p] = value;
			}
		}
		return newObj;
	}

	function isFixedRecord(recordFormat){
		return recordFormat === "F" ? true : false;
	}

	function handleLineWrap(event, editor, inputManager){
		return doHandleLineWrap(event, editor.getSelection(), editor.getModel(), inputManager.getLineDelimiter(), inputManager.getFileMetadata().textLength, editor.getTextView().getOptions("tabSize"), isFixedRecord(inputManager.getFileMetadata().RecordFormat));
	}

	function _getPrefix(selection, model){
		//(see logic in actions.js::autoIndent)
		var prefix = "";
		if (selection.start === selection.end) {
			var lineIndex = model.getLineAtOffset(selection.start);
			var lineText = model.getLine(lineIndex, true);
			var lineStart = model.getLineStart(lineIndex);
			var index = 0, end = selection.start - lineStart, c;
			while (index < end && ((c = lineText.charCodeAt(index)) === 32 || c === 9)) { index++; }
			if (index > 0 && index < end) {
				//TODO still wrong when typing inside folding
				prefix = lineText.substring(0, index);
			}
		}
		return prefix;
	}

	/**
	 * Calculate length of text (expands tab character into spaces it occupies)
	 */
	function _getTextLength(text, tabSize){
		var column=0,   // running count of characters occupied by 'text' (after expanding tabs)
			tabarr=text.split('\t');
		for (var i=0; i<tabarr.length; i++) {
			column += tabarr[i].length;
			if (i+1 < tabarr.length) {
				// advance 'column' to next tab stop. ~~ is a quick way of doing a floor()
				column = ~~((column + tabSize) / tabSize) * tabSize;
			}
		}
		return column;
	}

	/**
	 * Determine if new text should be allowed to be inserted
	 *
	 */
	function _allowTextEntry(event, lineStartOffset, lineText, lineLength, tabSize, delimiter){
		if (event.start === event.end &&		// A range has not been selected - ranges indicate a paste event
			event.text.length === 1 &&			// A keyboard entry or a paste of a single character
			event.text !== delimiter &&			// Not a newline character
			(event.start !== lineStartOffset + lineLength ||	// Not a character in the last column
			 (event.text === " " || event.text === "\t"))) {	// Do not allow a tab or space in the last column
			// If the added character causes the line to exceed the max length, don't allow it.
			// NOTE: if the line contains tab character(s), adding any character can expand the tab spacing by 0 or more characters
			var trimLineText = lineText.replace(/ +$/g,''),	// lineText with trailing spaces removed
				newLineText = trimLineText.substring(0, event.start - lineStartOffset) +
			                  event.text +
			                  trimLineText.substring(event.start - lineStartOffset);
			if (_getTextLength(newLineText, tabSize) > lineLength)
				return false;
		}
		return true;
	}

	function _isEmptyEvent(event){
		if(event.text === ""
			&& event.start === 0 && event.end === 0){
			return true;
		}
		return false;
	}

	function _isBackspaceOverNewline(event, previousLineEndOffset, delimiter){
		if(event.end === previousLineEndOffset
				&& event.text.length === 0
				&& (event.end - event.start) === delimiter.length){
			return true;
		}
		return false;
	}

	function doHandleLineWrap(event, selection, model, delimiter, maxLineLength, tabSize, fixedRecordFormat){

		var textUpdate = null;
		var needsSetText = false;
		var delimiter = model.getLineDelimiter();

		var firstLineIndex = model.getLineAtOffset(event.start);
		var firstLineStart = model.getLineStart(firstLineIndex);
		var firstLineEnd = model.getLineEnd(firstLineIndex, true);
		var firstLineText = model.getLine(firstLineIndex);

		var lastLineIndex;
		var lastLineText;
		var lastLineEnd;

		if(_isEmptyEvent(event)
			&& model.getText() === ""){
			// Empty content on an empty file - ignore
			return;
		}

		if(!_allowTextEntry(event, firstLineStart, model.getLine(firstLineIndex, false), maxLineLength, tabSize, delimiter)){
			event.text = null;
			return null;
		}

		if(_isBackspaceOverNewline(event, firstLineEnd, delimiter)){
			needsSetText = true;
			var trailingBlanks = /(\s+)$/.exec(firstLineText);
			if(trailingBlanks !== null){
				// Remove trailing whitespace from end of the previous line
				event.start -= trailingBlanks[1].length;
			}
		}

		if(event.end >= firstLineEnd){
			// The update crosses multiple lines - find the last line
			lastLineIndex = model.getLineAtOffset(event.end);
			lastLineText = model.getLine(lastLineIndex, false);
			lastLineEnd = model.getLineEnd(lastLineIndex, false);
		}else{
			lastLineIndex = firstLineIndex;
			lastLineText =  model.getLine(lastLineIndex, false);
			lastLineEnd =  model.getLineEnd(lastLineIndex, false);
		}

		// We have our damaged region - insert the new text into this region
		var trailingText = lastLineText.substring(lastLineText.length - (lastLineEnd - event.end));
		var lineText = firstLineText.substring(0, (event.start - firstLineStart)) + event.text + trailingText;

		// Separate damaged region into lines on existing delimiters
		var lines = splitFile(lineText);
		var result = "";
		var prefix = null;
		var numCharsAdded = 0;  // chars added to the inserted text

		for(var i = 0; i < lines.length; i++) {
			var line = lines[i];
			var lineLength = line.length;
			if((fixedRecordFormat && (lineLength !== maxLineLength)) ||	// Entry criteria for fixed records - visit any line that isn't padded out
			   (!fixedRecordFormat && (lineLength > maxLineLength))) {	// Entry criteria for variable records - visit any line that is too long
				needsSetText = true;
				var eolIndex = maxLineLength;
				if(lineLength > maxLineLength){

					// This line is too long - split it up into multiple lines
					line = line.replace(new RegExp("\\s{0," + (lineLength - maxLineLength) + "}$"), ""); // trim right side blanks that go past the end of the line - we don't want trailing blanks to flow to the next line

					lineLength = line.length; // recalculate line length after right trim

					if(i + 1 < lines.length){
						numCharsAdded -= (lines[i].length - lineLength);
					}

					if(prefix === null){
						prefix = _getPrefix(selection, model);
					}

					while(eolIndex < lineLength){
						var line_trailing = line.substring(eolIndex);
						line = line.substring(0, eolIndex) + delimiter + prefix + line_trailing;
						eolIndex += maxLineLength + delimiter.length;
						if (i+1 < lines.length ||	// Not the last line in 'lines'
							line_trailing.length > trailingText.replace(/\s+$/g,'').length){	// Line break was added to event.text
							numCharsAdded += delimiter.length + prefix.length;
						}
					}

					lineLength = line.length; // recalculate line length after additions
				}

				if((lineLength < eolIndex) && fixedRecordFormat){
					// This line is too short - pad it out if fixed record format
					var numBlanks = eolIndex - lineLength;
					line += (new Array(numBlanks + 1)).join(" ");
					if (i+1 < lines.length){
						numCharsAdded += numBlanks;
					}
				}
			}

			if(i + 1 < lines.length){
				line += delimiter;
			}

			result += line;
		}

		if(needsSetText){
			// invoking setText is expensive, so only do it if we have to
			// We are taking over the text update so that we can replace the text of the lines in the damaged region
			var theCursor;
			if(event.text.length > 0){
				// Text insert (typing, paste, etc)
				//  Each tab character in the inserted text count as one character position
				theCursor = event.start + event.text.length + numCharsAdded;
			}else{
				// Text removal (backspace, delete, cut, etc)
				theCursor = event.start;
			}
			textUpdate = {
					text : result,
					start: firstLineStart,
					end:  lastLineEnd,
					cursor: theCursor
			};
			event.text = null;
		}
		return textUpdate;
	}

	//return module exports
	return {
		updateJCLJobMap: updateJCLJobMap,
		adjustLineLengths: adjustLineLengths,
		adjustLineLength: adjustLineLength,
		removeSequenceNumbers: removeSequenceNumbers,
		splitMVSResourceName: splitMVSResourceName,
		splitFile: splitFile,
		splitInTwoFiles: splitInTwoFiles,
		clone: clone,
		isFixedRecord: isFixedRecord,
		handleLineWrap: handleLineWrap,
		doHandleLineWrap: doHandleLineWrap
	};
});
