/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dojo/string", "dojox/layout/TableContainer", "dijit/layout/ContentPane", "dijit/form/TextBox", "dijit/tree/ForestStoreModel",
		"dojo/data/ItemFileReadStore", "dojo/data/ItemFileWriteStore", "dojox/grid/DataGrid", "dijit/form/Button", "dijit/Dialog", "dijit/Tooltip", "./gLinesPane", 
		"dijit/TooltipDialog", "dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function ( string, TableContainer, ContentPane, TextBox, ForestStoreModel, ItemFileReadStore, ItemFileWriteStore, DataGrid, Button, Dialog, Tooltip, gLinesPane, TooltipDialog,
				PacbaseEditor, typeDict ) {	
	
	return {
		
		getCPLines : function ( data, type, streamId, project ) {	
			var CPLines = data.CPLines			
			var cpCPLines = new ContentPane( {
				content : "<strong>" + typeDict[ type ] + " " + data.name + " : " + data.label + "</strong><P>"
			    + "<strong>" + PacbaseEditor.CPLINE_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
						+ string.substitute( PacbaseEditor.CPLINE_SECTION_DESCRIPTION, [
							typeDict[ type ]
						] ) + "</p>",
						style : {
							'overflow' : 'auto',
						},
			} );
			
			// prepare layoutCSLines
			var layoutCPLines = [
 					{
						name : PacbaseEditor.CPLINE_COLUMN_MACRO,
						width : "400px",
						field : "_item",
						formatter : function ( item ) {
							return getMacroCode(item, project, streamId );
						}
					}, {
						name : PacbaseEditor.CPLINE_COLUMN_LINE_NUMBER,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {
							return getLineNumber(item);
						}
					},
						{
						name : PacbaseEditor.CPLINE_COLUMN_PARAMETERS,
						width : "600px",
						field : "_item",
						formatter : function ( item ) {
							return getParameters(item);
						}
					},
					{
						name : PacbaseEditor.CPLINE_COLUMN_COMMENT,
						width : "350px",
						field : "_item",
						formatter : function ( item ) {
							return getComment(item);
						}
					}
			];
			
												
			// prepare lazy tree grid
			var data = {
				items : CPLines
			};
			var store = new ItemFileReadStore( {
				data : data
			} );
			
			var hearderTooltips =  [PacbaseEditor.CPLINE_COLUMN_TOOL_TIP_MACRO,
			                        PacbaseEditor.CPLINE_LINE_NUMBER_TOOL_TIP_MACRO,
			                        PacbaseEditor.CPLINE_COLUMN_PARAMETERS,
			                        PacbaseEditor.MACRO_COMMENTS_LINES,];
			
			// create a new grid
			var CPLinesGrid = new DataGrid( {
				store : store,
				structure : layoutCPLines,
				height : ( rppCurrentPaneGridHeight - 70 ) + "px",
				autoWidth : true,
				onStyleRow: function (row) {
					row.customClasses = "rppGridFixedDefaultFont";
				},
				canSort : function () {
					return false;
				},
				
				onHeaderCellMouseOver  : function (e) {
					dijit.showTooltip(hearderTooltips[e.cellIndex], e.cellNode);
				},
				onHeaderCellMouseOut : function (e) {
					dijit.hideTooltip(e.cellNode);
				} 	
				
			} );
			
			//* Tooltip et boîte dialogue pour ligne parameters
			var cpLineTooltipDialog = null;
			
			CPLinesGrid.on('a.edit-current-parameters:click', function (event, project, streamId) {
				if ( cpLineTooltipDialog ) {
					dijit.popup.close(cpLineTooltipDialog);
					cpLineTooltipDialog.destroy();
					cpLineTooltipDialog = null;
				}
				displayMacroCall ( event.grid.getItem(event.rowIndex) , project, streamId);
			});
			
			CPLinesGrid.on('a.edit-current-parameters:mouseover', function (event, project, streamId) {
				var cpPane = getMacroCallPane(event.grid.getItem(event.rowIndex), project, streamId);
				cpPane.startup();
				
				if ( cpLineTooltipDialog ) {    				
    				cpLineTooltipDialog.destroy();
    				cpLineTooltipDialog = null;
    			}
			    cpLineTooltipDialog = new TooltipDialog ({
			    	style: "width:800px;height:100px;",
			    	onMouseLeave: function(){
			        	dijit.popup.close(cpLineTooltipDialog);
			        },
			    });
			    
			    cpLineTooltipDialog.addChild(cpPane);
			    cpLineTooltipDialog.startup();
		    	dijit.popup.open({
		            popup: cpLineTooltipDialog,
		            around: event.cellNode
		        });
		    	require(["dojo/on"], function(on){
		    		on(event.cellNode, 'mouseleave', function(){
		    			if ( cpLineTooltipDialog ) {
		    				dijit.popup.close(cpLineTooltipDialog);
		    				cpLineTooltipDialog.destroy();
		    				cpLineTooltipDialog = null;
		    			}
				    });
		    	});
			});
			
			cpCPLines.addChild(CPLinesGrid);
			return cpCPLines;
		
		}
	}
	
	function getValue ( value ) {		
		return value.toString().replace("_","").replace("None","");
	}
		
	function getMacroCode( item , project, streamId ) {
		if(item.macro != undefined && item.macro != "") {
			return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", item.macro[0], true, streamId, project, "");
		}
		return item.macro;
	}
	
	function getLineNumber( item ) {	
		if(item.lineNumber != undefined && item.lineNumber != "0") 
			return item.lineNumber;
		return "";
	}
	
	function getParameters( item ) {
		if(item.parameters != undefined && item.parameters != "") {
			param = "(";
			i = 0;
			while (item.parameters[i] != undefined && item.parameters[i] != "") {
				var newValue = '';
				newValue += item.parameters[i].value[0].replace(/ /g,"&nbsp;");			
				if ("" != (getValue(newValue))) {
					if ("(" != param) {
						param = param + ", ";
					} 
					param = param + getValue(item.parameters[i].id) + ": " + getValue(newValue);
					
					}
				i ++;
				} 
			param = param + ")";
			if ( "()" == param ) {
				return "";
			} else {
				return "<a class='edit-current-parameters'  href='#' onclick='return false;'>"+param+"</a>";
			}
		}
		return "";
	}
	
	function getComment( item ) {	
		if (item.comment != undefined) { 
			var newValue = '';
			newValue += item.comment[0].replace(/ /g,"&nbsp;");			
			return newValue;
		}
		return "";
	}
	
	function displayMacroCall ( dataItem , project, streamId ) {
	    dlg = new dijit.Dialog({
	        title:PacbaseEditor.CPLINE_MACRO_CALL_DETAIL_VALUE,
	        content:getMacroCallPane(dataItem , project, streamId).domNode
	      });
	    dlg.show();
	}
	
	function getMacroCallPane ( dataItem , project, streamId) {
		var mcLinesPane = new ContentPane( {
			title : PacbaseEditor.CPLINE_TABTEXT,
			style:"overflow:'auto'",
			tabindex : "0",
		} );
		// set up data store
		var mcLinesStore = new ItemFileWriteStore( {
			data : {
				items : dataItem.parameters
			}
		} );
		// set up layout
		var mcLinesLayout = [
				{
					'name' : PacbaseEditor.CPLINE_MACRO_CALL_PARAMETER,
					'field' : 'id',
					'width' : '65px'
				}, {
					'name' : PacbaseEditor.CPLINE_MACRO_CALL_VALUE,
					'field' : 'value',
					'width' : '300px',
					formatter : function ( item ) {
						return item.replace(/ /g,"&nbsp;");
					}
				}, {
					'name' : PacbaseEditor.CPLINE_MACRO_CALL_COMMENT,
					'field' : 'comment',
					'width' : '300px',
				}
		];

		// create a new grid
		var mcLinesGrid = new DataGrid( {
			store : mcLinesStore,
			structure : mcLinesLayout,
			selectable : true,
			autoHeight : true,
			autoWidth : true,
			onStyleRow: function (row) {
				row.customClasses = "rppGridFixedDefaultFont";
			},
			canSort : function () {
				return false;
			},				
		} );
		
		mcLinesPane.addChild( mcLinesGrid );
		return mcLinesPane;
			
	};

	} );