/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2016, 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dojo/request/xhr", "dojo/request/notify", "dojo/store/Memory", "dojo/store/Observable", "dijit/tree/ObjectStoreModel", "dijit/Tree", "dojo/dom-attr",
		"com.ibm.teampdp.query.engine.web/ui/internal/page/typeEnable", "com.ibm.teampdp.query.engine.web/ui/internal/page/typeRelation",
		"com.ibm.teampdp.query.engine.web/ui/internal/page/maxNumber",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/searchPage"
], function ( xhr, notify, Memory, Observable, ObjectStoreModel, Tree, domAttr, typeEnable, typeRelation, maxNumber, typeDict, searchPage ) {
	function sendReferenceForm ( direction, name, type, project, context, streamId, parentId ) {
		return xhr( net.jazz.ajax.contextRoot() + "/service/com.ibm.teampdp.query.engine.common.IRppQueryEngineService", {
			method : "POST",
			data : {
				name : name,
				type : type,
				project : project,
				context : context,
				direction : direction,
				streamId : streamId,
				search : "searchReferences",
				maxNumber : maxNumber.MAX_NUMBER_REFERENCE,
			},
			handleAs : "json"
		} ).then( function ( references ) {
			// check whether server cut the response.
			var totalNumbers = references[ 0 ].number;
			if ( totalNumbers > maxNumber.MAX_NUMBER_REFERENCE ) {
				require( [
					"dojo/string"
				], function ( string ) {
					displayRppWarningDialog( string.substitute( searchPage.ALERT_EXCEED, [
							totalNumbers, maxNumber.MAX_NUMBER_REFERENCE
					] ) );
				} );
			}
			if ( references[ 0 ].result != null ) {
				references[ 0 ].result.forEach( function ( item ) {
					// update the reference for putting it into reference tree
					item[ "parent" ] = parentId;
					// for preventing the same designId
					item[ "id" ] = item.name + "/" + item.type + "/" + item.project + "/" + parentId;
				} );
				return references[ 0 ].result;
			}
		}, function ( err ) {
			displayRppError( err );
		} );
	}
	return {
		createTree : function ( direction, name, type, project, context, fileItemId, compentId, streamId ) {
			// get the array
			var typesEnable = typeEnable;
			var typesRelation = typeRelation;
			var rootId = name + "/" + type + "/" + project + "/" + direction;
			var referenceRoot = {
				id : rootId,
				name : name,
				type : type,
				project : project,
				fileItemId : fileItemId,
				compentId : compentId,
				root : true
			};
			// send first request for reference
			var promise = sendReferenceForm( direction, name, type, project, context, streamId, rootId );
			var promiseReferenceTree = promise.then( function ( references ) {
				references.push( referenceRoot );
				// construct the reference tree
				var referenceStore = new Memory( {
					data : references,
					getChildren : function ( object ) {
						// Add a getChildren() method to store for the data model where
						// children objects point to their parent (aka relational model)
						return this.query( {
							parent : object.id
						} );
					}
				} );
				// Wrap the store in Observable so that updates to the store are reflected to the Tree
				referenceStore = new Observable( referenceStore );
				// connect data store with model
				var referenceModel = new ObjectStoreModel( {
					store : referenceStore,
					query : {
						root : true
					},
				} );
				var referenceTree = new Tree( {
					model : referenceModel,
					openOnClick : false,
					onDblClick : function ( item ) {
						if ( item.project == ""  )
							return;
						openArtifactFile( streamId, item.compentId, item.fileItemId, item.project, item.project, item.type, item.metaType, item.name );
					},
					getLabel : function ( item ) {
						var ligneLabel;
						if ( item.root ) {
							ligneLabel = typeDict[ item.type ] + ": " + item.name;
						} else {
							ligneLabel = item.name + (  item.metaType && item.metaType.length > 0  ?  " (" + item.metaType + ") " : "" ) + " [" + item.project + "] " + item.label;
							//if ( typesRelation.indexOf( item.type ) > -1 ) {
							if (direction == 'super' )
								ligneLabel += "<span style='font-family:monospace'>&nbsp;--&gt;&nbsp;</span>" + item.relations;
							else
								ligneLabel += "<span style='font-family:monospace'>&nbsp;&lt--&nbsp;</span>" + item.relations;
							//}
						}
						ligneLabel += "<button onclick='createReferencePane(\"" + item.name + "\",\"" + item.type + "\",\"" + item.project + "\",\"" + item.context
								+ "\",\"super\",\"" + item.fileItemId + "\",\"" + item.compentId + "\",\"" + streamId
								+ "\")' hidden='hidden' id='super_" + item.id + "'>" + searchPage[ "super" ] + "</button>";
						ligneLabel += "<button onclick='createReferencePane(\"" + item.name + "\",\"" + item.type + "\",\"" + item.project + "\",\"" + item.context
								+ "\",\"sub\",\"" + item.fileItemId + "\",\"" + item.compentId + "\",\"" + streamId
								+ "\")' hidden='hidden' id='sub_" + item.id + "'>" + searchPage[ "sub" ] + "</button>";
						if ( item.type == "cbl" || item.type == "cpy" || item.type == "ddl" || item.type == "txt"
							|| item.type == "bms" || item.type == "map" || item.type == "mfs" ) {
							ligneLabel += " <button onclick='downloadGeneratedFile(\"" + streamId + "\",\"" + item.compentId + "\",\""
									+ item.fileItemId + "\",\"" + item.name + "\",\"" + item.type + "\")'  hidden='hidden' id='gen_" + item.id + "'>"
									+ searchPage.GENERATED_DOWNLOAD + "</button>"
						}
						return ligneLabel;
					},
					getIconClass : function ( item ) {
						if ( item.project )
							return item.type + " display_icon";
						else if ( item.project == "" && item.type == "dataelement" )
							return "pacDataElementDescription display_icon";
						return "";
					},
					_createTreeNode : function ( args ) {
						var tnode = new dijit._TreeNode( args );
						tnode.labelNode.innerHTML = args.label;
						return tnode;
					},
					_onNodeMouseEnter : function ( node, evt ) {
						domAttr.remove( "super_" + node.item.id, "hidden" );
						domAttr.remove( "sub_" + node.item.id, "hidden" );
						if ( node.item.type == "cbl" || node.item.type == "cpy" || node.item.type == "ddl" || node.item.type == "txt"
							|| node.item.type == "bms" || node.item.type == "map" || node.item.type == "mfs" )
							domAttr.remove( "gen_" + node.item.id, "hidden" );
					},
					_onNodeMouseLeave : function ( node, evt ) {
						domAttr.set( "super_" + node.item.id, "hidden", "hidden" );
						domAttr.set( "sub_" + node.item.id, "hidden", "hidden" );
						if ( node.item.type == "cbl" || node.item.type == "cpy" || node.item.type == "ddl" || node.item.type == "txt"
							|| node.item.type == "bms" || node.item.type == "map" || node.item.type == "mfs" )
							domAttr.set( "gen_" + node.item.id, "hidden", "hidden" );
					},
					onKeyUp : function ( event ) {
						var item = referenceTree.attr("selectedItem");
						if ( item == undefined ) {
							return;
						}						
						if (event.keyCode == dojo.keys.ENTER) {
							dojo.connect(dojo.byId("super_" + item.id), "onkeyup", function(event) {
								if (event.keyCode == 13) {
									dojo.byId("super_" + item.id).click();
								}
							});
							dojo.connect(dojo.byId("sub_" + item.id), "onkeyup", function(event) {
								if (event.keyCode == 13) {
									dojo.byId("sub_" + item.id).click();
								}
							});
							domAttr.remove( "super_" + item.id, "hidden" );
							domAttr.remove( "sub_" + item.id, "hidden" );
							if ( item.type == "cbl" || item.type == "cpy" || item.type == "ddl" || item.type == "txt"
								|| item.type == "bms" || item.type == "map" || item.type == "mfs" ) {
								dojo.connect(dojo.byId("gen_" + item.id), "onkeyup", function(event) {
									if (event.keyCode == 13) {
										dojo.byId("gen_" + item.id).click();
									}
								});
								domAttr.remove( "gen_" + item.id, "hidden" );
							}
						} else if (event.keyCode != dojo.keys.TAB ) {
							domAttr.set( "super_" + item.id, "hidden", "hidden" );
							domAttr.set( "sub_" + item.id, "hidden", "hidden" );
							if ( item.type == "cbl" || item.type == "cpy" || item.type == "ddl" || item.type == "txt"
								|| item.type == "bms" || item.type == "map" || item.type == "mfs" )
								domAttr.set( "gen_" + item.id, "hidden", "hidden" );
						}
					},
					// Set tool tip
					getTooltip : function ( item ) {
						return typeDict[ item.type ] + " : " + item.name;
					},
					style : {
						"clear" : "both"
					}
				} );

				// Event listeners for expanding node actions
				referenceTree.on( "open", function ( item, node ) {
					if ( !node.isExpandable ) {
						// If node never been expanded
						var variables = item.id.split( "/" );
						var name = variables[ 0 ];
						var type = variables[ 1 ];
						var project = variables[ 2 ];
						var nodePromise = sendReferenceForm( direction, name, type, project, context, streamId, item.id );
						nodePromise.then( function ( referencesNode ) {
							referencesNode.forEach( function ( item ) {
								referenceStore.put( item );
							} );
						} );
					}
				} );
				return referenceTree;
			} );
			return promiseReferenceTree;
		}
	};
} );