/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2016, 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dojo/ready", "dijit/form/Select", "dojo/request/xhr", "dojo/dom", "dojo/dom-construct", "dojo/store/Memory", "dijit/Tree",
		"dijit/tree/ObjectStoreModel", "dijit/form/CheckBox"
], function ( ready, Select, xhr, dom, domConstruct, Memory, Tree, ObjectStoreModel, CheckBox ) {
	function getProject ( streamId ) {
		// empty the former project tree
		domConstruct.empty( "searchIn" );
		enableSearchButton();

		xhr( net.jazz.ajax.contextRoot() + "/service/com.ibm.teampdp.query.engine.common.IRppQueryEngineService", {
			method : "POST",
			handleAs : "json",
			data : {
				search : "searchProjects",
				streamId : streamId,
			}
		} ).then( function ( data ) {
			data[ 0 ][ "root" ] = true;
			// set up the store to get the tree data
			var myStore = new Memory( {
				data : data,
				getChildren : function ( object ) {
					return object.children || [];
				}
			} );
			// Create the model
			var myModel = new ObjectStoreModel( {
				store : myStore,
				query : {
					root : true
				},
				mayHaveChildren : function ( item ) {
					return "children" in item;
				}
			} );

			var projectTree = new Tree( {
				model : myModel,
				autoExpand : true,
				style : {
					height : "100%"
				},
				getIconClass : function ( /* dojo.store.Item */item, /* Boolean */opened ) {
					return ( !item || this.model.mayHaveChildren( item ) )
							? ( opened
									? "dijitFolderOpened" : "dijitFolderClosed" ) : "dijitFolderOpened"
				},
				_createTreeNode : function ( args ) {
					var tnode = new dijit._TreeNode( args );
					tnode.labelNode.innerHTML = args.label;
					var cb = new CheckBox( {
						name : "searchIn",
						value : args.label,
						checked : true,
						onChange : function ( isChecked ) {
							cb.set( 'checked', isChecked );
							// RppJazzPage.js
							enableSearchButton();
						},
					} );
					cb.placeAt( tnode.labelNode, "first" );
					return tnode;
				},
			} );
			projectTree.placeAt( "searchIn" );
			enableSearchButton();

		}, function ( err ) {
			displayRppError( err );
		} );
	}
	return {
		createStreamSelect : function ( data ) {
			data.sort(function(a,b) {
					var _a = a.streamDisplayName;
					var _b = b.streamDisplayName;
					if (_a < _b)
						return -1;
					if (_a > _b)
						return 1;
					return 0;
			});
			var streamOptions = [];
			data.forEach( function ( item ) {
				streamOptions.push( {
					label : item.streamDisplayName,
					value : item.streamId
				} );
			} );
			var serverScope = new Select( {
				id : "streamId",
				name : "streamId",
				options : streamOptions,
				style : {
					width : '200px'
				},
			} );
			// first time
			getProject( dijit.byId( 'streamId' ).get( 'value' ) );
			// connect the select with function getProject
			serverScope.on( 'change', function () {
				getProject( this.get( "value" ) );
			} );
			return serverScope;
		}
	};
} );