/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2016, 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dojox/layout/TableContainer", "dijit/layout/ContentPane", "dijit/form/TextBox", "dojo/string",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PTEditor",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacEnumerationLabel",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function ( TableContainer, ContentPane, TextBox, string, PacbaseEditor, PTEditor, PacEnumerationLabel, typeDict ) {
	function definitionBase ( dataDescriptionExtension, label, streamId, project ) {
		var definitionTable = new TableContainer( {
			cols : 2,
			labelWidth : 150,
			style : {
				"width" : "700px"
			}
		} );

		// inherits from
		var inheritsFrom = dataDescriptionExtension.parent;
		if (  inheritsFrom == "" ) {
			var texInheritsFrom = new TextBox( {
				label : PacbaseEditor.INHERITS_FROM,
				value : inheritsFrom == ""
						? PacbaseEditor.NO_INHERITANCE : inheritsFrom.name,
				colspan : 2,
				style : {
					"width" : "400px"
				},
				disabled : true
			} );
			definitionTable.addChild( texInheritsFrom );
		} else {
			var parentDataElement = 
				new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget(PacbaseEditor.INHERITS_FROM, inheritsFrom, true, streamId, project);	
			definitionTable.addChild( parentDataElement );
			var textBoxInherits = TextBox( {
				type : "hidden",
				colspan : 1,
			} );
			definitionTable.addChild( textBoxInherits );
		}

		if ( inheritsFrom != "" ) {
			if ( label && label.length > 0 )
				checkInherited( definitionTable, PTEditor.LABEL, label, "", 1 );
			else
				checkInherited( definitionTable, PTEditor.LABEL, label, inheritsFrom.label, 1 );
			checkInherited( definitionTable, PacbaseEditor.DE_TYPE, dataDescriptionExtension.type, inheritsFrom.type, 1 );
			checkInherited( definitionTable, PacbaseEditor.INTERNAL_FORMAT, dataDescriptionExtension.internalFormat, inheritsFrom.internalFormat, 1 );
			checkInherited( definitionTable, PacbaseEditor.INTERNAL_USAGE, dataDescriptionExtension.internalUsage, inheritsFrom.internalUsage, 1 );
		} else {
			checkInherited( definitionTable, PTEditor.LABEL, label, "", 1 );
			checkInherited( definitionTable, PacbaseEditor.DE_TYPE, dataDescriptionExtension.type, "", 1 );
			checkInherited( definitionTable, PacbaseEditor.INTERNAL_FORMAT, dataDescriptionExtension.internalFormat, "", 1 );
			checkInherited( definitionTable, PacbaseEditor.INTERNAL_USAGE, dataDescriptionExtension.internalUsage, "", 1 );
		}
		return definitionTable;
	}

	function checkInherited ( definitionTable, pacbaseEditorLabel, dataDescriptionExtensionsValue, parent, colspan ) {
		var value;
		if ( typeof dataDescriptionExtensionsValue === 'object' ) {
			dataDescriptionExtensionsValue[ "value" ].toUpperCase() == "_INHERITED"
					? value = PacEnumerationLabel[ dataDescriptionExtensionsValue[ "class" ] + parent ]
					: value = PacEnumerationLabel[ dataDescriptionExtensionsValue[ "class" ] + dataDescriptionExtensionsValue[ "value" ] ];
		} else {
			value = dataDescriptionExtensionsValue != ""
					? dataDescriptionExtensionsValue : parent;
		}
		var textBox = TextBox( {
			label : pacbaseEditorLabel,
			value : value,
			colspan : colspan,
			style : {
				"width" : "400px"
			},
			disabled : true
		} );

		var textBoxInherits = TextBox( {
			type : "hidden",
			colspan : colspan,
		} );
		if ( ( dataDescriptionExtensionsValue === "" && parent != "" )
				|| ( typeof dataDescriptionExtensionsValue === 'object' && dataDescriptionExtensionsValue[ "value" ].toUpperCase() === "_INHERITED" ) ) {
			textBoxInherits.set( "label", PacbaseEditor.FORMAT_INHERITED );
		} else if ( parent != "" && dataDescriptionExtensionsValue[ "value" ] != parent ) {
			textBoxInherits.set( "label", PacbaseEditor.FORMAT_REDEFINED );
		}
		definitionTable.addChild( textBox );
		definitionTable.addChild( textBoxInherits );
	}

	return {
		getOverviewDefinition : function ( dataDescriptionExtension, label, type, streamId, project ) {
			var definitionTable = definitionBase( dataDescriptionExtension, label, streamId, project );
			var overviewDefinition = new ContentPane( {
				content :  "<hr><strong>" + PacbaseEditor.DATA_ELEMENT_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><P>"
						+ string.substitute( PacbaseEditor.DATA_ELEMENT_SECTION_DESCRIPTION, [
							typeDict[ type ]
						] ) + "</p>",
			} );
			overviewDefinition.addChild( definitionTable );
			return overviewDefinition;
		},

		getDetailDefinition : function ( dataDescriptionExtension, name, label, type, streamId, project ) {
			var definitionTable = definitionBase( dataDescriptionExtension, label, streamId, project );
			var detailDefinition = new ContentPane( {
				content : "<strong>" + typeDict[ type ] + " " + name + " : " + label + "</strong><P>"
			    + "<strong>" + PacbaseEditor.DATA_ELEMENT_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
						+ string.substitute( PacbaseEditor.DATA_ELEMENT_EDIT_SECTION_DESCRIPTION, [
							typeDict[ type ]
						] ) + "</p>",
			} );
			if ( dataDescriptionExtension.parent != "" ) {
				var parent = dataDescriptionExtension.parent;
				checkInherited( definitionTable, PacbaseEditor.INPUT_FORMAT, dataDescriptionExtension.inputFormat, parent.inputFormat, 1 );
				checkInherited( definitionTable, PacbaseEditor.OUTPUT_FORMAT, dataDescriptionExtension.outputFormat, parent.outputFormat, 1 );
				checkInherited( definitionTable, PacbaseEditor.BLANK_WHEN_ZERO, dataDescriptionExtension.blkWhenZero, parent.blkWhenZero, 1 );
			} else {
				checkInherited( definitionTable, PacbaseEditor.INPUT_FORMAT, dataDescriptionExtension.inputFormat, "", 1 );
				checkInherited( definitionTable, PacbaseEditor.OUTPUT_FORMAT, dataDescriptionExtension.outputFormat, "", 1 );
				checkInherited( definitionTable, PacbaseEditor.BLANK_WHEN_ZERO, dataDescriptionExtension.blkWhenZero, "", 1 );
			}
			detailDefinition.addChild( definitionTable );
			return detailDefinition;
		}
	};
} );