/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2018. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dojo/string", "dojox/layout/TableContainer", "dijit/layout/ContentPane", "dijit/form/TextBox", "dijit/tree/ForestStoreModel",
		"dojo/data/ItemFileReadStore", "dojox/grid/LazyTreeGrid", "dijit/form/Button", "dijit/Dialog", "dijit/Tooltip", "./gLinesPane",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function ( string, TableContainer, ContentPane, TextBox, ForestStoreModel, ItemFileReadStore, LazyTreeGrid, Button, Dialog, Tooltip, gLinesPane, 
				PacbaseEditor, typeDict ) {	

	
	function getValue ( value ) {		
		return value.toString().replace("_","").replace("None","");
	}

	function getNodecode ( item ) {
		if (item.nodeCode != undefined && item.nodeCode != "" && item["class"] != undefined) {
			return item.nodeCode;
		}		

		return "";		
	}
	
	function getPacServer ( item, store, project, streamId ) {
		// item is pacServer
		if (item.pacServer != undefined) {
			return item.pacServer[0].name
		}
		return "";
	}

	function getLogicalView ( item, store, project, streamId ) {
		// item is logicalView
		if (item.logicalView != undefined) {
			return item.logicalView[0].name

		}
		return "";
	}

	
	return {
		
		getDetailCSLines : function ( data, type, project, streamId ) 
		{	
			var pacRootNode = data.pacRootNode
			var cpDetailCSLines = new ContentPane( {
				content : "<strong>" + typeDict[ type ] + " " + data.name + " : " + data.label + "</strong><P>"
			    + "<strong>" + PacbaseEditor.FOLDER_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
						+ string.substitute( PacbaseEditor.FOLDER_TABLE_SECTION_DESCRIPTION, [
							typeDict[ type ]
						] ) + "</p>",
						style : {
							'overflow' : 'auto',
						},
			} );
			// prepare lazy tree grid
			var layoutCSLines = [
					{
						name : PacbaseEditor.FOLDER_CSLINE_TABTEXT,
						width : "600px",
						field : "_item",
						formatter : function ( item ) {
							if (item == undefined || item == "") return "";
							return PacbaseEditor.FOLDERVIEW_NODE_CODE + ": " 
							+ getNodecode (item)
							+ " " + PacbaseEditor.FOLDERVIEW_LOGICAL_VIEW + ": " 
							+ getLogicalView (item, this.grid.store, project, streamId)
							+ " " + PacbaseEditor.FOLDERVIEW_COMPONENT + ": " 
							+ getPacServer (item, this.grid.store, project, streamId)							
							;
						}
					}				 
			];			

			// noeud racine vide
			var pacRootNodes = [];
			
			if (pacRootNode != undefined && pacRootNode != "") {
				pacRootNodes = [pacRootNode];
			} 

			// prepare lazy tree grid
			var data = {
				items : pacRootNodes
			};
			var store = new ItemFileReadStore( {
				data : data,
			} );

			
			var index = 0;
			data.items.forEach(function(item) {
				item.treeId = index++;
			});
			
			
			var model = new ForestStoreModel( {
				store : store,
				mayHaveChildren : function ( item ) {
					if ( item.childNodes != undefined && item.childNodes.length > 0 ) {						
						return true;
					}
					return false;
				},
				getChildren : function ( item, onComplete, onError ) {
					var childNodes = null;
					if ( item.childNodes != undefined && item.childNodes.length > 0) {
						childNodes = item.childNodes;
					}
					if ( childNodes != null ) {
						index = 0;
						childNodes.forEach(function(child) {
							child.treeId = item.treeId + index;
						});
						onComplete(childNodes);
					}
				}
			} );
		
//			var hearderTooltips =  [PacbaseEditor.CSLINE_COLUMN_TOOL_TIP_NODE_CODE, 
//			                        PacbaseEditor.CSLINE_COLUMN_TOOL_TIP_COMPONENT,
//			                        PacbaseEditor.CSLINE_COLUMN_TOOL_TIP_LOGICAL_VIEW,
//			                        PacbaseEditor.CSLINE_COLUMN_TOOL_TIP_TYPE_NODE,
//			                        PacbaseEditor.CSLINE_COLUMN_TOOL_TIP_CARDINALITY,
//			                        PacbaseEditor.CSLINE_COLUMN_TOOL_TIP_SUBSCHEMA,
//			                        PacbaseEditor.CSLINE_COLUMN_TOOL_TIP_DATAELEMENT_SOURCE,
//			                        PacbaseEditor.CSLINE_COLUMN_TOOL_TIP_DATAELEMENT_KEY];
			
			var grid = new LazyTreeGrid( {
				treeModel : model,
				structure : layoutCSLines,
				height : ( rppCurrentPaneGridHeight - 70 ) + "px",
				autoWidth : true,
				selectable : true,
				
				canSort : function () {
					return false;
//				},
//				onHeaderCellMouseOver  : function (e) {
//					dijit.showTooltip(hearderTooltips[e.cellIndex], e.cellNode);
//				},
//				onHeaderCellMouseOut : function (e) {
//					dijit.hideTooltip(e.cellNode);
				} 				
			
			} );
			cpDetailCSLines.addChild( grid );
			return cpDetailCSLines;
		}
	}
} );