/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2016, 2018. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dojo/string", "dijit/layout/TabContainer", "dojox/layout/TableContainer", "dijit/layout/ContentPane", 'dojox/grid/DataGrid', 'dojo/data/ItemFileWriteStore',
		"dijit/Dialog", "dijit/TooltipDialog",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function ( string, TabContainer, TableContainer, ContentPane, DataGrid, ItemFileWriteStore, Dialog, TooltipDialog, PacbaseEditor, typeDict ) {
	function getGCLines ( data, type, streamId, project ) {
		var gcLines;
		var extensions;
		if (data.extensions != undefined) {
			var extensions = data.extensions.filter( function ( object ) {
				return object.facet === "pacbase";
			} );
		}
		if ( data.extensions != undefined && extensions.length === 1 ) {		
			gcLines = extensions[ 0 ].GCLines
		} else {
		    gcLines = data.GCLines
		}
		var gcLinesContentPane = new ContentPane( {
			title : PacbaseEditor.GCLINE_EDIT_SECTION_HEADER,
		} );

		// set up data store
		gcLines.forEach( function ( gcLine ) {
			if ( gcLine.lineType == "I" && ( "data" in gcLine ) ) {
				gcLine.description = "["
				gcLine.description += gcLine.data.join( ', ' );
				gcLine.description += "]";
				gcLine.pacInputAid = gcLine.pacInputAid;
			} else {
				gcLine.pacInputAid = "";
			}
		} );

		var store = new ItemFileWriteStore( {
			data : {
				items : gcLines
			}
		} );
		// set up layout
		var layout = [
				{
					'name' : PacbaseEditor.GLINE_TYPE,
					'field' : 'lineType'
				}, {
					'name' : PacbaseEditor.GLINE_DESCRIPTION,
					'field' : 'description',
					'width' : '700px'
				}, {
					'name' : PacbaseEditor.GLINE_INPUTAID,
					'field' : 'pacInputAid',
					'width' : '100px',
					formatter : function ( item ) {
						if ( item == undefined || item == "" )
							return "";
						return "<a class='edit-current-inputaid'  href='#' onclick='return false;'>"+item.name+"</a>";
					}		
				}
		];

		// create a new grid
		var gcLinesGrid = new DataGrid( {
			store : store,
			structure : layout,
			height : ( rppCurrentPaneGridHeight - 70 ) + "px",
			autoWidth : true,
			selectable : true,
			canSort : function () {
				return false;
			},
			onHeaderCellMouseOver  : function (e) {
				dijit.showTooltip(e.cell.name, e.cellNode);
			},
			onHeaderCellMouseOut : function (e) {
				dijit.hideTooltip(e.cellNode);
			}
		} );

		var gLineTooltipDialog = null;
		
		gcLinesGrid.on('a.edit-current-inputaid:click', function (event) {
			if ( gLineTooltipDialog ) {
				dijit.popup.close(gLineTooltipDialog);
				gLineTooltipDialog.destroy();
				gLineTooltipDialog = null;
			}
			displayInputAid ( event.grid.getItem(event.rowIndex), streamId, project );
		});
		
		gcLinesGrid.on('a.edit-current-inputaid:mouseover', function (event) {
			var cpPane = getInputAidContentPane(event.grid.getItem(event.rowIndex));
			cpPane.startup();

		    var gLineTooltipDialog = new TooltipDialog({
		    	style: "width:800px;height:100px;",
		        onMouseLeave: function(){
		        	dijit.popup.close(gLineTooltipDialog);
		        },

		    });
		    gLineTooltipDialog.addChild(cpPane);
		    gLineTooltipDialog.startup();
	    	dijit.popup.open({
	            popup: gLineTooltipDialog,
	            around: event.cellNode
	        });
	    	require(["dojo/on"], function(on){
	    		on(event.cellNode, 'mouseleave', function(){
	    			if ( gLineTooltipDialog ) {
	    				dijit.popup.close(gLineTooltipDialog);
	    				gLineTooltipDialog.destroy();
	    				gLineTooltipDialog = null;
	    			}
			    });
	    	});
		});

// NOT WORKING !!!
//		gcLinesGrid.on('a.edit-current-inputaid:mouseleave', function (event) {
//			if ( gcLineTooltipDialog ) {
//				dijit.popup.close(gcLineTooltipDialog);
//			}
//		});
				
		var gcLinesCPane = new ContentPane( {
			content :  "<strong>" + typeDict[ type ] + " " + data.name + " : " + data.label + "</strong><P>"
		    + "<strong>" + PacbaseEditor.GCLINE_EDIT_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
			+ string.substitute( PacbaseEditor.GCLINE_EDIT_SECTION_DESCRIPTION, [
				typeDict[ type ]
			] ) + "</p>",
			style : {
				'overflow' : 'auto',
			},
			tabindex : "0",
		} );
		gcLinesCPane.addChild( gcLinesGrid );
		gcLinesContentPane.addChild( gcLinesCPane );
		return gcLinesContentPane;
	}

	function displayInputAid ( dataItem, streamId, project ) {
	    dlg = new dijit.Dialog({
	        title:PacbaseEditor.GLINE_INPUTAID_DETAIL_VALUE,
	        content:getInputAidContentPane(dataItem, true, streamId, project, true).domNode
	      });
	    dlg.show();
	}
	
	function getInputAidContentPane ( dataItem, isDialog, streamId, project) {
		var iacLinesPane = new ContentPane( {
			title : PacbaseEditor.GLINE_TABTEXT,
			style:"overflow:'auto'",
			tabindex : "0",
		} );

		
		if ( dataItem.iacLines == undefined ) {
			var content = "<p>"+PacbaseEditor.GLINE_INPUTAID_NOT_FOUND+"</p>";
			iacLinesPane.setContent (content);
			return iacLinesPane;
		}
		
		if ( isDialog ) {
			var iacLinesTable = new TableContainer( {
				labelWidth : rppDefaultLabelWidth,
			} );
		
			var pacInputAid = 
				new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget
					(PacbaseEditor.GLINE_INPUTAID, dataItem.pacInputAid[0], true, streamId, project);

			iacLinesTable.addChild( pacInputAid );
			iacLinesPane.addChild( iacLinesTable );
		}
		
		// set up data store
		var iacLinesStore = new ItemFileWriteStore( {
			data : {
				items : dataItem.iacLines
			}
		} );
		// set up layout
		var iacLinesLayout = [
				{
					'name' : PacbaseEditor.GLINE_TYPE,
					'field' : 'type',
					'width' : '30px',
					formatter : function ( item ) {
						if ( item == undefined || item == "" )
							return "";
						return item.value.toString().replace("_","").replace("None","");
					}
				}, {
					'name' : PacbaseEditor.GLINE_INPUTAID_FIXED_LABEL,
					'field' : 'fixedLabel',
					'width' : '250px',
					formatter : function ( item ) {
						return item.replace(/ /g,"&nbsp;");
					}
				}, {
					'name' : PacbaseEditor.GLINE_INPUTAID_VARIABLE_LABEL,
					'field' : 'variableLabel',
					'width' : '250px',
					formatter : function ( item ) {
						return item.replace(/ /g,"&nbsp;");
					}
				}, {
					'name' : PacbaseEditor.GLINE_INPUTAID_SYNBOLIC_VALUE,
					'field' : 'symbolicValue',
					'width' : '250px',
					formatter : function ( item ) {
						return item.replace(/ /g,"&nbsp;");
					}
				}
		];

		if ( isDialog ) {
			// create a new grid
			var iacLinesGrid = new DataGrid( {
				store : iacLinesStore,
				structure : iacLinesLayout,
				selectable : true,
				height : ( rppCurrentPaneGridHeight - 70 ) + "px",
				autoWidth : true,
				canSort : function () {
					return false;
				},	
				onStyleRow: function (row) {
					row.customClasses = "rppGridFixedDefaultFont";
				},
			} );
			iacLinesPane.addChild( iacLinesGrid );
		} else {
			// create a new grid
			var iacLinesGrid = new DataGrid( {
				store : iacLinesStore,
				structure : iacLinesLayout,
				selectable : true,
				autoHeight :true,
				autoWidth : true,
				canSort : function () {
					return false;
				},	
				onStyleRow: function (row) {
					row.customClasses = "rppGridFixedDefaultFont";
				},
			} );
			iacLinesPane.addChild( iacLinesGrid );			
		}
			
		return iacLinesPane;
			
	};
	
	function getGELines ( data, type, streamId, project ) {
		var geLines;
		var extensions;
		if (data.extensions != undefined) {
			var extensions = data.extensions.filter( function ( object ) {
				return object.facet === "pacbase";
			} );
		}
		if ( data.extensions != undefined && extensions.length === 1 ) {		
			geLines = extensions[ 0 ].GELines
		} else {
		    geLines = data.GELines
		}		
		var geLinesContentPane = new ContentPane( {
			title : PacbaseEditor.GELINE_EDIT_SECTION_HEADER,
		} );

		// set up data store
		var store = new ItemFileWriteStore( {
			data : {
				items : geLines
			}
		} );
		// set up layout
		var layout = [
				{
					'name' : PacbaseEditor.GLINE_TYPE,
					'field' : 'lineType'
				}, {
					'name' : PacbaseEditor.GLINE_LINK,
					'field' : 'linkedEntity',
					'width' : '300px',
					formatter : function ( linkedEntity ) {
						if ( linkedEntity != "" ) {
							return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", linkedEntity, true, streamId, project); 
						} else {
							return "";
						}
					},
				}, {
					'name' : PacbaseEditor.GLINE_DESCRIPTION,
					'field' : 'description',
					'width' : '700px'
				}
		];

		// create a new grid
		var geLinesGrid = new DataGrid( {
			store : store,
			structure : layout,
			height : ( rppCurrentPaneGridHeight - 70 ) + "px",
			autoWidth : true,
			selectable : true,
			canSort : function () {
				return false;
			},
			onHeaderCellMouseOver  : function (e) {
				dijit.showTooltip(e.cell.name, e.cellNode);
			},
			onHeaderCellMouseOut : function (e) {
				dijit.hideTooltip(e.cellNode);
			}
		} );
		
		var geLinesCPane = new ContentPane( {
			content :  "<strong>" + typeDict[ type ] + " " + data.name + " : " + data.label + "</strong><P>"
		    + "<strong>" + PacbaseEditor.GELINE_EDIT_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
			+ string.substitute( PacbaseEditor.GELINE_EDIT_SECTION_DESCRIPTION, [
				typeDict[ type ]
			] ) + "</p>",
			style : {
				'overflow' : 'auto',
			},
			tabindex : "0",
		} );
		geLinesCPane.addChild( geLinesGrid );
		geLinesContentPane.addChild( geLinesCPane );
		return geLinesContentPane;
	}

	function getGGLines ( data , type, streamId, project ) {
		var ggLines;
		var extensions;
		if (data.extensions != undefined) {
			extensions = data.extensions.filter( function ( object ) {
				return object.facet === "pacbase";
			} );
		}
		if ( data.extensions != undefined && extensions.length === 1 ) {		
			ggLines = extensions[ 0 ].GGLines
		} else {
		    ggLines = data.GGLines
		}
		var ggLinesContentPane = new ContentPane( {
			title : PacbaseEditor.GGLINE_EDIT_SECTION_HEADER,
		} );

		// set up data store
		// if ( ggLines.length > 0 ) {
			ggLines.forEach( function ( ggLine ) {
				if ( ggLine.lineType == "I" && ( "data" in ggLine ) ) {
					ggLine.description = "["
					ggLine.description += ggLine.data.join( ', ' );
					ggLine.description += "]";
					ggLine.pacInputAid = ggLine.pacInputAid;
				} else {
					ggLine.pacInputAid = "";
				}
			} );

			var store = new ItemFileWriteStore( {
				data : {
					items : ggLines
				}
			} );
			// set up layout
			var layout = [
					{
						'name' : PacbaseEditor.GLINE_TYPE,
						'field' : 'lineType'
					}, {
						'name' : PacbaseEditor.GLINE_DESCRIPTION,
						'field' : 'description',
						'width' : '700px'
					}, {
						'name' : PacbaseEditor.GLINE_INPUTAID,
						'field' : 'pacInputAid',
						'width' : '100px',
						formatter : function ( item ) {
							if ( item == undefined || item == "" )
								return "";
							return "<a class='edit-current-inputaid'  href='#' onclick='return false;'>"+item.name+"</a>";
						}	
					}, {
						'name' : PacbaseEditor.GLINE_LINK,
						'field' : 'linkedEntity',
						'width' : '300px',
						formatter : function ( linkedEntity ) {
							if ( linkedEntity != "" ) {
								return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", linkedEntity, true, streamId, project); 
							} else {
								return "";
							}
						},
					}
			];

			// create a new grid
			var ggLinesGrid = new DataGrid( {
				store : store,
				structure : layout,
				height : ( rppCurrentPaneGridHeight - 70 ) + "px",
				autoWidth : true,
				selectable : true,
				canSort : function () {
					return false;
				},
				onHeaderCellMouseOver  : function (e) {
					dijit.showTooltip(e.cell.name, e.cellNode);
				},
				onHeaderCellMouseOut : function (e) {
					dijit.hideTooltip(e.cellNode);
				}
			} );
			
			var gLineTooltipDialog = null;
			
			ggLinesGrid.on('a.edit-current-inputaid:click', function (event) {
				if ( gLineTooltipDialog ) {
					dijit.popup.close(gLineTooltipDialog);
					gLineTooltipDialog.destroy();
					gLineTooltipDialog = null;
				}
				displayInputAid ( event.grid.getItem(event.rowIndex), streamId, project );
			});
			
			ggLinesGrid.on('a.edit-current-inputaid:mouseover', function (event) {
				var cpPane = getInputAidContentPane(event.grid.getItem(event.rowIndex));
				cpPane.startup();

			    var gLineTooltipDialog = new TooltipDialog({
			    	style: "width:800px;height:100px;",
			        onMouseLeave: function(){
			        	dijit.popup.close(gLineTooltipDialog);
			        },

			    });
			    gLineTooltipDialog.addChild(cpPane);
			    gLineTooltipDialog.startup();
		    	dijit.popup.open({
		            popup: gLineTooltipDialog,
		            around: event.cellNode
		        });
		    	require(["dojo/on"], function(on){
		    		on(event.cellNode, 'mouseleave', function(){
		    			if ( gLineTooltipDialog ) {
		    				dijit.popup.close(gLineTooltipDialog);
		    				gLineTooltipDialog.destroy();
		    				gLineTooltipDialog = null;
		    			}
				    });
		    	});
			});
				
			var ggLinesCPane = new ContentPane( {
				content :  "<strong>" + typeDict[ type ] + " " + data.name + " : " + data.label + "</strong><P>"
			    + "<strong>" + PacbaseEditor.GGLINE_EDIT_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
				+ string.substitute( PacbaseEditor.GGLINE_EDIT_SECTION_DESCRIPTION, [
					typeDict[ type ]
				] ) + "</p>",
				style : {
					'overflow' : 'auto',
				},
				tabindex : "0",
			} );
			ggLinesCPane.addChild( ggLinesGrid );
			ggLinesContentPane.addChild( ggLinesCPane );
		//}

		return ggLinesContentPane;
	}

	function getGOLines ( data, type, streamId, project ) {
		var goLines;
		var extensions;
		if (data.extensions != undefined) {
			var extensions = data.extensions.filter( function ( object ) {
				return object.facet === "pacbase";
			} );
		}
		if ( data.extensions != undefined && extensions.length === 1 ) {		
			goLines = extensions[ 0 ].GOLines
		} else {
		    goLines = data.GOLines
		}	
		var goLinesContentPane = new ContentPane( {
			title : PacbaseEditor.GOLINE_EDIT_SECTION_HEADER,
		} );

		// set up data store
		//if ( goLines.length > 0 ) {
			goLines.forEach( function ( goLine ) {
				if ( goLine.lineType == "I" && ( "data" in goLine ) ) {
					goLine.description = "["
					goLine.description += goLine.data.join( ', ' );
					goLine.description += "]";
					goLine.pacInputAid = goLine.pacInputAid;
				} else {
					goLine.pacInputAid = "";
				}
			} );

			var store = new ItemFileWriteStore( {
				data : {
					items : goLines
				}
			} );
			// set up layout
			var layout = [
					{
						'name' : PacbaseEditor.GLINE_TYPE,
						'field' : 'lineType'
					}, {
						'name' : PacbaseEditor.GLINE_DESCRIPTION,
						'field' : 'description',
						'width' : '700px'
					}, {
						'name' : PacbaseEditor.GLINE_INPUTAID,
						'field' : 'pacInputAid',
						'width' : '100px',
						formatter : function ( item ) {
							if ( item == undefined || item == "" )
								return "";
							return "<a class='edit-current-inputaid'  href='#' onclick='return false;'>"+item.name+"</a>";
						}	
					}
			];

			// create a new grid
			var goLinesGrid = new DataGrid( {
				store : store,
				structure : layout,
				height : ( rppCurrentPaneGridHeight - 70 ) + "px",
				autoWidth : true,
				selectable : true,
				canSort : function () {
					return false;
				},
				onHeaderCellMouseOver  : function (e) {
					dijit.showTooltip(e.cell.name, e.cellNode);
				},
				onHeaderCellMouseOut : function (e) {
					dijit.hideTooltip(e.cellNode);
				}
			} );
			
			var gLineTooltipDialog = null;
			
			goLinesGrid.on('a.edit-current-inputaid:click', function (event) {
				if ( gLineTooltipDialog ) {
					dijit.popup.close(gLineTooltipDialog);
					gLineTooltipDialog.destroy();
					gLineTooltipDialog = null;
				}
				displayInputAid ( event.grid.getItem(event.rowIndex), streamId, project );
			});
			
			goLinesGrid.on('a.edit-current-inputaid:mouseover', function (event) {
				var cpPane = getInputAidContentPane(event.grid.getItem(event.rowIndex));
				cpPane.startup();

			    var gLineTooltipDialog = new TooltipDialog({
			    	style: "width:800px;height:100px;",
			        onMouseLeave: function(){
			        	dijit.popup.close(gLineTooltipDialog);
			        },

			    });
			    gLineTooltipDialog.addChild(cpPane);
			    gLineTooltipDialog.startup();
		    	dijit.popup.open({
		            popup: gLineTooltipDialog,
		            around: event.cellNode
		        });
		    	require(["dojo/on"], function(on){
		    		on(event.cellNode, 'mouseleave', function(){
		    			if ( gLineTooltipDialog ) {
		    				dijit.popup.close(gLineTooltipDialog);
		    				gLineTooltipDialog.destroy();
		    				gLineTooltipDialog = null;
		    			}
				    });
		    	});
			});
			
			var goLinesCPane = new ContentPane( {
				content :  "<strong>" + typeDict[ type ] + " " + data.name + " : " + data.label + "</strong><P>"
			    + "<strong>" + PacbaseEditor.GOLINE_EDIT_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
				+ string.substitute( PacbaseEditor.GOLINE_EDIT_SECTION_DESCRIPTION, [
					typeDict[ type ]
				] ) + "</p>",
				style : {
					'overflow' : 'auto',
				},
				tabindex : "0",
			} );
			goLinesCPane.addChild( goLinesGrid );
			goLinesContentPane.addChild( goLinesCPane );
			
		//}
		return goLinesContentPane;
	}

	return {
		getOverviewGLines : function ( type ) {
			var overviewGLines = new ContentPane( {
				content : "<hr><strong>" + PacbaseEditor.GLINE_TABTEXT + PacbaseEditor.SEPARATOR + "</strong><P>" + string.substitute( PacbaseEditor.GLINE_SECTION_DESCRIPTION, [
					typeDict[ type ]
				] ) + "</p>",
				tabindex : "0",
			} );
			return overviewGLines;
		},
		getGLines : function ( data, type, hasGC, hasGE, hasGG, hasGO, streamId, project ) {
			var cpGLines = new ContentPane( {
			} );
			var gLinesTab = new TabContainer();
			if ( hasGC== true ) {
				// get gc lines
				gLinesTab.addChild( getGCLines( data, type, streamId, project  ) );
			}
			if ( hasGE == true ) {
				// get ge lines
				gLinesTab.addChild( getGELines( data, type, streamId, project ) );
			}
			if ( hasGG == true ) {
				// get gg lines
				gLinesTab.addChild( getGGLines( data, type, streamId, project  ) );
			}
			if ( hasGO == true ) {
				// get go lines
				gLinesTab.addChild( getGOLines( data, type, streamId, project  ) );
			}
			//return gLinesTab;
			cpGLines.addChild(gLinesTab);
			return cpGLines;
		},
	
	};
} );