/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2016, 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
	"dojo/string", "dijit/layout/ContentPane", "dijit/tree/ForestStoreModel", "dojo/data/ItemFileReadStore", 
	"dojox/grid/LazyTreeGrid", "dijit/Tooltip", 
	"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",		
	"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function (string, ContentPane, ForestStoreModel, ItemFileReadStore, LazyTreeGrid, Tooltip, PacbaseEditor, typeDict ) {
	
	function getValue ( value ) {		
		return value.toString().replace("_","").replace("None","");
	}
	
	function getType ( item ) {
		if (item.lineType != undefined ) {
			var type = getValue(item.lineType[0].value);
			return "<span class='display_icon pacInputAidDescriptionLine'></span>&nbsp" + type;
		}
		return "";
	}	
	
	function getFixLabel ( item ) {
		if (item.fixedLabel != undefined) {	
			var newFixedLabel  = '';			
			newFixedLabel += item.fixedLabel[0].replace(/ /g,"&nbsp;");
			return newFixedLabel;
		}
		return "";
	}
	
	function getVariableLabel ( item ) {
		if (item.variableLabel != undefined) {
			var newVariableLabel  = '';			
			newVariableLabel += item.variableLabel[0].replace(/ /g,"&nbsp;");
			return newVariableLabel;			
		}		
		return "";			
	}
	
	function getSymVal ( item ) {	
		if (item.symbolicParameter != undefined)		
			return item.symbolicParameter;
		return "";
	}
	
	function getLength ( item ) {	
		if (item.length != undefined) 
			return item.length;
		return "";
	}
	
	function getCallType ( item ) {	
		if (item.callTypeLine != undefined )
			var callType = getValue(item.callTypeLine[0].value);
			return callType;
		return "";
	}	
	
	function getKey ( item ) {	
		if (item.xRefKey != undefined) 
			return item.xRefKey;
		return "";
	}
		
	return {		
		getDetailDLines : function ( data, type, project, streamId, compentId, fileItemId ) {
			
			var DLines = data.descriptionLines

			var cpDetailDLines = new ContentPane( {
				content : "<strong>" + typeDict[ type ] + " " + data.name + " : " + data.label + "</strong><P>"
					    + "<strong>" + PacbaseEditor.INPUT_AID_DLINE_TABLE_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
						+ string.substitute( PacbaseEditor.INPUT_AID_DLINE_TABLE_SECTION_DESCRIPTION, [
							typeDict[ type ]
						] ) + "</p>",
						style : {
							'overflow' : 'auto',
						},
			} );			
			// prepare lazy tree grid
			var layoutDLines = [
					{
						name : PacbaseEditor.INPUT_AID_DLINE_COLUMN_TYPE,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getType( item );
						}, 
					}, {
						name : PacbaseEditor.INPUT_AID_DLINE_COLUMN_FIX_LABEL,
						width : "300px",
						field : "_item",
						formatter : function ( item ) {
							return getFixLabel( item );
						},
					},{
						name : PacbaseEditor.INPUT_AID_DLINE_COLUMN_VAR_LABEL,
						width : "300px",
						field : "_item",
						formatter : function ( item ) {
							return getVariableLabel( item );
						},
					},  {
						name : PacbaseEditor.INPUT_AID_DLINE_COLUMN_SYM_VAL,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getSymVal( item );
						},
					}, {
						name : PacbaseEditor.INPUT_AID_DLINE_COLUMN_LENGTH,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getLength( item );
						},
					}, {
						name : PacbaseEditor.INPUT_AID_DLINE_COLUMN_CALL_TYPE,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getCallType( item );
						},
					}, {
						name : PacbaseEditor.INPUT_AID_DLINE_COLUMN_KEY,
						width : "150px",
						field : "_item",
						formatter : function ( item ) {
							return getKey( item );
						},
					},					
			];
			
			// prepare lazy tree grid
			var data = {
				items : DLines
			};
			var store = new ItemFileReadStore( {
				data : data,
			} );
			var model = new ForestStoreModel( {
				store : store,
				
				getParent : function ( item ) {
										
					var arr = this.store._arrayOfTopLevelItems[0];
					return arr; 
				}
			} );
			
			var hearderTooltips =  [PacbaseEditor.INPUT_AID_DLINE_COLUMN_TOOL_TIP__TYPE,
					                PacbaseEditor.INPUT_AID_DLINE_COLUMN_TOOL_TIP__FIX_LABEL,
					                PacbaseEditor.INPUT_AID_DLINE_COLUMN_TOOL_TIP__VAR_LABEL,			                
					                PacbaseEditor.INPUT_AID_DLINE_COLUMN_TOOL_TIP__SYM_VAL,
					                PacbaseEditor.INPUT_AID_DLINE_COLUMN_TOOL_TIP__LENGTH,
					                PacbaseEditor.INPUT_AID_DLINE_COLUMN_TOOL_TIP__CALL_TYPE,					                
					                PacbaseEditor.INPUT_AID_DLINE_COLUMN_TOOL_TIP__KEY];
			
			var grid = new LazyTreeGrid( {
				treeModel : model,
				structure : layoutDLines,
				height : ( rppCurrentPaneGridHeight - 70 ) + "px",
				autoWidth : true,
				onStyleRow: function (row) {
					row.customClasses = "rppGridFixedDefaultFont";
				},
				selectable : true,
				canSort : function () {
					return false;
				},
				onHeaderCellMouseOver  : function (e) {
					dijit.showTooltip(hearderTooltips[e.cellIndex], e.cellNode);
				},
				onHeaderCellMouseOut : function (e) {
					dijit.hideTooltip(e.cellNode);
				} 				
			
			} );

			cpDetailDLines.addChild( grid );
			return cpDetailDLines;
		}
	}
} );