/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2016, 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
	"dojo/string", "dijit/layout/ContentPane", "dijit/tree/ForestStoreModel", "dojo/data/ItemFileReadStore", 
	"dojox/grid/LazyTreeGrid", "dijit/Tooltip", 
	"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",		
	"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function (string, ContentPane, ForestStoreModel, ItemFileReadStore, LazyTreeGrid, Tooltip, PacbaseEditor, typeDict ) {
		
	function getParameter ( item ) {	
		if (item.parameter != undefined)		
			return item.parameter;
		return "";
	}
	
	function getValue ( item ) {	
		if (item.value != undefined) 
			return item.value;
		return "";
	}
	
	return {		
		getSymbolicValues : function ( data, type, project, streamId, compentId, fileItemId ) {
			
			var symbolicValueLines = data.symbolicValueLines

			var cpSymbolicValueLines = new ContentPane( {
				content : "<strong>" + typeDict[ type ] + " " + data.name + " : " + data.label + "</strong><P>"
			            + "<strong>" + PacbaseEditor.INPUT_AID_SYMBOLIC_VALUES_TABLE_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
						+ string.substitute( PacbaseEditor.INPUT_AID_SYMBOLIC_VALUES_TABLE_SECTION_DESCRIPTION, [
							typeDict[ type ]
						] ) + "</p>",
						style : {
							'overflow' : 'auto',
						},
			} );			
			// prepare lazy tree grid
			var layoutSymbolicValueLines = [
					{
						name : PacbaseEditor.INPUT_AID_SYMBOLIC_PARAMETER,
						width : "100px",
						field : "_item",
						formatter : function ( item ) {
							return getParameter( item );
						}, 
					}, {
						name : PacbaseEditor.INPUT_AID_SYMBOLIC_VALUE,
						width : "300px",
						field : "_item",
						formatter : function ( item ) {
							return getValue( item );
						},
					},
			];
			
			// prepare lazy tree grid
			var data = {
				items : symbolicValueLines
			};
			var store = new ItemFileReadStore( {
				data : data,
			} );
			var model = new ForestStoreModel( {
				store : store,
				
				getParent : function ( item ) {
										
					var arr = this.store._arrayOfTopLevelItems[0];
					return arr; 
				}
			} );
			
			var hearderTooltips =  [PacbaseEditor.INPUT_AID_SYMBOLIC_COLUMN_TOOL_TIP__PARAMETER,					                		                
					                PacbaseEditor.INPUT_AID_SYMBOLIC_COLUMN_TOOL_TIP__VALUE];
			
			var grid = new LazyTreeGrid( {
				treeModel : model,
				structure : layoutSymbolicValueLines,
				height : ( rppCurrentPaneGridHeight - 70 ) + "px",
				autoWidth : true,
				selectable : true,
				onStyleRow: function (row) {
					row.customClasses = "rppGridFixedDefaultFont";
				},
				canSort : function () {
					return false;
				},
				onHeaderCellMouseOver  : function (e) {
					dijit.showTooltip(hearderTooltips[e.cellIndex], e.cellNode);
				},
				onHeaderCellMouseOut : function (e) {
					dijit.hideTooltip(e.cellNode);
				} 				
			
			} );

			cpSymbolicValueLines.addChild( grid );
			return cpSymbolicValueLines;
		}
	}
} );