/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dojo/string", "dojox/layout/TableContainer", "dijit/layout/ContentPane", "dijit/form/TextBox", "dijit/form/CheckBox", "dijit/registry", "dijit/tree/ForestStoreModel",
		"dojo/data/ItemFileReadStore", "dojox/grid/LazyTreeGrid", "dijit/layout/BorderContainer",		
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function ( string,  TableContainer, ContentPane, TextBox, CheckBox, registry, ForestStoreModel, ItemFileReadStore, LazyTreeGrid, BorderContainer, PacbaseEditor, typeDict ) {	
	
	return {		
		getComposition : function ( data, type, project, streamId, compentId, fileItemId ) {	
			var dataDescription = data.dataDescription
			var bc = new BorderContainer({
		        style: "height: 90%; width: 100%;",
		        gutters:true
		    });

			var cpComposition = new ContentPane( {
				content : "<strong>" + typeDict[ type ] + " " + data.name + " : " + data.label + "</strong><P>"
			    + "<strong>" + PacbaseEditor.META_COMPOSITION + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
						+ string.substitute( PacbaseEditor.META_COMPOSITION_DESCRIPTION, [
							typeDict[ type ]
						] ) + "</p>",
						style : {
							'overflow' : 'auto',
							'width': '50%',
						},
						region: "left",
						splitter:true
			} );
			
			// prepare lazy tree grid
			var layoutCompositionLines = [
					{
						name : PacbaseEditor.META_COMPOSITION,
						width : "400px",
						field : "_item",
						formatter : function ( item ) {
							return getComposition(item);
						}
					}, 				 
			];
			
			// prepare lazy tree grid
			var data = {
				items : [dataDescription]
			};
			var store = new ItemFileReadStore( {
				data : data,
			} );
			var model = new ForestStoreModel( {
				store : store,
				mayHaveChildren : function ( item ) {
					if ( item.components != undefined )
						return true;
					else if (  item.dataDescription != undefined && item.dataDescription[ 0 ].components != undefined && item.dataDescription[ 0 ].components.length > 0 ) {						
						return true;
					}					
					return false;
				},
				getChildren : function ( item, onComplete, onError ) {
					if ( item.components != undefined ) {
						onComplete(item.components);
					} else if ( item.dataDescription[ 0 ].components != undefined && item.dataDescription[ 0 ].components.length > 0) {
						onComplete(item.dataDescription[ 0 ].components);
					}					
				}				
			} );
		
			var hearderTooltips =  [PacbaseEditor.META_COMPOSITION_TOOL_TIP];					               
			
			var grid = new LazyTreeGrid( {
				treeModel : model,
				structure : layoutCompositionLines,
				height : ( rppCurrentPaneGridHeight - 50 ) + "px",
				autoWidth : true,
				selectable : true,
				canSort : function () {
					return false;
				},
				onRowClick : function ( evt ) {
					var rowData = this.getItem( evt.rowIndex );	
					var cpWidget = registry.byId( streamId + "#" + fileItemId + "CompositionDetail" );
					var childs = [];
					cpWidget.getChildren().forEach(function(child) {
						childs.push(child);
					});
					childs.forEach(function(child) {
						cpWidget.removeChild( child );
					});			
					if (rowData.dataDescription != undefined && rowData.dataDescription != "") {
						var classe = rowData.dataDescription [0] [ "class" ];					
						if (classe == "DataElementDescription" ) {
							cpWidget.addChild( getUse( rowData ) );
							cpWidget.addChild( getDescription( rowData ) ); 
							var classeType = rowData.dataDescription [0].type[0]["class"];
							if (classeType == "ReferenceType") {
								cpWidget.addChild( getDetailDescriptionReferenceType( rowData, project, streamId ) );
							} else {
								cpWidget.addChild( getDetailDescriptionOtherType( rowData, classeType ) );
							}
						}
						if (classe == "DataAggregateDescription" ) {
							cpWidget.addChild( getDataAggregateDescription( rowData ) );
							cpWidget.addChild( getUse( rowData ) );
							if (getCardMin(rowData) != 1 || getCardMax(rowData) != 1 ) {
								cpWidget.addChild(getDescending(rowData));
								cpWidget.addChild( getUseComponentDataAggregate ( rowData ) );	
							}
						}
					}
					else if (rowData.dataDefinition != undefined && rowData.dataDefinition != "") {
						cpWidget.addChild( getUse( rowData ) );
					}
					else {
						var classe = rowData[ "class" ];		
						if (classe == "DataAggregateDescription" ) {
							cpWidget.addChild( getDataAggregateDescription( rowData ) );
						} else {
							cpWidget.addChild( getDescription( rowData ) ); 
							var classeType = rowData.type[0]["class"];
							if (classeType == "ReferenceType") {
								cpWidget.addChild( getDetailDescriptionReferenceType( rowData, project, streamId ) );
							} else {
								cpWidget.addChild( getDetailDescriptionOtherType( rowData, classeType ) );
							}
						}
					}
				},	
			} );
			cpComposition.addChild( grid );	
			bc.addChild(cpComposition);

			var compositionDetail = new ContentPane( {
				region: "center",
				id :  streamId + "#" + fileItemId + "CompositionDetail"
			} );
			
			bc.addChild(compositionDetail);

			return bc;
			
		}	
	}
	
	function getComposition(item, streamId, project) {			
		if ( item.components )
			return item.name;
		if (item.dataDescription != undefined && item.dataDescription != "") {
			var max = item.maximumCardinality;
			if (max == -1) {
				max = "n";
			}											
			var name = getName(item);
			var classe = item.dataDescription [0] [ "class" ];					
			if (classe == "DataElementDescription" ) {
				var type = getItemType(item);
				var ref = " ";
				if (type[ "class" ] == "DateTimeType") {
					ref = " - Date Time ";
				}
				if (type[ "class" ] == "DecimalType") {
					ref = " - Decimal ";
				}
				if (type[ "class" ] == "FloatType") {
					ref = " - Float ";
				}
				if (type[ "class" ] == "IntegerType") {
					ref = " - Integer ";
				}
				if (type[ "class" ] == "StringType") {
					ref = " - String ";
				}
				if (type[ "class" ] == "ByteStringType") {
					ref = " - ByteString ";
				}
				if (type[ "class" ] == "ReferenceType") {
					ref = " - Reference ";
				}
				if (type[ "class" ] == "TimeStampType") {
					ref = " - Time Stamp ";
				}		
				return  "<span class='display_icon dataelementdescription'></span> " + name + ref + " (" + item.minimumCardinality + "-" + max + ")";
			} 
			if (classe == "DataAggregateDescription" ) {
				return  "<span class='display_icon dataaggregatedescription'></span> " + name + " (" + item.minimumCardinality + "-" + max + ")";
			}
		}
		if (item.dataDefinition != undefined && item.dataDefinition != "") {
			var max = item.maximumCardinality;
			if (max == -1) {
				max = "n";
			}											
			var name = item.dataDefinition [0].name[0];
			var classe = item.dataDefinition [0] [ "class" ];
			if (classe == "metadataaggregate" ) {
				return  "<span class='display_icon metadataaggregate'></span> " + name + " (" + item.minimumCardinality + "-" + max + ")";
			}
			if (classe == "dataelement" ) {
				var de = item.dataDefinition [0]
				var format = "";
				if ( item.dataDefinition [0].internalFormat != undefined ) {
					format = item.dataDefinition [0].internalFormat[0];
				}
				var extraLabel = name + " " + format + " (" + item.minimumCardinality + "-" + max + ")";
				if (de != undefined) {
					var dataElement = 
						new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget(PacbaseEditor.DATA_ELEMENT, de, true, streamId, project, extraLabel);
				}				
				return dataElement;				
			}
		}
		var name = getName(item);
		var classe = item[ "class" ];					
		if (classe == "DataElementDescription" ) {
			var type = getItemType(item);
			var ref = " ";
			if (type[ "class" ] == "DateTimeType") {
				ref = " - Date Time ";
			}
			if (type[ "class" ] == "DecimalType") {
				ref = " - Decimal ";
			}
			if (type[ "class" ] == "FloatType") {
				ref = " - Float ";
			}
			if (type[ "class" ] == "IntegerType") {
				ref = " - Integer ";
			}
			if (type[ "class" ] == "StringType") {
				ref = " - String ";
			}
			if (type[ "class" ] == "ByteStringType") {
				ref = " - ByteString ";
			}
			if (type[ "class" ] == "ReferenceType") {
				ref = " - Reference ";
			}
			if (type[ "class" ] == "TimeStampType") {
				ref = " - Time Stamp ";
			}		
			return  "<span class='display_icon dataelementdescription'></span> " + name + ref;
		} 
	}
	
	function getDescending ( item ) {
		var table = new TableContainer( {
			labelWidth : 100,
			valueWidth : 100,
		} );		
		var descending = new CheckBox({
	    	label: PacbaseEditor.SORT_DESCENDING,	    	
	        checked: item.sort[0].descending == "true",
	        disabled : true
	    });		
		table.addChild( descending );
		return table;
	}
		
	function getUse ( item ) {
		var cpUse = new ContentPane( {
			content : "<strong>" + PacbaseEditor.DATACALL_EDIT_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
					+ PacbaseEditor.DATACALL_EDIT_SECTION_DESCRIPTION + "</p>",
		} );
		var tableDetail = new TableContainer( {
			labelWidth : 150,
			valueWidth : 200,
		} );					
		var cardMin = new TextBox( {
			label : PacbaseEditor.DATACALL_MIN_CARDINALITY,			
			value : getCardMin(item),			
			disabled : true
		} );
		var valueControl = new CheckBox({
	    	label: PacbaseEditor.VALUE_CONTROL,	    	
	        checked: item.controlValue == "true",
	        disabled : true
	    });		
		var cardMax = new TextBox( {
			label : PacbaseEditor.DATACALL_MAX_CARDINALITY,
			value : getCardMax(item),		
			disabled : true
		} );		
		tableDetail.addChild( cardMin );
		tableDetail.addChild( valueControl );
		tableDetail.addChild( cardMax );
		cpUse.addChild( tableDetail );
		return cpUse;
	}
	
	function getDataAggregateDescription ( item ) {
		var cpDataAggregateDescription = new ContentPane( {
			content : "<strong>" + PacbaseEditor.META_ENTITY_DAD_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
					+ PacbaseEditor.META_ENTITY_DAD_SECTION_DESCRIPTION + "</p>",
		} );
		var tableDetail = new TableContainer( {
			labelWidth : 150,
			valueWidth : 200,
		} );					
		var name = new TextBox( {
			label : PacbaseEditor.INTERVAL_NAME,					
			value : getName(item),			
			disabled : true
		} );				
		var label = new TextBox( {
			label : PacbaseEditor.INTERVAL_VALUE_LABEL,
			value : getLabel(item),		
			disabled : true
		} );
		tableDetail.addChild( name );
		tableDetail.addChild( label );
		cpDataAggregateDescription.addChild( tableDetail );
		return cpDataAggregateDescription;
	}
	
	function getDescription ( item ) {
		var cpDescription = new ContentPane( {
			content : "<strong>" + PacbaseEditor.META_ENTITY_DED_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
					+ PacbaseEditor.META_ENTITY_DED_SECTION_DESCRIPTION + "</p>",
		} );
		var tableDetail = new TableContainer( {
			labelWidth : 150,
			valueWidth : 200,
		} );				
		var name = new TextBox( {
			label : PacbaseEditor.INTERVAL_NAME,					
			value : getName(item),			
			disabled : true
		} );				
		var label = new TextBox( {
			label : PacbaseEditor.INTERVAL_VALUE_LABEL,
			value : getLabel(item),		
			disabled : true
		} );
		var type = new TextBox( {
			label : PacbaseEditor.SIMPLE_TYPE_SECTION_HEADER,					
			value : getType(item),			
			disabled : true
		} );				
		var initialValue = new TextBox( {
			label : PacbaseEditor.INITIAL_VALUE_SECTION_HEADER,	
			value : getInitialValue(item),
			disabled : true
		} );
		tableDetail.addChild( name );
		tableDetail.addChild( label );
		tableDetail.addChild( type );
		tableDetail.addChild( initialValue );
		cpDescription.addChild( tableDetail );
		return cpDescription;
	}
	
	function getValuesAndIntervals (item) {
		var bcvi = new TableContainer({
	        style: "height: 30%; width: 100%;",
	        cols : 2,
	        customClass : "rppMetaBorderTableContainer"
	    });
		// prepare lazy tree grid
		var layoutValue = [
				{	
					name : PacbaseEditor.ALLOWED_VALUE,				
					width : "400px",
					field : "_item",
					formatter : function ( item ) {
						return getValue(item);
					}
				}, 				 
		];		
		// prepare lazy tree grid		
		var data = {
			items : item.allowedValues			
		};
		var store = new ItemFileReadStore( {
			data : data,
		} );
		var model = new ForestStoreModel( {
			store : store,
			mayHaveChildren : function ( item ) {
				if ( item.allowedValues != undefined && item.allowedValues.length > 0 ) {						
					return true;
				}					
				return false;
			},
			getChildren : function ( item, onComplete, onError ) {
				if ( item.allowedValues != undefined && item.allowedValues.length > 0 ) {	
					onComplete(item.allowedValues);
				}					
			}				
		} );               
		var grid = new LazyTreeGrid( {
			treeModel : model,
			structure : layoutValue,
			height : "400px",			
			autoWidth : true,
			selectable : true,
			canSort : function () {
				return false;
			},			
		} );
		bcvi.addChild( grid );
		
		// prepare lazy tree grid
		var layoutInterval = [
				{					
					name : PacbaseEditor.INTERVAL,		
					width : "400px",
					field : "_item",
					formatter : function ( item ) {
						return getInterval(item);
					}
				}, 				 
		];		
		// prepare lazy tree grid		
		var data = {		
			items : item.allowedIntervals
		};
		var store = new ItemFileReadStore( {
			data : data,
		} );
		var model = new ForestStoreModel( {
			store : store,
			mayHaveChildren : function ( item ) {
				if ( item.allowedIntervals != undefined && item.allowedIntervals.length > 0 ) {			
					return true;
				}					
				return false;
			},
			getChildren : function ( item, onComplete, onError ) {
				if ( item.allowedIntervals != undefined && item.allowedIntervals.length > 0 ) {	
					onComplete(item.allowedIntervals);
				}				
			}				
		} );           
		var grid = new LazyTreeGrid( {
			treeModel : model,
			structure : layoutInterval,
			height : "400px",			
			autoWidth : true,
			selectable : true,
			canSort : function () {
				return false;
			},			
		} );
		bcvi.addChild( grid );
		
		return bcvi
	}	
	
	function getUseComponentDataAggregate ( item ) {
		var bcucdd = new TableContainer({
	        style: "height: 30%; width: 100%;",
	        cols : 3,
	        customClass : "rppMetaBorderTableContainer"
	    });
	
		// prepare lazy tree grid
		var layoutIdent = [
				{	
					name : PacbaseEditor.IDENTIFIER,				
					width : "150px",
					field : "_item",
					formatter : function ( item ) {
						return getIdent(item);
					}
				}, 				 
		];		
		// prepare lazy tree grid		
		var data = {			
			items : item.identifier == undefined || item.identifier[0] == "" ? [] : item.identifier[0].calls
		};
		var store = new ItemFileReadStore( {
			data : data,
		} );
		var model = new ForestStoreModel( {
			store : store,
			mayHaveChildren : function ( item ) {
				if ( item.identifier != undefined && item.identifier.length > 0 ) {						
					return true;
				}					
				return false;
			},
			getChildren : function ( item, onComplete, onError ) {
				if ( item.identifier != undefined && item.identifier.length > 0) {
					onComplete(item.identifier);
				}					
			}				
		} );               
		var grid = new LazyTreeGrid( {
			treeModel : model,
			structure : layoutIdent,
			height : "150px",
			autoWidth : true,
			selectable : true,
			canSort : function () {
				return false;
			},			
		} );
		bcucdd.addChild( grid );
		
		// prepare lazy tree grid
		var layoutUnique = [
				{					
					name : PacbaseEditor.UNIQUE,		
					width : "150px",
					field : "_item",
					formatter : function ( item ) {
						return getUnique(item);
					}
				}, 				 
		];		
		// prepare lazy tree grid		
		var data = {		
			items : item.unique == undefined ||  item.unique[0] == "" ? [] : item.unique[0].comparator[0].calls
		};
		var store = new ItemFileReadStore( {
			data : data,
		} );
		var model = new ForestStoreModel( {
			store : store,
			mayHaveChildren : function ( item ) {
				if ( item.unique != undefined && item.unique.length > 0 ) {						
					return true;
				}					
				return false;
			},
			getChildren : function ( item, onComplete, onError ) {
				if ( item.unique != undefined && item.unique.length > 0) {
					onComplete(item.unique);
				}					
			}				
		} );           
		var grid = new LazyTreeGrid( {
			treeModel : model,
			structure : layoutUnique,
			height : "150px",
			autoWidth : true,
			selectable : true,
			canSort : function () {
				return false;
			},			
		} );
		bcucdd.addChild( grid );	    
		
		var layoutSort = [
		    				{					
		    					name : PacbaseEditor.SORT,	
		    					width : "150px",
		    					field : "_item",
		    					formatter : function ( item ) {
		    						return getSort(item);
		    					}
		    				}, 				 
		    		];		
		// prepare lazy tree grid		
		var data = {			
			items : item.sort == undefined ||  item.sort[0] == "" ? [] : item.sort[0].comparator[0].calls
		};
		var store = new ItemFileReadStore( {
			data : data,
		} );
		var model = new ForestStoreModel( {
			store : store,
			mayHaveChildren : function ( item ) {
				if ( item.sort != undefined && item.sort.length > 0 ) {						
					return true;
				}					
				return false;
			},
			getChildren : function ( item, onComplete, onError ) {
				if ( item.sort != undefined && item.sort.length > 0) {
					onComplete(item.sort);
				}					
			}				
		} );	               
		var grid = new LazyTreeGrid( {
			treeModel : model,
			structure : layoutSort,
			height : "150px",
			autoWidth : true,
			selectable : true,
			canSort : function () {
				return false;
			},			
		} );
		bcucdd.addChild( grid );
		
		return bcucdd
	}
		
	function getDetailDescriptionReferenceType ( item, project, streamId ) {
		
		var types = [PacbaseEditor.PAC_DATA_ELEMENT_TYPE, PacbaseEditor.PAC_DATA_AGGREGATE_TYPE, PacbaseEditor.PAC_DATA_UNIT_TYPE, PacbaseEditor.PAC_META_ENTITY, 
	                 PacbaseEditor.PAC_USER_ENTITY, PacbaseEditor.PAC_LIBRARY_TYPE, PacbaseEditor.PAC_BLOCK_BASE_TYPE, PacbaseEditor.PAC_MACRO_TYPE, PacbaseEditor.PAC_PROGRAM_TYPE,
	                 PacbaseEditor.PAC_TEXT_TYPE, PacbaseEditor.PAC_REPORT_TYPE, PacbaseEditor.PAC_SCREEN_TYPE, PacbaseEditor.PAC_DIALOG_TYPE, PacbaseEditor.PAC_SERVER_TYPE, PacbaseEditor.PAC_DIALOG_SERVER_TYPE, 
	                 PacbaseEditor.PAC_INPUT_AID_TYPE, PacbaseEditor.PAC_VOLUME, PacbaseEditor.PAC_COPYBOOK_TYPE, PacbaseEditor.PAC_ERROR_LABEL_TYPE, 
	                 PacbaseEditor.PAC_COMMUNICATION_MONITOR_TYPE, PacbaseEditor.PAC_DIALOGFOLDER_TYPE, PacbaseEditor.PAC_FOLDER_TYPE, PacbaseEditor.PAC_DIALOGFOLDERVIEW_TYPE, PacbaseEditor.PAC_FOLDERVIEW_TYPE];
		
		var list = [false, false, false, false,
		              false, false, false, false, false,
		              false, false, false, false, false, false,
		              false, false, false, false, 
		              false, false, false, false, false]		
		
		var cpDetailDescriptionReferenceType = new ContentPane( {
			content : PacbaseEditor.DATA_ELEMENT_DESCRIPTION_DETAIL + "</p>",
		} );
		
		var tableInfos = new TableContainer( {
			labelWidth : 150,
			valueWidth : 200,
		} );
		if (getItemType(item).extensions.length > 0 && getItemType(item).extensions[0].dataElement != undefined) {
			var de = getItemType(item).extensions[0].dataElement[0]
			if (de != undefined) {
				var dataElement = 
					new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget(PacbaseEditor.DATA_ELEMENT, de, true, streamId, project);	
				tableInfos.addChild( dataElement );
				var textBoxDe = TextBox( {
					type : "hidden",
					colspan : 1,
				} );
				tableInfos.addChild( textBoxDe );
			}
		}
		var detailInfos = new CheckBox({
	    	label: PacbaseEditor.RELATION_CONSTRAINT,
	        checked: getItemType(item).constraint == "true",
	        disabled : true
	    });	
		tableInfos.addChild( detailInfos );
		var allTypes = new CheckBox({
	    	label: PacbaseEditor.ALL_TYPES,	    	 
	        checked: getItemType(item).referenceConstraints[0] != undefined && getItemType(item).referenceConstraints[0].referenceType[0].value == "ALL",
	        disabled : true
	    });		
		tableInfos.addChild( allTypes );
		cpDetailDescriptionReferenceType.addChild( tableInfos );
				
		var bcdd = new TableContainer({
	        style: "height:30%; width: 100%;",
	        cols : 2,
	        customClass : "rppMetaBorderTableContainer"
	    });
		
		// create a ContentPane as the left pane in the BorderContainer
	    var cpLeft = new ContentPane({
	        style : {				
	        	width : "400px",
	        	height : "400px",
				field : "_item",
			}
	    });
	    
	    var begin = 0
	    var end = types.length
	    while (begin < end) { 	    	
	    	var table = new TableContainer( {
	    		labelWidth : 200,
				
			} );
	    	if (getItemType(item).referenceConstraints[0] == undefined && getItemType(item).extensions[0] == undefined) {
	    		var checkBoxChain = new CheckBox( {		    		
	    			checked : false,	    		
	    			label : types[begin],
	    			disabled : true
	    		} );
	    		table.addChild( checkBoxChain );
	    	} else if (getItemType(item).referenceConstraints[0] != undefined && getItemType(item).referenceConstraints[0].referenceType[0].value == "ALL") {
	    		var checkBoxChain = new CheckBox( {		    		
	    			checked : true,	    		
	    			label : types[begin],
	    			disabled : true
	    		} );
	    		table.addChild( checkBoxChain );	    		
	    	} else {
	    		if (begin == 0) {
	    			var selectionTypes = getSelectionTypes( item, list );
	    		}
	    		var checkBoxChain = new CheckBox( {		    		
	    			checked : list[begin],	    		
	    			label : types[begin],
	    			disabled : true
	    		} );
	    		table.addChild( checkBoxChain );
	    	}
	    	cpLeft.addChild( table );
	    	begin ++
	    }	    
	    bcdd.addChild(cpLeft);	
	    
	    // create a ContentPane as the right pane in the BorderContainer
		var cpRight = new ContentPane({
	        style : {				
	        	width : "400px",
	        	height : "400px",
				field : "_item",
			}
	    });
		var table = new TableContainer( {
    		labelWidth : 200,
			
		} );
		
		var userEntities = getUserEntities (item)
		var begin = 0
		var end = userEntities.length
		while (begin < end) {
			var entity = new ContentPane( {
				content : "<span class='display_icon metaentity'></span> " + userEntities[begin][0],
			} );			
			table.addChild(  entity );
			begin ++
		}
		cpRight.addChild( table );
	    bcdd.addChild(cpRight);
	    
	    cpDetailDescriptionReferenceType.addChild(bcdd)
					
		return cpDetailDescriptionReferenceType;
	}
	
	function getDetailDescriptionOtherType ( item, classeType ) {
		
		var cpDetailDescriptionOtherType = new ContentPane( {			
			content : "<strong>" + PacbaseEditor.DATA_ELEMENT_DESCRIPTION_DETAIL+ "</strong><P>"
			+  PacbaseEditor.TYPE_DEFINITION + "</p>",
		} );
		
		var tableDetail = new TableContainer( {
			labelWidth : 150,
			valueWidth : 200,
		} );
		
		if (classeType == "DateTimeType") {
			var format = new TextBox( {
				label : PacbaseEditor.DATE_FORMAT,			
				value : getFormat(item),			
				disabled : true
			} );	
			tableDetail.addChild( format );
			cpDetailDescriptionOtherType.addChild( tableDetail );
		}
		
		if (classeType == "DecimalType") {
			var minValue = new TextBox( {
				label : PacbaseEditor.MIN_VALUE,			
				value : getMinValue(item),			
				disabled : true
			} );				
			var maxValue = new TextBox( {
				label : PacbaseEditor.MAX_VALUE,
				value : getMaxValue(item),		
				disabled : true
			} );
			tableDetail.addChild( minValue);
			tableDetail.addChild( maxValue );
			cpDetailDescriptionOtherType.addChild( tableDetail );
		}
		
		if (classeType == "FloatType") {
			var capacity = new TextBox( {
				label : PacbaseEditor.FLOAT_CAPACITY,			
				value : getCapacity(item),			
				disabled : true
			} );	
			tableDetail.addChild( capacity );
			cpDetailDescriptionOtherType.addChild( tableDetail );
		}
		
		if (classeType == "IntegerType") {
			var minValue = new TextBox( {
				label : PacbaseEditor.MIN_VALUE,			
				value : getMinValue(item),			
				disabled : true
			} );				
			var maxValue = new TextBox( {
				label : PacbaseEditor.MAX_VALUE,
				value : getMaxValue(item),		
				disabled : true
			} );
			tableDetail.addChild( minValue);
			tableDetail.addChild( maxValue );
			cpDetailDescriptionOtherType.addChild( tableDetail );
		}
		
		if (classeType == "StringType") {
			var lengthMax = new TextBox( {
				label : PacbaseEditor.MAX_LENGTH,			
				value : getLengthMax(item),			
				disabled : true
			} );				
			var caseValue = new TextBox( {
				label : PacbaseEditor.CASE,
				value : getCaseValue(item),		
				disabled : true
			} );
			tableDetail.addChild( lengthMax);
			tableDetail.addChild( caseValue );
			cpDetailDescriptionOtherType.addChild( tableDetail );
		}
		
		if (classeType == "ByteStringType") {
			var lengthMax = new TextBox( {
				label : PacbaseEditor.MAX_LENGTH,			
				value : getLengthMax(item),			
				disabled : true
			} );				
			tableDetail.addChild( lengthMax );
			cpDetailDescriptionOtherType.addChild( tableDetail );
		}
		
		if (classeType == "TimeStampType") {
			var minValue = new TextBox( {
				label : PacbaseEditor.MIN_VALUE,			
				value : getMinValue(item),			
				disabled : true
			} );				
			var maxValue = new TextBox( {
				label : PacbaseEditor.MAX_VALUE,
				value : getMaxValue(item),		
				disabled : true
			} );
			tableDetail.addChild( minValue);
			tableDetail.addChild( maxValue );
			cpDetailDescriptionOtherType.addChild( tableDetail );
		}
		
		if (classeType != "ByteStringType") {
			cpDetailDescriptionOtherType.addChild( getValuesAndIntervals (item.dataDescription [0]) );			
		}
		
		return cpDetailDescriptionOtherType;
	}
	
	function getIdent ( item ) {
		if (item.type == "dataelement") {
				return "<span class='display_icon pacDataElement'></span> " + item.name;
		} else {
			return "<span class='display_icon dataelementdescription'></span> " + item.name;
		}
	}
	
	function getUnique ( item ) {
		if (item.type == "dataelement") {
			return "<span class='display_icon pacDataElement'></span> " + item.name;
		} else {
			return "<span class='display_icon dataelementdescription'></span> " + item.name;
		}
	}
	
	function getSort ( item ) {
		if (item.type == "dataelement") {
			return "<span class='display_icon pacDataElement'></span> " + item.name;
		} else {
			return "<span class='display_icon dataelementdescription'></span> " + item.name;
		}
	}
	
	function getValue(item) {		
		return "<span class='display_icon Value'></span> " + "   " + item.value + "          " + item.label;
	}
	
	function getInterval(item) {
		var minIncluded = "] ";		
		if (item.minValueIncluded == "true") {
			minIncluded = "[ ";	
		}
		var maxIncluded = " [";
		if (item.maxValueIncluded == "true") {
			maxIncluded = " ]";	
		}		
		var minValue = item.minValue[0].value;		
		var maxValue = item.maxValue[0].value;	
		var minLabel = item.minValue[0].label;	
		var maxLabel = item.maxValue[0].label;
		return "<span class='display_icon Interval'></span> " + "   " + minIncluded + minValue + "(" + minLabel + "), " + maxValue  + "(" + maxLabel + ")," + maxIncluded; 
	}
	
	function getFormat(item) {
		return getItemType(item).format;
	}
	
	function getMinValue(item) {
		return getItemType(item).minValue;
	}
	
	function getCapacity(item) {
		return getItemType(item).capacity[0].value;
	}
	
	function getMaxValue(item) {
		return getItemType(item).maxValue;
	}
	
	function getLengthMax(item) {
		return getItemType(item).maxLength;
	}
	
	function getCaseValue(item) {		
		return getItemType(item)["case"][0].value;;
	}	
	
	function getCardMin ( item ) {	
		if (item.minimumCardinality != undefined) 
			return item.minimumCardinality[0];
		return "";
	}
	
	function getCardMax ( item ) {	
		if (item.maximumCardinality != undefined) 
			var max = item.maximumCardinality[0];
			if (max == -1) {
				max = "n";
			}		
			return max;
		return "";
	}
	
	function getName ( item ) {	
		if (item.dataDescription != undefined) 
			return item.dataDescription [0].name[0];
		return item.name;
	}
	
	function getLabel ( item ) {	
		if (item.dataDescription != undefined) 
			return item.dataDescription [0].label[0];
		return item.label;
	}
	
	function getType ( item ) {	
		var classType = getItemType(item)["class"];
		if (classType == "DateTimeType") {
			return PacbaseEditor.DATETIME_TYPE;
		}		
		if (classType == "DecimalType") {
			return PacbaseEditor.DECIMAL_TYPE;
		}		
		if (classType == "FloatType") {
			return PacbaseEditor.FLOAT_TYPE;
		}		
		if (classType == "IntegerType") {
			return PacbaseEditor.INTEGER_TYPE;
		}		
		if (classType == "StringType") {
			return PacbaseEditor.STRING_TYPE;
		}		
		if (classType == "ByteStringType") {
			return PacbaseEditor.BYTESTRING_TYPE;
		}	
		if (classType == "ReferenceType") {
			return PacbaseEditor.REFERENCE_TYPE;
		}	
		if (classType == "TimeStampType") {
			return PacbaseEditor.TIMESTAMP_TYPE;
		}		
		return "";
	}	
	
	function getItemType ( item ) {	
		if ( item.dataDescription != undefined )
			return item.dataDescription [0].type[0]
		else
			return item.type[0]
	}
	
	function getInitialValue ( item ) {	
		if (item.dataDescription != undefined)
			return item.dataDescription [0].initialValue[0].value;
		return "";
	}	
	
	function searchExtensionForReferenceConstraint( type)  {
		begin = 0
		end = type.extensions.length
		while (begin < end) {
			if (type.extensions[begin].referenceConstraints != undefined) {
				return begin
			}
			begin ++
		}
	}
	
	function getSelectionTypes (item, list) {		
		var begin = 0
		var end = getItemType(item).referenceConstraints.length
		while (begin < end) {
			var value = getItemType(item).referenceConstraints[begin].referenceType[0].value;
			if (value == "DATA_ELEMENT") {
	    		list[0] = true
	    	} else if (value == "DATA_AGGREGATE") {
	    		list[1] = true
	    	} else if (value == "DATA_UNIT") {
	    		list[2] = true
	    	} else if (value == "META_ENTITY") {
	    		list[3] = true
	    	} else if (value == "USER_ENTITY") {
	    		list[4] = true
	    	}	   	    	
	    	begin ++
	    }
		var begin = 0
		var index = searchExtensionForReferenceConstraint(getItemType(item))
		if (index != undefined) {
			var end = getItemType(item).extensions[index].referenceConstraints.length		
			while (begin < end) {
				var value = getItemType(item).extensions[index].referenceConstraints[begin].referenceType[0].value;
				if (value == "LIBRARY") {
					list[5] = true
				} else if (value == "BLOCKBASE") {
					list[6] = true
				} else if (value == "MACRO") {
					list[7] = true
				} else if (value == "PROGRAM") {
					list[8] = true
				} else if (value == "TEXT") {
					list[9] = true
				} else if (value == "REPORT") {
					list[10] = true
				} else if (value == "SCREEN") {
					list[11] = true
				} else if (value == "DIALOG") {
					list[12] = true
				} else if (value == "DIALOG_SERVER") {
					list[13] = true
				} else if (value == "SERVER") {
					list[14] = true
				} else if (value == "INPUT_AID") {
					list[15] = true
				} else if (value == "VOLUME") {
					list[16] = true
				} else if (value == "COPYBOOK") {
					list[17] = true
				} else if (value == "ERRORLABEL") {
					list[18] = true
				} else if (value == "COMMUNICATION_MONITOR") {
					list[19] = true
				} else if (value == "DIALOG_FOLDER") {
					list[20] = true
				} else if (value == "FOLDER") {
					list[21] = true
				} else if (value == "DIALOG_FOLDERVIEW") {
					list[22] = true
				} else if (value == "FOLDERVIEW") {
					list[23] = true
				}	    		    		    			    		    		    		    	
				begin ++
			}
		}
		return list;
	}
	
	function getUserEntities (item) {
		var userEntities = []
		var begin = 0
		var end = getItemType(item).referenceConstraints.length
		while (begin < end) {			
			if (getItemType(item).referenceConstraints[begin].metaEntityTypeNames != undefined) {
				var begin1 = 0
				var end1 = getItemType(item).referenceConstraints[begin].metaEntityTypeNames.length
				while (begin1 < end1) {
					userEntities [begin1] = getItemType(item).referenceConstraints[begin].metaEntityTypeNames
					begin1 ++
				}				
			}
			begin ++
		}
		return userEntities;
	}
	
} );