/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
        "dojo/string",
		"dijit/layout/ContentPane",
		"dojox/layout/TableContainer",
		"./generalInfoPane", 
		"./generationParameterPane", 
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict",
		"./keywordsPane",
], function ( string, ContentPane, TableContainer, generalInfoPane, generationParameterPane, PacbaseEditor, typeDict, keywordsPane ) {
	return {
		getOverview : function ( data, type, project, streamId, compentId, fileItemId ) {
			var overview = new ContentPane( {
				title : data.name,
				closable : true,
			} );
			
			// set the general information
			overview.addChild( generalInfoPane.getGeneralInfo( type, project, data.name, data.label ) );
			
			// Types d'entité étendues par les méta entité type 
			var types = [PacbaseEditor.PAC_META_ENTITY, PacbaseEditor.PAC_META_ENTITY_TYPE, PacbaseEditor.PAC_DATA_UNIT_TYPE, PacbaseEditor.PAC_DATA_ELEMENT_TYPE, PacbaseEditor.PAC_DATA_AGGREGATE_TYPE,
						 PacbaseEditor.PAC_USER_ENTITY, PacbaseEditor.PAC_DATA_AGGREGATE, PacbaseEditor.PAC_VOLUME, PacbaseEditor.PAC_REPORT_TYPE, PacbaseEditor.PAC_LIBRARY_TYPE, PacbaseEditor.PAC_MACRO_TYPE, PacbaseEditor.PAC_PROGRAM_TYPE,
						 PacbaseEditor.PAC_BLOCK_BASE_TYPE, PacbaseEditor.PAC_SCREEN_TYPE, PacbaseEditor.PAC_DIALOG_TYPE, PacbaseEditor.PAC_TEXT_TYPE,
						 PacbaseEditor.PAC_DIALOG_SERVER_TYPE, PacbaseEditor.PAC_SERVER_TYPE, PacbaseEditor.PAC_INPUT_AID_TYPE, PacbaseEditor.PAC_ERROR_LABEL_TYPE, PacbaseEditor.PAC_COPYBOOK_TYPE, PacbaseEditor.PAC_DIALOGFOLDER_TYPE,
						 PacbaseEditor.PAC_FOLDER_TYPE, PacbaseEditor.PAC_DIALOG_CM_TYPE, PacbaseEditor.PAC_COMMUNICATION_MONITOR_TYPE, PacbaseEditor.PAC_DIALOGFOLDERVIEW_TYPE, PacbaseEditor.PAC_FOLDERVIEW_TYPE];
			
			var cpReferencedEntities = new ContentPane( {				
				content : string.substitute( PacbaseEditor.META_ENTITY_TYPE_SECTION_DESCRIPTION, [
					typeDict[ type ]
				] ),
			} );
			
			var tc = new TableContainer({
		        style: "height:100%; width: 30%;",
		        cols : 1,
		        customClass : "rppMetaBorderTableContainer"
		    });
			
			 var cp = new ContentPane({
			        style : {				
			        	width : "400px",
			        	height : "400px",
						field : "_item",
						'overflow' : 'auto',
					}
			    });	
			 var table = new TableContainer( {
		    		labelWidth : "100px",		    		
				} );
			 var begin = 0
			 var end = types.length
			 while (begin < end) {	
				 var entity = new ContentPane( {
						content : "<span class='display_icon folder'></span> " + types[begin],
					} );		
				 table.addChild(  entity );	
				 begin ++			    	
			 }
			cp.addChild(table);
			tc.addChild(cp);
			cpReferencedEntities.addChild(tc)
			    
			overview.addChild(cpReferencedEntities);
					
			// set generation parameters
			if ( data.generationParameter != undefined )
				overview.addChild( generationParameterPane.getOverviewLibraryCall( data.generationParameter, type, project, streamId ) );

			// set the keywords
			overview.addChild( keywordsPane.getOverviewKeywords( data.keywords, type ) );

			return overview;
		}
	};
} );