/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2016, 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dojo/string", "dojox/layout/TableContainer", "dijit/layout/ContentPane", "dijit/form/TextBox", "dijit/tree/ForestStoreModel",
		"dojo/data/ItemFileReadStore", "dojox/grid/LazyTreeGrid", "dijit/form/Button", "dijit/Dialog", "dijit/Tooltip", "./gLinesPane",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function ( string, TableContainer, ContentPane, TextBox, ForestStoreModel, ItemFileReadStore, LazyTreeGrid, Button, Dialog, Tooltip, gLinesPane, 
				PacbaseEditor, typeDict ) {	

	
	function getValue ( value ) {		
		return value.toString().replace("_","").replace("None","");
	}
	
	function getCodeInProgram( item, store, project, streamId  ) {

		if (item.commonDescription != undefined) {
			// Memorize code in program when item is CDLine
			var code = item.commonDescription[0].codeInProgram;
			var ioMode = getValue(item.IOMode[0].value);	
			var org = getValue(item.commonDescription[0].organization[0].value);
			if (org == "W") 
				return "<span class='display_icon pacCDLineDataStructure'></span> " + code;			
			if (ioMode == "I") 
				return "<span class='display_icon pacCDLineInput'></span> " + code;
			if (ioMode == "O") 
				return "<span class='display_icon pacCDLineOutput'></span> " + code;
			return 	"<span class='display_icon pacCDLineDataStructure'></span> " + code;
		}
		var rootCode = null;
		store.fetch({
			  query: { treeId: item.treeId.substring(0,1) },
			  onComplete: function(items, request){
				  rootCode = items[0].commonDescription[0].codeInProgram;
			  }
		});

		// item is datastructure 
		if (item.dataStructure != undefined)
			return 	"<span class='display_icon dataunit'></span> " + rootCode;
		// item is segmentCalls
		if (item.segment != undefined) {
			var code = item.codeInProgram;
			if (code == "") 
				code = item.segment[0].name.toString().substring(2);
			return 	"<span class='display_icon dataaggregate'></span> " + rootCode+code;
		} 
		// item is reportCalls
		if (item.report != undefined) {
			// affichage selection code état bib = code état dans pgm
			var code = item.suffixInProgram;
			if (code == "")	
				code = item.report[0].name.toString().substring(2);
			return 	"<span class='display_icon pacreport'></span> " + rootCode+code;
		}
		return "";
	}
	
	function getSDCode( item, store, project, streamId  ) {
		// item is dataStructureCalls
		if (item.dataStructure != undefined) {
			return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("",item.dataStructure[0], true, streamId, project, item.dataStructure[0].name, "",false);			

		}
		//item is segmentCalls
		if (item.segment != undefined) {
			return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", item.segment[0], true, streamId, project, item.segment[0].name, "",false);			
		} 
		//item is reportCalls
		if (item.report != undefined) {
			return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", item.report[0], true, streamId, project, item.report[0].name, "",false);			
		} 		
		return "";
	}
	
	function getExternalName( item ) {	
		if(item.externalName != undefined) 
			return item.externalName;
		return "";
	}
	
	function getOamou( item ) {		
		if (item.commonDescription != undefined) {
			var res1 = getValue(item.commonDescription[0].organization[0].value);
			var res2 = getValue(item.accessMode[0].value);
			var res3 = getValue(item.commonDescription[0].blockMode[0].value);
			var res4 = getValue(item.IOMode[0].value);
			var res5 = getValue(item.unitType[0].value);							
			return res1+res2+res3+res4+res5;
		}
		return "";
	}
	
	function getBlockFactor( item ) {	
		if(item.blockFactor != undefined) 
			return item.blockFactor;
		return "";
	}
	
	function getBlockType( item ) {	
		if (item.blockType != undefined && item.blockType != "") 
			return  getValue(item.blockType[0].value);
		return "";
	}
	
	function getBreakLevel( item ) {	
		if (item.breakLevel != undefined && item.breakLevel != "") 
			return item.breakLevel.toString().replace("0","");
		return "";
	}
	
	function getSyncLevel( item ) {	
		if (item.syncLevel != undefined && item.syncLevel != "") 
			return item.syncLevel.toString().replace("0","");
		return "";
	}
	
	function getUsage( item ) {	
		if (item.usage != undefined && item.usage != "")  
			return  getValue(item.usage[0].value);
		return "";
	}
	
	function getResultDs( item ) {	
		if (item.resultDataStructureCode != undefined) 
			return item.resultDataStructureCode;
		return "";
	}
	
	function getSourceDs( item ) {	
		if (item.sourceDataStructureCode != undefined) 
			return  item.sourceDataStructureCode;
		return "";
	}
	
	function getTxBreakLevel( item ) {	
		if (item.transactionBreakLevel != undefined) 
			return  item.transactionBreakLevel.toString().replace("0","");
		return "";
	}
	
	function getPhysicalUnitType( item ) {	
		if (item.physicalUnitType != undefined && item.physicalUnitType != "") 
			return getValue(item.physicalUnitType);
		return "";
	}
	
	function getPhysicalUnitTypeComplement( item ) {	
		if (item.physicalUnitTypeComplement != undefined && item.physicalUnitTypeComplement != "")  
			return getValue(item.physicalUnitTypeComplement[0].value);
		return "";
	}
	
	function getSelection( item ) {
		if (item.sortKeys != undefined) 
			return item.sortKeys;
		// item is segmentCalls or reportCalls
		if (item.segment != undefined || item.report != undefined) {
			var res = item.codeInProgram;
			var code= "";
			if (item.segment != undefined) {
				// item is segmentCalls
				// affichage selection code segment bib = code segment dans pgm
				var res = item.codeInProgram;
				var code = item.segment[0].name.toString().substring(2);
			}
			else {
				if (item.report != undefined) {
					// affichage selection code état bib = code état dans pgm
					var res = item.suffixInProgram;
					var code = item.report[0].name.toString().substring(2);
				}
			}
			if (res != "") {
				if (res != code) {
					res = code + "=" + res;
				}
				else 
					res = "";
			}
			return res;
		}
		return "";
	}
	
	function getFormatType( item ) {
		if (item.commonDescription != undefined  && item.commonDescription[0].formatType != "") { 
			var res = item.commonDescription[0].formatType[0].value;
			return getValue(res);
		}
		return "";
	}
	
	function getReservedCodes( item ) {
		if (item.commonDescription != undefined && item.commonDescription[0].subScheme != "") { 
			var res = item.commonDescription[0].subScheme[0].value;
			return res != undefined ? getValue(res) : "";
			}
		return "";
	}
	
	function getGeneratedDescriptionType( item ) {
		if (item.commonDescription != undefined && item.commonDescription[0].generatedDescriptionType != "") { 
			var res = item.commonDescription[0].generatedDescriptionType[0].value;
			return getValue(res);
		}
		return "";
	}
	
	function getCobolRecordLevel( item ) {
		if (item.commonDescription != undefined && item.commonDescription[0].cobolRecordLevel != "") {
			var res = item.commonDescription[0].cobolRecordLevel[0].value;
			return getValue(res);
		}
		return "";
	}
	
	function getCobolPlacement( item ) {
		//item is segmentCalls
		if (item.segment != undefined) {
			var res = item.cobolPosition[0];
			return res != undefined ? res.toString().replace("00","") : "";
		}
		return "";
	}
	
	
	function getFileStatus( item ) {
		if(item.fileStatus != undefined) 
			return item.fileStatus;
		return "";
	}
	
	function getAccessKeyDataElementCode( item ) {
		if(item.accessKeyDataElementCode != undefined) 
			return item.accessKeyDataElementCode;
		return "";
	}
	
	function getRecordTypeDataElementCode( item ) {
		if (item.recordTypeDataElementCode != undefined) 
			return res = item.recordTypeDataElementCode;
		return "";
	}
	
	
	return {
		
		getDetailCDLines : function ( data, type, project, streamId ) 
		{	
			var CDLines = data.CDLines
			var cpDetailCDLines = new ContentPane( {
				content : "<strong>" + typeDict[ type ] + " " + data.name + " : " + data.label + "</strong><P>"
			    + "<strong>" + PacbaseEditor.CDLINE_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
						+ string.substitute( PacbaseEditor.CDLINE_TABLE_SECTION_DESCRIPTION, [
							typeDict[ type ]
						] ) + "</p>",
						style : {
							'overflow' : 'auto',
						},
			} );
			// prepare lazy tree grid
			var layoutCDLines = [
					{
						name : PacbaseEditor.CDLINE_COLUMN_CODEINPGM,
						width : "120px",
						field : "_item",
						formatter : function ( item ) {
							return getCodeInProgram(item, this.grid.store, project, streamId);
						}
					}, {						
						name : PacbaseEditor.CDLINE_COLUMN_SD,
						width : "60px",
						field : "_item",
						formatter : function ( item ) {
							return getSDCode(item); 
						} 
					}, {						
						name : PacbaseEditor.CDLINE_COLUMN_EXTERNAL_NAME,
						width : "60px",
						field : "_item",
						formatter : function ( item ) {
							return getExternalName(item);
						} 
					}, {						
						name : PacbaseEditor.CDLINE_COLUMN_OAMOU,
						width : "50px",
						field : "_item",
						formatter : function ( item ) {
							return getOamou(item);
						} 
					}, {						
						name : PacbaseEditor.CDLINE_COLUMN_BLOCKFACTOR,
						width : "50px",
						field : "_item",
						formatter : function ( item ) {							
							return getBlockFactor(item);
						} 
					}, {						
						name : PacbaseEditor.CDLINE_COLUMN_BLOCKTYPE,
						width : "10px",
						field : "_item",
						formatter : function ( item ) {
							return getBlockType(item);
						}
					}, {						
						name : PacbaseEditor.CDLINE_COLUMN_BREAKLEVEL,
						width : "10px",
						field : "_item",
						formatter : function ( item ) {	
							return getBreakLevel(item);
						}
					}, {						
						name : PacbaseEditor.CDLINE_COLUMN_SYNCLEVEL,
						width : "10px",
						field : "_item",
						formatter : function ( item ) {								
							return getSyncLevel(item);
						}
					}, {						
						name : PacbaseEditor.CDLINE_COLUMN_USAGE,
						width : "10px",
						field : "_item",
						formatter : function ( item ) {
							return getUsage(item);
						}
					}, {						
						name : PacbaseEditor.CDLINE_COLUMN_RESULT_DS_CODE,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {
							return getResultDs(item);
						}
					}, {						
						name : PacbaseEditor.CDLINE_COLUMN_SOURCE_DS_CODE,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {
							return getSourceDs(item);
						}
					}, {						
						name : PacbaseEditor.CDLINE_COLUMN_TRANSACTION_BREAK_LEVEL,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {
							return getTxBreakLevel(item);
						}
					}, {						
						name : PacbaseEditor.CDLINE_COLUMN_PHYSICAL_UNIT,
						width : "40px",
						field : "_item",
						formatter : function ( item ) {
							return getPhysicalUnitType(item);
						}							
					}, {						
						name : PacbaseEditor.CDLINE_COLUMN_UNITCOMPL,
						width : "10px",
						field : "_item",
						formatter : function ( item ) {
							return getPhysicalUnitTypeComplement(item);
						}
					}, {						
						name : PacbaseEditor.CDLINE_COLUMN_SELECTION,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getSelection(item);
						}
					}, {
						name : PacbaseEditor.CDLINE_COLUMN_FORMAT,
						width : "10px",
						field : "_item",
						formatter : function ( item ) {
							return getFormatType(item);
						}
					}, {
						name : PacbaseEditor.CDLINE_COLUMN_RESERVEDCODES,
						width : "10px",
						field : "_item",
						formatter : function ( item ) {
							return getReservedCodes(item);
						}
					}, {
						name : PacbaseEditor.CDLINE_COLUMN_DESCRIPTION_TYPE,
						width : "10px",
						field : "_item",
						formatter : function ( item ) {
							return getGeneratedDescriptionType(item);
						}
					}, {
						name : PacbaseEditor.CDLINE_COLUMN_RECORD_LEVEL,
						width : "10px",
						field : "_item",
						formatter : function ( item ) {	
							return getCobolRecordLevel(item);
						}
					}, {
						name : PacbaseEditor.CDLINE_COLUMN_PLACEMENT,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {	
							return getCobolPlacement(item);
						}
					}, {
						name : PacbaseEditor.CDLINE_COLUMN_STATUSFIELD,
						width : "80px",
						field : "_item",
						formatter : function ( item ) {
							return getFileStatus(item);
						}
					}, {
						name : PacbaseEditor.CDLINE_COLUMN_ACCESS_KEY_DE,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {	
							return getAccessKeyDataElementCode(item);
						}
					}, {
						name : PacbaseEditor.CDLINE_COLUMN_RECORD_TYPE_DE,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getRecordTypeDataElementCode(item);
						}
					}
				 
			];			
			
			// prepare lazy tree grid
			var data = {
				items : CDLines
			};
			var store = new ItemFileReadStore( {
				data : data,
			} );
			
			var index = 0;
			data.items.forEach(function(item) {
				item.treeId = index++;
			});
			
			
			var model = new ForestStoreModel( {
				store : store,
				mayHaveChildren : function ( item ) {
					if ( item.dataStructureCalls != undefined && item.dataStructureCalls.length > 0 ) {						
						return true;
					}
					if (item.segmentCalls != undefined && item.segmentCalls.length > 0) {
						return true;
					}		
					if (item.reportCalls != undefined && item.reportCalls.length > 0) {
						return true;
					}
					return false;
				},
				getChildren : function ( item, onComplete, onError ) {
					var childs = null;
					if ( item.dataStructureCalls != undefined && item.dataStructureCalls.length > 0) {
						childs = item.dataStructureCalls;
					}
					if (item.segmentCalls != undefined && item.segmentCalls.length > 0) {
						childs = item.segmentCalls;
					}
					if (item.reportCalls != undefined && item.reportCalls.length > 0) {
						childs = item.reportCalls;
					}		
					if ( childs != null ) {
						index = 0;
						childs.forEach(function(child) {
							child.treeId = item.treeId + index;
						});
						onComplete(childs);
					}
				}
			} );
		
			var hearderTooltips =  [PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_CODEINPGM, 
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_SD,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_EXTERNAL_NAME,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_OAMOU,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_BLOCKFACTOR,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_BLOCKTYPE,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_BREAKLEVEL,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_SYNCLEVEL,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_USAGE,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_RESULT_DS_CODE,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_SOURCE_DS_CODE,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_TRANSACTION_BREAK_LEVEL,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_PHYSICAL_UNIT,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_UNITCOMPL,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_SELECTION,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_FORMAT,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_RESERVEDCODES,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_DESCRIPTION_TYPE,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_RECORD_LEVEL,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_PLACEMENT,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_STATUSFIELD,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_ACCESS_KEY_DE,
				      				PacbaseEditor.CDLINE_COLUMN_TOOL_TIP_RECORD_TYPE_DE];
			
			var grid = new LazyTreeGrid( {
				treeModel : model,
				structure : layoutCDLines,
				height : ( rppCurrentPaneGridHeight - 70 ) + "px",
				autoWidth : true,
				selectable : true,
				
				canSort : function () {
					return false;
				},
				onHeaderCellMouseOver  : function (e) {
					dijit.showTooltip(hearderTooltips[e.cellIndex], e.cellNode);
				},
				onHeaderCellMouseOut : function (e) {
					dijit.hideTooltip(e.cellNode);
				} 				
			
			} );
			cpDetailCDLines.addChild( grid );
			return cpDetailCDLines;
		}
	}
} );