/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dojo/string", "dojox/layout/TableContainer", "dijit/layout/ContentPane", "dijit/form/TextBox", "dijit/tree/ForestStoreModel",
		"dojo/data/ItemFileReadStore","dojox/grid/LazyTreeGrid", "dojo/data/ItemFileWriteStore", "dojox/grid/DataGrid", "dijit/form/Button", "dijit/Dialog", "dijit/Tooltip", "./gLinesPane", 
		"dijit/TooltipDialog", "dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function ( string, TableContainer, ContentPane, TextBox, ForestStoreModel, ItemFileReadStore, LazyTreeGrid, ItemFileWriteStore, DataGrid, Button, Dialog, Tooltip, gLinesPane, TooltipDialog,
				PacbaseEditor, typeDict ) {	
	
	
	return {
		
		getDetailCELines : function ( data, type, project, streamId ) 
		{	
			var CELines = data.CELines
			var detailCELines = new ContentPane( {
				content :  "<strong>" + typeDict[ type ] + " " + data.name + " : " + data.label + "</strong><P>"
			    + "<strong>" + PacbaseEditor.REPORT_CELINE_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
						+ string.substitute( PacbaseEditor.REPORT_CELINE_TABLE_SECTION_DESCRIPTION, [
							typeDict[ type ]
						] ) + "</p>",
						style : {
							'overflow' : 'auto',
						},
			} );
		
			// prepare lazy tree grid
			var layoutCELines = [
					{
						name : PacbaseEditor.REPORT_CELINE_COLUMN_STRUCTURE,
						width : "80px",
						field : "_item",
						formatter : function ( item ) {
							return getStructureID(item);
						}
					}, {						
						name : PacbaseEditor.REPORT_CELINE_COLUMN_ELEMENT,
						width : "80px",
						field : "_item",
						formatter : function ( item ) {
							return getElement(item, this.grid.store, project, streamId); 
						} 
					}, {						
						name : PacbaseEditor.REPORT_CELINE_COLUMN_COLUMN_DEB,
						width : "50px",
						field : "_item",
						formatter : function ( item ) {
							return getColumn(item);
						} 
					}, {						
						name : PacbaseEditor.REPORT_CELINE_COLUMN_COND_SUIT,
						width : "50px",
						field : "_item",
						formatter : function ( item ) {
							return getCondSuit(item);
						} 
					}, {						
						name : PacbaseEditor.REPORT_CELINE_COLUMN_OPERATION,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {							
							return getOperation(item);
						} 
					}, {						
						name : PacbaseEditor.REPORT_CELINE_COLUMN_WSPref,
						width : "50px",
						field : "_item",
						formatter : function ( item ) {
							return getWSPref(item);
						}
					}, {						
						name : PacbaseEditor.REPORT_CELINE_COLUMN_SOURCE,
						width : "80px",
						field : "_item",
						formatter : function ( item ) {	
							return getSource(item);
						}
					}, {						
						name : PacbaseEditor.REPORT_CELINE_COLUMN_COND,
						width : "120px",
						field : "_item",
						formatter : function ( item ) {	
							return getCond(item);
						}
					}, {						
						name : PacbaseEditor.REPORT_CELINE_COLUMN_UNDEFINED_DE,
						width : "80px",
						field : "_item",
						formatter : function ( item ) {	
							return getUndefinedDE(item);
						}
					}, {						
						name : PacbaseEditor.REPORT_CELINE_COLUMN_FORMAT,
						width : "80px",
						field : "_item",
						formatter : function ( item ) {	
							return getFormat(item);
						}
					}, {						
						name : PacbaseEditor.REPORT_CELINE_COLUMN_BLANK_WHEN_ZERO,
						width : "50px",
						field : "_item",
						formatter : function ( item ) {	
							return getBWZ(item);
						}
					},
				
				 
			];			
			
			// prepare lazy tree grid
			var data = {
				items : CELines
			};
			var store = new ItemFileReadStore( {
				data : data,
			} );
			var model = new ForestStoreModel( {
				store : store,
				mayHaveChildren : function ( item ) {
					if ( item.targets != undefined && item.targets.length > 0 ) {						
						return true;
					}
					if (item.sourceLines != undefined && item.sourceLines.length > 0) {
						return true;
					}	
					
					
					return false;
				},
				getChildren : function ( item, onComplete, onError ) {
					if ( item.targets != undefined && item.targets.length > 0) {
						onComplete(item.targets);
					}
					if ( item.sourceLines != undefined && item.sourceLines.length > 0) {
						onComplete(item.sourceLines);
					}
					
						 
				}
			} );
		
			var hearderTooltips =  [PacbaseEditor.REPORT_CELINE_COLUMN_TOOL_TIP_STRUCTURE,
					                PacbaseEditor.REPORT_CELINE_COLUMN_TOOL_TIP_ELEMENT,
					                PacbaseEditor.REPORT_CELINE_COLUMN_TOOL_TIP_COLUMN_DEB,
					                PacbaseEditor.REPORT_CELINE_COLUMN_TOOL_TIP_COND_SUIT,
					                PacbaseEditor.REPORT_CELINE_COLUMN_TOOL_TIP_OPERATION,
					                PacbaseEditor.REPORT_CELINE_COLUMN_TOOL_TIP_WSPref,
					                PacbaseEditor.REPORT_CELINE_COLUMN_TOOL_TIP_SOURCE,
					                PacbaseEditor.REPORT_CELINE_COLUMN_TOOL_TIP_COND,
					                PacbaseEditor.REPORT_CELINE_COLUMN_TOOL_TIP_UNDEFINED_DE,
					                PacbaseEditor.REPORT_CELINE_COLUMN_TOOL_TIP_FORMAT,
					                PacbaseEditor.REPORT_CELINE_COLUMN_TOOL_TIP_BLANK_WHEN_ZERO];
			
			var grid = new LazyTreeGrid( {
				treeModel : model,
				structure : layoutCELines,
				height : ( rppCurrentPaneGridHeight - 70 ) + "px",
				autoWidth : true,
				selectable : true,
				canSort : function () {
					return false;
				},
				onHeaderCellMouseOver  : function (e) {
					dijit.showTooltip(hearderTooltips[e.cellIndex], e.cellNode);
				},
				onHeaderCellMouseOut : function (e) {
					dijit.hideTooltip(e.cellNode);
				} 					
			
			} );
			detailCELines.addChild( grid );
			return detailCELines;
		}
	}
	function getStructureID(item) {	
		if(item.structureID != undefined && item.structureID != "") {
			return "<span class='display_icon pacStructure'></span> " + item.structureID;	
		}
		if ( item[ "class" ] == "PacTarget" ){
			if(item.dataDefinition[0]!= undefined && item.dataDefinition[0] != "") {
				return "<span class='display_icon pacDataElement'></span> "; 
			}else{
				return "<span class='display_icon pacDataElementDescription'></span> "; 
			}
		}
		if ( item[ "class" ] == "PacSourceLine"  ){
			return "<span class='display_icon pacTarget'></span> "; 
		}
		return "";
	}
	
	
	function getElement( item, store, project, streamId ) {	
		if ( item[ "class" ] == "PacTarget" ){
			if(item.dataDefinition[0]!= undefined && item.dataDefinition[0] != "") {
				// Memorize name for data elt
				store.code = item.dataDefinition[0].name;
				//pour débranchement radical entity
				return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", item.dataDefinition[0], true, streamId, project, store.code, "", false );		
				//return item.dataDefinition[0].name;
			}else{
				if (item.dataDescription[0]!= undefined && item.dataDescription[0] != "") {
					return item.dataDescription[0].name;
				}
			}
		}
		return "";
	}
	function getColumn( item ) {	
		if ( item[ "class" ] == "PacTarget" ){
			if(item.column != undefined && item.column != "0") {
				return item.column;
			}
		}
		return "";
	}
	function getCondSuit( item ) {	
		if ( item[ "class" ] == "PacSourceLine"  ){
			if(item.continued != undefined && item.continued != "") {
				return item.continued;
			}
		}
		return "";
	}
	function getOperation( item ) {	
		if ( item[ "class" ] == "PacSourceLine" ){
			if(item.operation != undefined && item.operation != "") {
				return item.operation;
			}
		}
		return "";
	}
	function getWSPref( item ) {	
		if ( item[ "class" ] == "PacSourceLine"  ){
			if(item.wsPrefix!= undefined && item.wsPrefix != "") {
				return item.wsPrefix;
			}
		}
		return "";
	}
	function getSource( item ) {	
		if ( item[ "class" ] == "PacSourceLine" ){
			if(item.source!= undefined && item.source != "") {
				return item.source;
			}
		}
		return "";
	}
	function getCond( item ) {	
		if ( item[ "class" ] == "PacSourceLine"  ){
			if(item.condition!= undefined && item.condition != "") {
				return item.condition;
			}
		}
		return "";
	}
	function getUndefinedDE( item ) {	
		//rubrique non définie au dictionnaire
		if ( item[ "class" ] == "PacTarget" ){
			if (item.dataDescription[0]!= undefined && item.dataDescription[0] != "") {
				return item.dataDescription[0].name;
			}
		}
		return "";
	}
	function getFormat( item ) {	
		//Pour rubrique non définie au dictionnaire
		if ( item[ "class" ] == "PacTarget" ){
			if (item.dataDescription[0]!= undefined && item.dataDescription[0] != "") {
				return item.dataDescription[0].extensions[0].outputFormat;
			}
			
		}
		return "";
	}
	
	function getBWZ( item ) {	
		//Pour rubrique non définie au dictionnaire
		if ( item[ "class" ] == "PacTarget" ){
			if (item.dataDescription[0]!= undefined && item.dataDescription[0] != "") {
				if( item.dataDescription[0].extensions[0].blkWhenZero = "_TRUE"){
					return "Z";
				}else{
					return "";
				}
			}
		}
		return "";
	}
	
	
} );