/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dojo/string", "dojox/layout/TableContainer", "dijit/layout/ContentPane", "dijit/form/TextBox", "dijit/tree/ForestStoreModel",
		"dojo/data/ItemFileReadStore","dojox/grid/LazyTreeGrid", "dojo/data/ItemFileWriteStore", "dojox/grid/DataGrid", "dijit/form/Button", "dijit/Dialog", "dijit/Tooltip", "./gLinesPane", 
		"dijit/TooltipDialog", "dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function ( string, TableContainer, ContentPane, TextBox, ForestStoreModel, ItemFileReadStore, LazyTreeGrid, ItemFileWriteStore, DataGrid, Button, Dialog, Tooltip, gLinesPane, TooltipDialog,
				PacbaseEditor, typeDict ) {	
	
	
	return {
		
		getDetailDLines : function ( data, type, project, streamId ) 
		{	
			var DLines = data.DLines
			var detailDLines = new ContentPane( {
				content :  "<strong>" + typeDict[ type ] + " " + data.name + " : " + data.label + "</strong><P>"
			    + "<strong>" + PacbaseEditor.REPORT_DLINE_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
						+ string.substitute( PacbaseEditor.REPORT_DLINE_TABLE_SECTION_DESCRIPTION, [
							typeDict[ type ]
						] ) + "</p>",
						style : {
							'overflow' : 'auto',
						},
			} );
		
			// prepare lazy tree grid
			var layoutCDLines = [
					{
						name : PacbaseEditor.REPORT_DLINE_COLUMN_NAME,
						width : "80px",
						field : "_item",
						formatter : function ( item ) {
							return getCategoryID(item);
						}
					}, {						
						name : PacbaseEditor.REPORT_DLINE_COLUMN_CATEGORY_TYPE,
						width : "50px",
						field : "_item",
						formatter : function ( item ) {
							return getCategoryTyp(item); 
						} 
					}, {						
						name : PacbaseEditor.REPORT_DLINE_COLUMN_TOTALIZATION_TYPE,
						width : "50px",
						field : "_item",
						formatter : function ( item ) {
							return getTotalizationTyp(item);
						} 
					}, {						
						name : PacbaseEditor.REPORT_DLINE_COLUMN_FOREIGN_TOTAL,
						width : "50px",
						field : "_item",
						formatter : function ( item ) {
							return getTotalizationState(item);
						} 
					}, {						
						name : PacbaseEditor.REPORT_DLINE_COLUMN_REPETITION,
						width : "50px",
						field : "_item",
						formatter : function ( item ) {							
							return getRepeat(item);
						} 
					}, {						
						name : PacbaseEditor.REPORT_DLINE_COLUMN_CALLED_STRUCTURE,
						width : "50px",
						field : "_item",
						formatter : function ( item ) {
							return getStructureID(item);
						}
					}, {						
						name : PacbaseEditor.REPORT_DLINE_COLUMN_CALLED_LABEL,
						width : "50px",
						field : "_item",
						formatter : function ( item ) {	
							return getLabelID(item);
						}
					}, {						
						name : PacbaseEditor.REPORT_DLINE_COLUMN_SKIP,
						width : "50px",
						field : "_item",
						formatter : function ( item ) {								
							return getSkip(item);
						}
					}, {						
						name : PacbaseEditor.REPORT_DLINE_COLUMN_SKIP_TYPE,
						width : "50px",
						field : "_item",
						formatter : function ( item ) {
							return getSkipTyp(item);
						}
					}, {						
						name : PacbaseEditor.REPORT_DLINE_COLUMN_FOSF,
						width : "50px",
						field : "_item",
						formatter : function ( item ) {
							return getFOSF(item);
						}
					}, {						
						name : PacbaseEditor.REPORT_DLINE_COLUMN_COMMENT,
						width : "90px",
						field : "_item",
						formatter : function ( item ) {
							return getComment(item);
						}
					}, {						
						name : PacbaseEditor.REPORT_DLINE_COLUMN_CONDITION,
						width : "150px",
						field : "_item",
						formatter : function ( item ) {
							return getCondition(item);
						}
					}, 
				 
			];			
			
			// prepare lazy tree grid
			var data = {
				items : DLines
			};
			var store = new ItemFileReadStore( {
				data : data,
			} );
			var model = new ForestStoreModel( {
				store : store,
				mayHaveChildren : function ( item ) {
					if ( item.editionLines != undefined && item.editionLines.length > 0 ) {						
						return true;
					}
					
					return false;
				},
				getChildren : function ( item, onComplete, onError ) {
					if ( item.editionLines != undefined && item.editionLines.length > 0) {
						onComplete(item.editionLines);
					}
						 
				}
			} );
		
			var hearderTooltips =  [PacbaseEditor.REPORT_DLINE_TOOL_TIP_COLUMN_NAME,
					                PacbaseEditor.REPORT_DLINE_COLUMN_TOOL_TIP_CATEGORY_TYPE,
					                PacbaseEditor.REPORT_DLINE_COLUMN_TOOL_TIP_TOTALIZATION_TYPE,
					                PacbaseEditor.REPORT_DLINE_COLUMN_TOOL_TIP_FOREIGN_TOTAL,
					                PacbaseEditor.REPORT_DLINE_COLUMN_TOOL_TIP_REPETITION,
					                PacbaseEditor.REPORT_DLINE_COLUMN_TOOL_TIP_CALLED_STRUCTURE,
					                PacbaseEditor.REPORT_DLINE_COLUMN_TOOL_TIP_CALLED_LABEL,
					                PacbaseEditor.REPORT_DLINE_COLUMN_TOOL_TIP_SKIP,
					                PacbaseEditor.REPORT_DLINE_COLUMN_TOOL_TIP_SKIP_TYPE,
					                PacbaseEditor.REPORT_DLINE_COLUMN_TOOL_TIP_FOSF,
					                PacbaseEditor.REPORT_DLINE_COLUMN_TOOL_TIP_COMMENT,
					                PacbaseEditor.REPORT_DLINE_COLUMN_TOOL_TIP_CONDITION ];
			
			var grid = new LazyTreeGrid( {
				treeModel : model,
				structure : layoutCDLines,
				height : ( rppCurrentPaneGridHeight - 70 ) + "px",
				autoWidth : true,
				selectable : true,				
				canSort : function () {
					return false;
				},
				onHeaderCellMouseOver  : function (e) {
					dijit.showTooltip(hearderTooltips[e.cellIndex], e.cellNode);
				},
				onHeaderCellMouseOut : function (e) {
					dijit.hideTooltip(e.cellNode);
				} 							
			
			} );
			detailDLines.addChild( grid );
			return detailDLines;
		}
	}
	function getCategoryID( item  ) {	
		if(item.categoryID != undefined && item.categoryID != "") {
			return "<span class='display_icon category'></span> " + item.categoryID;
		}
		return "<span class='display_icon category'></span> ";
	}
	
	function getCategoryTyp( item ) {	
		if(item.categoryType != undefined ){
			//categoryType is PacCategoryTypeValues
			var res = item.categoryType[0].value;
			return getValue(res);
		}
		return "";
	}
	function getTotalizationTyp( item ) {	
		/*if (item.editionLines != undefined  && item.editionLines[0].totalisationType[0] != "") { 
			//totalisationType is PacEditionLineTotalisationTypeValues
			var res = item.editionLines[0].totalisationType[0].value;
			return getValue(res);
		}*/
		if ( item[ "class" ] == "PacEditionLine" ){
			if(item.totalisationType[0]!= undefined && item.totalisationType[0] != "0") {
				var res = item.totalisationType[0].value;
				return getValue(res);
			}
		}
		
		return "";
	}
	
	function getTotalizationState( item ) {	
		/*if (item.editionLines != undefined  && item.editionLines[0].foreignTotalStructure != "") { 
			return item.editionLines[0].foreignTotalStructure;
		}*/
		if ( item[ "class" ] == "PacEditionLine" ){
			if(item.foreignTotalStructure != undefined && item.foreignTotalStructure != "0") {
				return item.foreignTotalStructure;
			}
		}
		return "";
	}
	
	function getRepeat( item  ) {	
		if(item.repetition != undefined && item.repetition != "0") {
			return item.repetition;
		}
		return "";
	}
	
	function getStructureID( item ) {
		
			if ( item[ "class" ] == "PacEditionLine" ){
				if(item.structureID != undefined && item.structureID != "0") {
					return item.structureID;
				}
			}
			return "";
					
	}
	
	function getLabelID( item ) {
		if ( item[ "class" ] == "PacEditionLine" ){
			if(item.labelID != undefined && item.labelID != "0") {
				return item.labelID;
			}
		}
		return "";
	}
	
	function getSkip(item) {
		
		if ( item[ "class" ] == "PacEditionLine" ){
			if(item.jump != undefined && item.jump != "0") {
				return item.jump;
			}
		}
		return "";
	}
	
	function getSkipTyp( item ) {	
		if ( item[ "class" ] == "PacEditionLine" ){
			if(item.jumpType != undefined ) {
				var res = item.jumpType[0].value;
				return getValueS(res);
				
			}
		}
		return "";
	}
	
	function getFOSF(item) {
		if ( item[ "class" ] == "PacEditionLine" ){
			if(item.functionToPerform != undefined && item.functionToPerform != "") {
				return item.functionToPerform;
				
			}
		}
		return "";
	}
	
	function getComment( item  ) {	
		if(item.comment != undefined && item.comment != "") {
			return item.comment;
		}
		return "";
	}
	
	function getCondition( item  ) {	
		if(item.categoryCondition != undefined && item.categoryCondition != "") {
			return item.categoryCondition;
		}
		return "";
	}
	/////////////
	function getValue ( value ) {		
		return value.toString().replace("_","").replace("None","");
	}
	function getValueS ( value ) {		
		return value.toString().replace("_","").replace("STAR","*").replace("None","");
	}
	
	
	
} );