/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
         "dojo/string", "dojox/layout/TableContainer", "dijit/layout/ContentPane", "dijit/form/TextBox", "dijit/tree/ForestStoreModel",
         "dojo/data/ItemFileReadStore", "dojox/grid/LazyTreeGrid", "dijit/form/Button", "dijit/Dialog", "dijit/Tooltip", "./gLinesPane",
         "dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",
         "dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function ( string, TableContainer, ContentPane, TextBox, ForestStoreModel, ItemFileReadStore, LazyTreeGrid, Button, Dialog, Tooltip, gLinesPane, 
		PacbaseEditor, typeDict ) {	

	function getValue ( value ) {		
		return value.toString().replace("_","").replace("None","");
	}
		
	function getDataElement( item, project, streamId ) {	
		if (item[ "class" ] == "PacCELineCategory") {
			if (item.categoryName != undefined && item.categoryName != "") {
				return "<span class='display_icon PacCELineCategory' role='presentation'></span> " + item.categoryName;
			}
			return "<span class='display_icon PacCELineCategory' role='presentation'></span>";
		}	
		if (item[ "class" ] == "PacCELineScreenCall") {
			return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", item.screen[0], false, streamId, project, "", "display_icon PacCELineScreenCall");
		}	
		if (item[ "class" ] == "PacCELineField") {
			if (item.dataElement != undefined && item.dataElement != "") {
				return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", item.dataElement[0], false, streamId, project, "", "display_icon PacCELineField");			
			}
			var field = getValue(item.fieldType[0].value).toUpperCase();
			return "<span class='display_icon PacCELineField' role='presentation'></span> " + field;
		}	
		
		if (item[ "class" ] == "PacCELineFieldComplement") {
			if (item.screen != undefined && item.screen != "" )
					return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", item.screen[0], false, streamId, project, "", "display_icon PacCELineFieldComplement");
			return "<span class='display_icon PacCELineFieldComplement' role='presentation'></span>"; 
		}	
		if (item[ "class" ] == "PacCELineLabel") {
			if (item.screen != undefined && item.screen != "") {
				var dataEle = getValue(item.screen[0].name);
				return "<span class='display_icon PacCELineLabel'></span> " + dataEle;
			}
			return "<span class='display_icon PacCELineLabel' role='presentation'></span>";
		}	
		return "";
	}
	
	function getPositionType( item ) {	
		if (item.positionType != undefined && item.positionType != "") 
			return  getValue(item.positionType[0].value);
		return "";
	}
	
	function getLinePosition( item ) {	
		if(item.linePosition != undefined && item.linePosition != "0") 
			return item.linePosition.toString();
		return "";
	}
	
	function getColumnPosition( item ) {	
		if(item.columnPosition != undefined && item.columnPosition != "0" ) 
			return item.columnPosition.toString();
		return "";
	}
	
	function getNature( item ) {	
		if (item.labelNature != undefined && item.labelNature != "") 
			return  getValue(item.labelNature[0].value);
		if (item.fieldNature != undefined && item.fieldNature != "") 
			return  getValue(item.fieldNature[0].value);
		if (item.categoryNature != undefined && item.categoryNature != "") 
			return  getValue(item.categoryNature[0].value);
		return "";
	}
	
	function getLabelPresentation( item ) {	
		if (item.labelPresentation != undefined && item.labelPresentation != "") 
			return  getValue(item.labelPresentation[0].value);
		return "";
	}
	
	function getCursor( item ) {	
		if(item.cursor != undefined) 
			return item.cursor;
		return "";
	}
	
	function getHorizontalRepetition( item ) {	
		if(item.horizontalRepetition != undefined && item.horizontalRepetition != "0") 
			return item.horizontalRepetition.toString();
		return "";
	}
	
	function getVerticalRepetition( item ) {	
		if(item.verticalRepetition != undefined && item.verticalRepetition != "0") 
			return item.verticalRepetition.toString();
		return "";
	}
	
	function getPresenceCheck( item ) {
		// Field complement
		if ( item[ "class" ] == "PacCELineFieldComplement" ) {
			return getValue(item.presenceCheck[0].value);
		}
		return "";
	}
	
	function getActionCode( item ) {
		// Field complement
		if ( item[ "class" ] == "PacCELineFieldComplement" ) {
			return getValue(item.actionCode[0].value);
		}
		return "";
	}
	
	function getUpdateOption( item ) {
		// Field complement
		if ( item[ "class" ] == "PacCELineFieldComplement" ) {
			return getValue(item.updateOption[0].value);
		}
		return "";
	}
	
	function getUpdateSegmentCode( item ) {
		// Field complement
		if ( item[ "class" ] == "PacCELineFieldComplement" ) {
			return getValue(item.updateSegmentCode);
		}
		return "";
	}
	
	function getSourceType( item ) {
		// Field complement
		if ( item[ "class" ] == "PacCELineFieldComplement" ) {
			return getValue(item.sourceType[0].value.toString().replace("STAR","*"));
		}
		return "";
	}
					
	function getDisplaySegmentCode( item ) {
		// Field complement
		if ( item[ "class" ] == "PacCELineFieldComplement" ) {
			return getValue(item.displaySegmentCode);
		}
		return "";
	}
	
	function getGenerateLevel( item ) {	
		// Field complement
		if ( item[ "class" ] == "PacCELineFieldComplement" ) {
			return getValue(item.generateLevel.toString());
		}
		return "";
	}
	
	function getIntensityAtt( item ) {
		intAtt = "";
		if (item.intensityAtt != undefined && item.intensityAtt != "")
			intAtt = getValue(item.intensityAtt[0].value);
		if (item.fieldIntensityAtt != undefined)
			intAtt = getValue(item.fieldIntensityAtt[0].value) + getValue(item.labelIntensityAtt[0].value);
		return intAtt;
		}
	
	function getPresentationAtt( item ) {
		preAtt = "";
		if (item.presentationAtt != undefined && item.presentationAtt != "")
			intAtt = getValue(item.presentationAtt[0].value);
		if (item.fieldPresentationAtt != undefined)
			intAtt = getValue(item.fieldPresentationAtt[0].value) + getValue(item.labelPresentationAtt[0].value);
		return preAtt;
		}
	
	function getColorAtt( item ) {
		colorAtt = "";
		if (item.colorAtt != undefined && item.colorAtt != "")
			intAtt = getValue(item.colorAtt[0].value);
		if (item.fieldColorAtt != undefined)
			intAtt = getValue(item.fieldColorAtt[0].value) + getValue(item.labelColorAtt[0].value);
		return colorAtt;
		}
	
	function getRepetition( item ) {
		// PacCELineLabel
		if (item.screen != undefined) 
			if (item.repetition != undefined && item.repetition != "" && item.repetition != "0")
				return getValue(item.repetition);
		return "";
	}	
	
	function getRepeatedCharacter( item ) {
		// PacCELineLabel
		if (item.screen != undefined) 
			if (item.repeatedCharacter != undefined && item.repeatedCharacter != "")
				return getValue(item.repeatedCharacter);
		return "";
	}
	
	function getLabel( item ) {
		// PacCELineLabel
		if (item.screen != undefined) {  
			if (item.label != undefined && item.label != "") {
				var newLabel = '';
				newLabel += item.label[0].replace(/ /g,"&nbsp;");
				return getValue(newLabel);
			}
		}
		return "";
	}
	
	
	function getInitialValue( item ) {
		// PacCELineLabel
		if (item.initialValue != undefined && item.initialValue != "")
				return getValue(item.initialValue);
		return "";
	}
	
	function getSimulationValue( item ) {		
		if (item.simulationValue != undefined && item.simulationValue != "") {
			var newValue = '';
			newValue += item.simulationValue[0].replace(/ /g,"&nbsp;");
			return getValue(newValue);
		}
		return "";
	}
	
	function getIndicator( item ) {
		if (item.indAttributePosition != undefined) {
			if (item.indAttributePosition != undefined && item.indAttributePosition != "")
				return getValue(item.indAttributePosition);
		}
		return "";
	}
	
	return {
		
		getDetailCELines : function ( data, type, project, streamId ) 
		{	var CELines = data.CELines
			var cpDetailCELines = new ContentPane( {
				content : "<strong>" + typeDict[ type ] + " " + data.name + " : " + data.label + "</strong><P>"
			    + "<strong>" + PacbaseEditor.CELINE_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
						+ string.substitute( PacbaseEditor.CELINE_TABLE_SECTION_DESCRIPTION, [
							typeDict[ type ]
						] ) + "</p>",
						style : {
							'overflow' : 'auto',
						},
			} );
			
			// prepare layoutCELines
			var layoutCELines = [
 					{
						name : PacbaseEditor.CELINE_COLUMN_DATA_ELEMENT,
						width : "100px",
						field : "_item",
						formatter : function ( item , project, streamId) {
							return getDataElement(item);
						}	
					}, {
						name : PacbaseEditor.CELINE_COLUMN_POSITION_TYPE,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {
							return getPositionType(item);
						}
					}, {
						name : PacbaseEditor.CELINE_COLUMN_LINE_POSITION,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {
							return getLinePosition (item);
						}
					},{
						name : PacbaseEditor.CELINE_COLUMN_COLUMN_POSITION,
						width : "25px",
						field : "_item",
						formatter : function ( item ) {
							return getColumnPosition(item);
						}
					},{
						name : PacbaseEditor.CELINE_COLUMN_NATURE,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {
							return getNature(item);
						}
					},{
						name : PacbaseEditor.CELINE_COLUMN_LABEL_PRESENTATION,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {
							return getLabelPresentation(item);
						}
					},{
						name : PacbaseEditor.CELINE_COLUMN_CURSOR,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {
							return getCursor(item);
						}
					},{
						name : PacbaseEditor.CELINE_COLUMN_HORIZONTAL_REPETITION,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {
							return getHorizontalRepetition(item);
					    }
					},{
						name : PacbaseEditor.CELINE_COLUMN_VERTICAL_REPETITION,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {
							return getVerticalRepetition(item);
					    }
					},{
						name : PacbaseEditor.CELINE_COLUMN_PRESENCE_CHECK,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {
							return getPresenceCheck(item);
					    }
					},{
						name : PacbaseEditor.CELINE_COLUMN_ACTION_CODE,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {
							return getActionCode(item);
					    }
					},{
						name : PacbaseEditor.CELINE_COLUMN_UPDATE_OPTION,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {
							return getUpdateOption(item);
					    }
					},{
						name : PacbaseEditor.CELINE_COLUMN_UPDATE_SEGMENT_CODE,
						width : "75px",
						field : "_item",
						formatter : function ( item ) {
							return getUpdateSegmentCode(item);
					    }
					},{
						name : PacbaseEditor.CELINE_COLUMN_SOURCE_TYPE,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {
							return getSourceType(item);
					    }
					},{
						name : PacbaseEditor.CELINE_COLUMN_DISPLAY_SEGMENT_CODE,
						width : "90px",
						field : "_item",
						formatter : function ( item ) {
							return getDisplaySegmentCode(item);
					    }
					},{
						name : PacbaseEditor.CELINE_COLUMN_GENERATE_LEVEL,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {
							return getGenerateLevel(item);
					    }
					},{
						name : PacbaseEditor.CELINE_COLUMN_INTENSITY_ATT,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {
							return getIntensityAtt(item);
					    }
					},{
						name : PacbaseEditor.CELINE_COLUMN_PRESENTATION_ATT,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {
							return getPresentationAtt(item);
					    }
					},{
						name : PacbaseEditor.CELINE_COLUMN_COLOR_ATT,
						width : "20px",
						field : "_item",
						formatter : function ( item ) {
							return getColorAtt(item);
					    }
					},{
						name : PacbaseEditor.CELINE_COLUMN_REPETITION,
						width : "25px",
						field : "_item",
						formatter : function ( item ) {
							return getRepetition(item);
					    }
					},{
						name : PacbaseEditor.CELINE_COLUMN_REPETEAD_CHARACTER,
						width : "25px",
						field : "_item",
						formatter : function ( item ) {
							return getRepeatedCharacter(item);
					    }
					},{
						name : PacbaseEditor.CELINE_COLUMN_LABEL,
						width : "200px",
						field : "_item",
						formatter : function ( item ) {
							return getLabel(item);
					    }
					},{
						name : PacbaseEditor.CELINE_COLUMN_INITIAL_VALUE,
						width : "100px",
						field : "_item",
						formatter : function ( item ) {
							return getInitialValue(item);
					    }
					},{
						name : PacbaseEditor.CELINE_COLUMN_SIMULATION_VALUE,
						width : "220px",
						field : "_item",
						formatter : function ( item ) {
							return getSimulationValue(item);
					    }
					}, {
						name : PacbaseEditor.CELINE_COLUMN_INDICATOR,
						width : "100px",
						field : "_item",
						formatter : function ( item ) {
							return getIndicator(item);
					    }
					},
					
			];
			
														
			// prepare lazy tree grid
			var data = {
				items : CELines
			};
			var store = new ItemFileReadStore( {
				data : data
			} );
			var model = new ForestStoreModel( {
				store : store,
				mayHaveChildren : function ( item ) {
					if ( (item.dataElement != undefined && item.dataElement != "") ||
						(item.fieldType != undefined && item.fieldType != "")) {						
						return true;
					}
					return false;
				},
				getChildren : function ( item, onComplete, onError ) {
					if ( item.complements != undefined && item.complements != "") {
						onComplete(item.complements);
					}
				}	
			} );
			
			var hearderTooltips =  [PacbaseEditor.CELINE_COLUMN_TOOL_TIP_DATA_ELEMENT,
					                PacbaseEditor.CELINE_COLUMN_TOOL_TIP_POSITION_TYPE,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_LINE_POSITION,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_COLUMN_POSITION,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_NATURE,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_LABEL_PRESENTATION,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_CURSOR,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_HORIZONTAL_REPETITION,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_VERTICAL_REPETITION,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_PRESENCE_CHECK,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_ACTION_CODE,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_UPDATE_OPTION,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_UPDATE_SEGMENT_CODE,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_SOURCE_TYPE,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_DISPLAY_SEGMENT_CODE,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_GENERATE_LEVEL,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_INTENSITY_ATT,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_PRESENTATION_ATT,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_COLOR_ATT,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_REPETITION,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_REPETEAD_CHARACTER,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_LABEL,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_INITIAL_VALUE,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_SIMULATION_VALUE,
									PacbaseEditor.CELINE_COLUMN_TOOL_TIP_INDICATOR];
			
			// create a new grid
			var CELinesGrid = new LazyTreeGrid( {
				treeModel : model,
				structure : layoutCELines,
				height : ( rppCurrentPaneGridHeight - 70 ) + "px",
				autoWidth : true,
				selectable : true,
				onStyleRow: function (row) {
					row.customClasses = "rppGridFixedDefaultFont";
				},
				canSort : function () {
					return false;
				},
				onHeaderCellMouseOver  : function (e) {
					dijit.showTooltip(hearderTooltips[e.cellIndex], e.cellNode);
				},
				onHeaderCellMouseOut : function (e) {
					dijit.hideTooltip(e.cellNode);
				} 
			} );

			cpDetailCELines.addChild(CELinesGrid);
			return cpDetailCELines;
		
		}
	}
} );