/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dojox/layout/TableContainer", "dijit/layout/ContentPane", "dijit/form/TextBox",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacEnumerationLabel",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function ( TableContainer, ContentPane, TextBox, PacbaseEditor, PacEnumerationLabel, typeDict ) {
	function baseDefinition ( tableSize, occurencesNumber, structureCodeValue, actionCodeValue, segmentType, tableNumber, transfertDirection ) {
		var definitionTable = new TableContainer( {
			labelWidth : 150,
		} );

		if ( "_None" == segmentType ) {
			var textTableSize = new TextBox({
				label : PacbaseEditor.PAC_TABLE_TABLE_SIZE,
				value : tableSize,
				disabled : true
			});
			var textOccurencesNumber = new TextBox({
				label : PacbaseEditor.LOGICAL_VIEW_TABLE_SIZE,
				value : occurencesNumber,
				disabled : true
			});
			var textStructureCodeValue = new TextBox({
				label : PacbaseEditor.STRUCTURE_CODE_VALUE,
				value : structureCodeValue,
				disabled : true
			});
			var textActionCodeValue = new TextBox({
				label : PacbaseEditor.ACTION_CODE_VALUE,
				value : actionCodeValue,
				disabled : true
			});
			definitionTable.addChild(textTableSize);
			definitionTable.addChild(textOccurencesNumber);
			definitionTable.addChild(textStructureCodeValue);
			definitionTable.addChild(textActionCodeValue);
		} else if ( "_G" == segmentType ) {
			var textTableSize = new TextBox({
				label : PacbaseEditor.PAC_TABLE_TABLE_SIZE,
				value : tableSize,
				disabled : true
			});
			var textTableNumber = new TextBox({
				label : PacbaseEditor.PAC_TABLE_TABLE_NUMBER,
				value : tableNumber,
				disabled : true
			});			
			definitionTable.addChild(textTableNumber);
			definitionTable.addChild(textTableSize);
		} else if ( "_V" == segmentType ) {
			var textOccurencesNumber = new TextBox({
				label : PacbaseEditor.LOGICAL_VIEW_TABLE_SIZE,
				value : occurencesNumber,
				disabled : true
			});
			definitionTable.addChild(textOccurencesNumber);
			if ( transfertDirection != undefined ) {
				var textTransfertDirection = new TextBox({
					label : PacbaseEditor.LOGICAL_VIEW_TRANSFER_DIRECTION,
					value : PacEnumerationLabel[transfertDirection["class"] + transfertDirection["value"].toUpperCase()],
					disabled : true
				});
				definitionTable.addChild(textTransfertDirection);
			}
		}

		return definitionTable;
	}

	return {
		getOverviewDefinition : function ( tableSize, occurencesNumber, structureCodeValue, actionCodeValue, segmentType, tableNumber, transfertDirection, type ) {
			var overviewDefinition;
			require( [
				"dojo/string"
			], function ( string ) {
				overviewDefinition = new ContentPane( {
					content : "<hr><strong>" + PacbaseEditor.SEGMENT_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><P>"
							+ string.substitute( PacbaseEditor.SEGMENT_SECTION_DESCRIPTION, [
								typeDict[ type ]
							] ) + "</p>",
				} );
			} );
			overviewDefinition.addChild( baseDefinition( tableSize, occurencesNumber, structureCodeValue, actionCodeValue, segmentType, tableNumber, transfertDirection ) );

			return overviewDefinition;
		},
		getDetailDefinition : function ( extension, label, name, type  ) {
			var segmentType = extension.dataAggregateType["value"];
			var tableSize = extension.tableSize;
			var occurencesNumber = extension.occurencesNumber;
			var structureCodeValue = extension.structureCodeValue;
			var actionCodeValue = extension.actionCodeValue;
			var presenceCheck = extension.presenceCheck;
			var detailDefinition;
			var tableNumber = "";
			var transfertDirection; 
			
			if ( "_None" == segmentType && 
					extension.alias != undefined && extension.alias.endsWith("00")) {
				structureCodeValue = extension.structureCode;
			    actionCodeValue = extension.actionCode;
			} 
			
			if ( extension.tableAttributes != undefined )
				tableNumber = extension.tableAttributes.tableNumber;

			if ( extension.logicalViewAttributes != undefined )
				transfertDirection = extension.logicalViewAttributes.transfertDirection;
			
			require( [
				"dojo/string"
			], function ( string ) {
				detailDefinition = new ContentPane( {
					content : "<strong>" + typeDict[ type ] + " " + name + " : " + label + "</strong><P>"
				    + "<strong>" + PacbaseEditor.SEGMENT_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
							+ string.substitute( PacbaseEditor.SEGMENT_EDIT_SECTION_DESCRIPTION, [
								typeDict[ type ]
							] ) + "</p>",
				} );
			} );
			detailDefinition.addChild( baseDefinition( tableSize, occurencesNumber, structureCodeValue, actionCodeValue, segmentType, tableNumber, transfertDirection ) );

			if ( "_None" == segmentType ) {
					if ( extension.alias != undefined && extension.alias.endsWith("00")) {
						var cpActionCode = new ContentPane({
							content : "<hr>" + PacbaseEditor.ACTION_CODE_VALUE,
						});
						var actionCodeTable = new TableContainer({
							labelWidth : 150,
							valueWidth : 200,
						});
						var textCreation = new TextBox(
								{
									label : PacbaseEditor.CREATE,
									value : extension.creationCode,
									disabled : true
								});
						var textModification = new TextBox(
								{
									label : PacbaseEditor.UPDATE,
									value : extension.modificationCode,
									disabled : true
								});
						var textDeletion = new TextBox(
								{
									label : PacbaseEditor.DELETE,
									value : extension.deletionCode,
									disabled : true
								});
						var textUpdate4 = new TextBox(
								{
									label : PacbaseEditor.UPDATE4,
									value : extension.inType4Code,
									disabled : true
								});
						var textUpdate5 = new TextBox(
								{
									label : PacbaseEditor.UPDATE5,
									value : extension.inType5Code,
									disabled : true
								});
						var textUpdate6 = new TextBox(
								{
									label : PacbaseEditor.UPDATE6,
									value : extension.inType6Code,
									disabled : true
								});
						actionCodeTable.addChild(textCreation);
						actionCodeTable.addChild(textModification);
						actionCodeTable.addChild(textDeletion);
						actionCodeTable.addChild(textUpdate4);
						actionCodeTable.addChild(textUpdate5);
						actionCodeTable.addChild(textUpdate6);

						cpActionCode.addChild(actionCodeTable);
						detailDefinition.addChild(cpActionCode);							
					} else {
						var cpPresenceCheck = new ContentPane({
							content : "<hr>" + PacbaseEditor.PRESENCE_CHECK_GRP
									+ " : ",
						});
						var presenceCheckTable = new TableContainer({
							labelWidth : 150,
							valueWidth : 200,
						});
						var textCreation = new TextBox(
								{
									label : PacbaseEditor.CREATE,
									value : presenceCheck.inCreation != undefined ? PacEnumerationLabel[presenceCheck.inCreation["class"]
											+ presenceCheck.inCreation["value"]]
											: "",
									disabled : true
								});
						var textModification = new TextBox(
								{
									label : PacbaseEditor.UPDATE,
									value : presenceCheck.inModification != undefined ? PacEnumerationLabel[presenceCheck.inModification["class"]
											+ presenceCheck.inModification["value"]]
											: "",
									disabled : true
								});
						var textDeletion = new TextBox(
								{
									label : PacbaseEditor.DELETE,
									value : presenceCheck.inDeletion != undefined ? PacEnumerationLabel[presenceCheck.inDeletion["class"]
											+ presenceCheck.inDeletion["value"]]
											: "",
									disabled : true
								});
						var textUpdate4 = new TextBox(
								{
									label : PacbaseEditor.UPDATE4,
									value : presenceCheck.inType4 != undefined ? PacEnumerationLabel[presenceCheck.inType4["class"]
											+ presenceCheck.inType4["value"]]
											: "",
									disabled : true
								});
						var textUpdate5 = new TextBox(
								{
									label : PacbaseEditor.UPDATE5,
									value : presenceCheck.inType5 != undefined ? PacEnumerationLabel[presenceCheck.inType5["class"]
											+ presenceCheck.inType5["value"]]
											: "",
									disabled : true
								});
						var textUpdate6 = new TextBox(
								{
									label : PacbaseEditor.UPDATE6,
									value : presenceCheck.inType6 != undefined ? PacEnumerationLabel[presenceCheck.inType6["class"]
											+ presenceCheck.inType6["value"]]
											: "",
									disabled : true
								});
						presenceCheckTable.addChild(textCreation);
						presenceCheckTable.addChild(textModification);
						presenceCheckTable.addChild(textDeletion);
						presenceCheckTable.addChild(textUpdate4);
						presenceCheckTable.addChild(textUpdate5);
						presenceCheckTable.addChild(textUpdate6);

						cpPresenceCheck.addChild(presenceCheckTable);
						detailDefinition.addChild(cpPresenceCheck);
					}
			}
			return detailDefinition;
		}
	};
} );