/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2016, 2018. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dojox/layout/TableContainer", "dijit/layout/ContentPane", "dijit/form/TextBox", "dijit/form/Button", 'dojox/grid/DataGrid',
		'dojo/data/ItemFileWriteStore', "dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function ( TableContainer, ContentPane, TextBox, Button, DataGrid, ItemFileWriteStore, PacbaseEditor, typeDict ) {
	function getStatistics ( statistics ) {
		var tableStatistics = new TableContainer( {
			labelWidth : 200,
		} );
		var textName = new TextBox( {
			label : PacbaseEditor.LEVEL_ADDRESS_LENGTH_SEGMENT,
			value : statistics.name,
			disabled : true,
		} );
		var textNbDataElements = new TextBox( {
			label : PacbaseEditor.LEVEL_ADDRESS_LENGTH_SEGMENT_NB_ELEMENTS,
			value : statistics.nbDataElements,
			disabled : true
		} );
		var textNbElementaryFields = new TextBox( {
			label : PacbaseEditor.LEVEL_ADDRESS_LENGTH_SEGMENT_NB_ELEMENTARY_FIELDS,
			value : statistics.nbElementaryFields,
			disabled : true
		} );
		var textTotalInputLength = new TextBox( {
			label : PacbaseEditor.LEVEL_ADDRESS_LENGTH_SEGMENT_TOTAL_INPUT,
			value : statistics.totalInputLength,
			disabled : true
		} );
		var textTotalInternalLength = new TextBox( {
			label : PacbaseEditor.LEVEL_ADDRESS_LENGTH_SEGMENT_TOTAL_INTERNAL,
			value : statistics.totalInternalLength,
			disabled : true
		} );
		var textTotalOutputLength = new TextBox( {
			label : PacbaseEditor.LEVEL_ADDRESS_LENGTH_SEGMENT_TOTAL_OUTPUT,
			value : statistics.totalOutputLength,
			disabled : true
		} );
		var textNbSegmentCalls = new TextBox( {
			label : PacbaseEditor.LEVEL_ADDRESS_LENGTH_SEGMENT_NB_SEGMENT_CALLS,
			value : statistics.nbSegmentCalls,
			disabled : true
		} );

		tableStatistics.addChild( textName );
		tableStatistics.addChild( textNbDataElements );
		tableStatistics.addChild( textNbElementaryFields );
		tableStatistics.addChild( textTotalInputLength );
		tableStatistics.addChild( textTotalInternalLength );
		tableStatistics.addChild( textTotalOutputLength );
		tableStatistics.addChild( textNbSegmentCalls );
		return tableStatistics;
	}

	function getTablePreview ( tablePreview ) {
		if ( rppDebug )
			console.debug ( "-LAL mumber of lines : " + tablePreview.length )
		// set up data store
		var data = {
			items : tablePreview,
		};
		var store = new ItemFileWriteStore( {
			data : data
		} );
		var layout = [
				{
					name : PacbaseEditor.CELINE_COLUMN_LEVEL,
					field : "level"
				}, {
					name : PacbaseEditor.CELINE_COLUMN_NAME,
					field : "name"
				}, {
					name : PacbaseEditor.CELINE_COLUMN_LABEL,
					field : "label",
					width : "200px"
				}, {
					name : PacbaseEditor.CELINE_COLUMN_RELLABEL,
					field : "rellable"
				}, {
					name : PacbaseEditor.CELINE_COLUMN_OCCURS,
					field : "occurs"
				}, {
					name : PacbaseEditor.CELINE_COLUMN_INTERNAL_FORMAT,
					field : "internalFormat"
				}, {
					name : PacbaseEditor.CELINE_COLUMN_USAGE,
					field : "usage"
				}, {
					name : PacbaseEditor.CELINE_COLUMN_INTERNAL_LENGTH,
					field : "internalLength"
				}, {
					name : PacbaseEditor.CELINE_COLUMN_INTERNAL_POSITION,
					field : "internalAddress"
				}, {
					name : PacbaseEditor.CELINE_COLUMN_INPUT_FORMAT,
					field : "inputFormat"
				}, {
					name : PacbaseEditor.CELINE_COLUMN_INPUT_LENGTH,
					field : "inputLength"
				}, {
					name : PacbaseEditor.CELINE_COLUMN_INPUT_POSITION,
					field : "inputAdress"
				}, {
					name : PacbaseEditor.CELINE_COLUMN_OUTPUT_FORMAT,
					field : "outputFormat"
				}, {
					name : PacbaseEditor.CELINE_COLUMN_OUTPUT_LENGTH,
					field : "outputLength"
				}, {
					name : PacbaseEditor.CELINE_COLUMN_OUTPUT_POSITION,
					field : "outputAdress"
				}
		];
		
		var hearderTooltips =  [PacbaseEditor.CELINE_COLUMN_LEVEL,
		                        PacbaseEditor.CELINE_COLUMN_NAME,
		                        PacbaseEditor.CELINE_COLUMN_LABEL,
		                        PacbaseEditor.CELINE_COLUMN_RELLABEL,
		                        PacbaseEditor.CELINE_COLUMN_OCCURS,
		                        PacbaseEditor.CELINE_COLUMN_INTERNAL_FORMAT,
		                        PacbaseEditor.CELINE_COLUMN_USAGE,
		                        PacbaseEditor.CELINE_COLUMN_INTERNAL_LENGTH,
		                        PacbaseEditor.CELINE_COLUMN_INTERNAL_POSITION,
		                        PacbaseEditor.CELINE_COLUMN_INPUT_FORMAT,
		                        PacbaseEditor.CELINE_COLUMN_INPUT_LENGTH,
		                        PacbaseEditor.CELINE_COLUMN_INPUT_POSITION,
								PacbaseEditor.CELINE_COLUMN_OUTPUT_FORMAT,
								PacbaseEditor.CELINE_COLUMN_OUTPUT_LENGTH,
								PacbaseEditor.CELINE_COLUMN_OUTPUT_POSITION,];
		// create a new grid
		var grid = new DataGrid( {
			store : store,
			structure : layout,
			autoHeight : true,
			autoWidth : true, 
			selectable : true,
			canSort : function () {
				return false;
			},
			
			onHeaderCellMouseOver  : function (e) {
				dijit.showTooltip(hearderTooltips[e.cellIndex], e.cellNode);
			},
			onHeaderCellMouseOut : function (e) {
				dijit.hideTooltip(e.cellNode);
			} 	
			
		} );
		return grid;
	}

	return {
		getLaL : function ( data, type, streamId, project, compentId, fileItemId ) {
			var cpSTALAL = new ContentPane( {
				content : "<strong>" + typeDict[ type ] + " " + data.name + " : " + data.label + "</strong><P>"
			    + "<strong>" + PacbaseEditor.LEVEL_ADDRESS_LENGTH_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
						+ PacbaseEditor.LEVEL_ADDRESS_LENGTH_SECTION_DESCRIPTION + "</p>",
			} );

			var buttonRefresh = new Button( {
				label : PacbaseEditor.CELINE_LALREFRESH,
				onClick : function () {
					if ( rppDebug == true )
						buttonRefresh.set( "disabled", false );
					else
						buttonRefresh.set( "disabled", true );
					buttonRefresh.set( "label", PacbaseEditor.CELINE_LAL_LOADING );
					require( [
						"dojo/request/xhr"
					], function ( xhr ) {
						xhr( net.jazz.ajax.contextRoot() + "/service/com.ibm.teampdp.query.engine.common.IRppDesignFileService", {
							handleAs : "json",
							method : "POST",
							data : {
								streamId : streamId,
								compentId : compentId,
								fileItemId : fileItemId,
								project : project,
								dependencies : true,
								rppDebug : rppDebug
							}
						} ).then( function ( data ) {
							var childs = [];
							cpSTALAL.getChildren().forEach(function(child) {
								childs.push(child);
							});
							childs.forEach(function(child) {
								cpSTALAL.removeChild( child );
							});
							cpSTALAL.addChild( buttonRefresh );
							var cpStatistics = new ContentPane( {
								content : PacbaseEditor.LEVEL_ADDRESS_LENGTH_STATISTICS + "<hr>",
							} );
							cpStatistics.addChild( getStatistics( data.statistics ) );
							cpSTALAL.addChild( cpStatistics );
							var cpTablePreview = new ContentPane( {
								content : PacbaseEditor.LEVEL_ADDRESS_LENGTH_HEADER + "<hr>",
							} );
							cpTablePreview.addChild( getTablePreview( data.lengthAndPosition ) );
							cpSTALAL.addChild( cpTablePreview );
							buttonRefresh.set( "label", PacbaseEditor.CELINE_LALREFRESH );
						}, function ( err ) {
							var cpError = new ContentPane();
							displayRppError( err );
							buttonRefresh.set( "label", PacbaseEditor.CELINE_LALREFRESH );
							buttonRefresh.set( "disabled", false );
						} );
					} );
				},
			} );

			cpSTALAL.addChild( buttonRefresh );
			return cpSTALAL;
		}
	};
} );