/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dojo/string", "dijit/layout/ContentPane", "dijit/tree/ForestStoreModel",
		"dojo/data/ItemFileReadStore","dojox/grid/LazyTreeGrid",	
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function ( string,  ContentPane, ForestStoreModel, ItemFileReadStore, LazyTreeGrid, PacbaseEditor, typeDict ) {	
	
	return {		
		getField : function ( data, type, project, streamId, compentId, fileItemId, metaEntity ) {
			
			var memoName;
			
			var Field = [data.field];

			var cpField = new ContentPane( {
				content : "<strong>" + typeDict[ type ] + " " + data.name + " : " + data.label + "</strong><P>"
			    + "<strong>" + PacbaseEditor.USER_ENTITY_FIELD + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
						+ string.substitute( PacbaseEditor.USER_ENTITY_FIELD_DESCRIPTION, [
							typeDict[ type ]
						] ) + "</p>",
						style : {
							'overflow' : 'auto',
						},
			} );
			
			function getField(item) {	
				var classeValue = item [ "class" ];				
				if (classeValue == "Field" ) {
					memoName = item.name;
					var classeField = item.value[0] [ "class" ];
					if (classeField == "AggregateValue" || classeField == "MultipleValue") {
						return  "<span class='display_icon aggregatevalue'></span> " + " "+ getFieldLabel ( item ) +" [" + item.name + "]";
					} else {
						if (classeField == "IntegerValue") {
							return  "<span class='display_icon IntegerValue'></span> " + " "+ getFieldLabel ( item ) +" " + "["  + item.name + "]";
						} else if (classeField == "ReferenceValue") {
							return  "<span class='display_icon ReferenceValue'></span> " + " "+ getFieldLabel ( item ) +" " + "["  + item.name + "]";
						} else {
							return  "<span class='display_icon PacCELineLabel'></span> " + " "+ getFieldLabel ( item ) +" " + "["  + item.name + "]";
						}
					}
				}
				if (classeValue == "AggregateValue" ) {								
					return  "<span class='display_icon aggregatevalue'></span> " + ""+ getFieldLabel ( item ) +" [" + memoName + "]";
				}				
			}
			
			function getFieldLabel(item) {
				var paths = [];
				var parent = item.parent;
				if ( item.name != undefined ) {
					if ( item.name instanceof Array )
						paths.push(item.name[0]);
					else
						paths.push(item.name);
				}
				while (parent != undefined ) {
					if ( parent.name != undefined ) {
						paths.push(parent.name[0]);
					}
					parent = parent.parent;
				}
				var components = metaEntity.dataDescription.components;
				for ( var i=paths.length-1; i>=0; i-- ) {
					for ( var j=0; j<components.length; j++ ) {
						var component = components[j];
						if (component.dataDefinition == "" ) {
							if ( component.dataDescription.name == paths[i] ) {
								if ( i == 0 )									
									return component.dataDescription.label.replace("Relation ","");
								else {
									components = component.dataDescription.components;
								}
							}							
						} else {
							if ( component.dataDefinition.name == paths[i] ) {
								if ( i == 0 )
									return component.dataDefinition.label;
								else {
									components = component.dataDefinition.components;
								}
							}	
						}
					}
				}	
				return metaEntity.dataDescription.label;
			}
			
			function getValue(item) {
				var classeValue = item [ "class" ];				
				if (classeValue == "Field" ) {					
					var classeField = item.value[0] [ "class" ];
					if (classeField == "AggregateValue" || classeField == "MultipleValue") {
						return  "";
					} else if (classeField == "ReferenceValue") {
						var ref = item.value[0].entity[0]["class"];
						if (ref == undefined) {
							return " ";
						}						
						if (ref == "dataelement") {
							var entity = 
								new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget(PacbaseEditor.PAC_DATA_ELEMENT_TYPE, item.value[0].entity[0], true, streamId, project);	
							if (entity.reLabel != undefined) {
								return entity;
							} else {
								return "<span class='display_icon dataelement'></span> " + item.value[0].entity[0].name;
							}
						}
						if (ref == "dataunit") {
							var entity = 
								new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget(PacbaseEditor.PAC_DATA_UNIT_TYPE, item.value[0].entity[0], true, streamId, project);	
							return entity;
						}
						if (ref == "dataaggregate") {
							var entity = 
								new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget(PacbaseEditor.PAC_DATA_AGGREGATE_TYPE, item.value[0].entity[0], true, streamId, project);	
							return entity;
						}
						if (ref == "pacscreen") {
							var entity = 
								new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget(PacbaseEditor.PAC_SCREEN_TYPE, item.value[0].entity[0], true, streamId, project);	
							return entity;
						}
						if (ref == "pacreport") {
							var entity = 
								new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget(PacbaseEditor.PAC_REPORT_TYPE, item.value[0].entity[0], true, streamId, project);	
							return entity;
						}
						if (ref == "pacblockbase") {
							var entity = 
								new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget(PacbaseEditor.PAC_BLOCK_BASE_TYPE, item.value[0].entity[0], true, streamId, project);	
							return entity;
						}
						if (ref == "pactext") {
							var entity = 
								new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget(PacbaseEditor.PAC_TEXT_TYPE, item.value[0].entity[0], true, streamId, project);	
							return entity;
						}
						if (ref == "metaentity") {
							var entity = 
								new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget(PacbaseEditor.PAC_META_ENTITY, item.value[0].entity[0], true, streamId, project);	
							return entity;
						}
						if (ref == "pacvolume") {
							var entity = 
								new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget(PacbaseEditor.PAC_VOLUME, item.value[0].entity[0], true, streamId, project);	
							return entity;
						}
						if (ref == "pacprogram") {
							var entity = 
								new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget(PacbaseEditor.PAC_PROGRAM_TYPE, item.value[0].entity[0], true, streamId, project);	
							return entity;
						}						
					} else {
						return  item.value[0].value;
					}					
				}
				if (classeValue == "AggregateValue" ) {
					if ( item.fields.length == 0 )
						return "";
					var field = item.fields[0];
					var name = field.name;					
					return  "(" + name + ":" + field.value[0].value + ")";
				}				
			}
			
			// prepare lazy tree grid
			var layoutFieldLines = [
					{
						name : PacbaseEditor.USER_ENTITY_FIELD,
						width : "400px",
						field : "_item",
						formatter : function ( item ) {
							return getField(item);
						}
					},
					
					{
						name : PacbaseEditor.USER_ENTITY_VALUE,
						width : "400px",
						field : "_item",
						formatter : function ( item ) {
							return getValue(item);
						}
					}, 				
			];
			
			// prepare lazy tree grid
			var _data = {
				items : Field
			};
			var store = new ItemFileReadStore( {
				data : _data,
			} );
			var model = new ForestStoreModel( {
				store : store,
				mayHaveChildren : function ( item ) {
					var classe = item [ "class" ];
					if (classe == "Field" ) {
						var value;
						if ( item.value != undefined )
							value =  item.value[0]; 
						else
							return false;
						if ( value.values != undefined && value.values.length > 0 ) {						
							return true;
						} else if ( value.fields != undefined && value.fields.length > 0 ) {						
							return true;
						}
						return false;
					}
					if (classe == "AggregateValue" ) {
						if ( item.fields != undefined && item.fields.length > 0 ) {						
							return true;
						}
					}	
					return false;
				},
				getChildren : function ( item, onComplete, onError ) {
					var classe = item [ "class" ];
					if (classe == "Field" ) {
						var value =  item.value[0]; 
						if ( value.values != undefined && value.values.length > 0) {
							value.values.forEach(function(son) {
								if ( son.parent == undefined )
									son.parent = item;
							});
							onComplete(value.values);
						} else if ( value.fields != undefined && value.fields.length > 0 ) {						
							value.fields.forEach(function(son) {
								if ( son.parent == undefined )
									son.parent = item;
							});
							onComplete(value.fields);
						}
					}
					if (classe == "AggregateValue" ) {
						if ( item.fields != undefined && item.fields.length > 0) {
							item.fields.forEach(function(son) {
								if ( son.parent == undefined )
									son.parent = item;
							});
							onComplete(item.fields);
						}
					}					
				}				
			} );
		
			var hearderTooltips =  [PacbaseEditor.USER_ENTITY_FIELD_TOOL_TIP,
			                        PacbaseEditor.USER_ENTITY_VALUE_TOOL_TIP];
			
			var grid = new LazyTreeGrid( {
				treeModel : model,
				structure : layoutFieldLines,
				height : ( rppCurrentPaneGridHeight - 70 ) + "px",
				autoWidth : true,
				selectable : true,
				canSort : function () {
					return false;
				},
				onHeaderCellMouseOver  : function (e) {
					dijit.showTooltip(hearderTooltips[e.cellIndex], e.cellNode);
				},
				onHeaderCellMouseOut : function (e) {
					dijit.hideTooltip(e.cellNode);
				} 					
			
			} );
			cpField.addChild( grid );			
			return cpField;
		}
	}
} );