/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2018. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dojo/string", "dojox/layout/TableContainer", "dijit/layout/ContentPane", "dijit/form/TextBox", "dijit/tree/ForestStoreModel",
		"dojo/data/ItemFileReadStore", "dojo/data/ItemFileWriteStore", "dojox/grid/DataGrid", "dijit/form/Button", "dijit/Dialog", "dijit/Tooltip", "./gLinesPane", 
		"dijit/TooltipDialog", "dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacEnumerationLabel",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function ( string, TableContainer, ContentPane, TextBox, ForestStoreModel, ItemFileReadStore, ItemFileWriteStore, DataGrid, Button, Dialog, Tooltip, gLinesPane, TooltipDialog,
				PacbaseEditor, PacEnumerationLabel, typeDict ) {	
	
	
	return {
		
		getDetailDLines : function ( data, type, project, streamId, compentId, fileItemId ) {	
			var DLines = data.DLines
			var detailDLines = new ContentPane( {
				content :  "<strong>" + typeDict[ type ] + " " + data.name + " : " + data.label + "</strong><P>"
			    + "<strong>" + PacbaseEditor.VOLUME_DLINE_SECTION_HEADER + PacbaseEditor.SEPARATOR + "</strong><hr><P>"
						+ string.substitute( PacbaseEditor.VOLUME_DLINE_SECTION_DESCRIPTION, [
							typeDict[ type ]
						] ) + "</p>",
						style : {
							'overflow' : 'auto',
						},
			} );

			// prepare layoutDLines
			var layoutDLines = [
 					{
						name : PacbaseEditor.VOLUME_DLINE_TABTEXT,
						width : "680px",
						field : "_item",
						formatter : function ( item ) {
							return getDetailDLine(item, this.grid.store, streamId, project);
						}
					},
					{
						name : PacbaseEditor.VOLUME_DLINE_COLUMN_DESCRIPTION,
						width : "120px",
						field : "_item",
						formatter : function ( item ) {
							return getDescription (item);
						}
					},
					
			];
			
			// prepare lazy tree grid
			var data = {
				items : DLines
			};
			var store = new ItemFileReadStore( {
				data : data
			} );
			
			var hearderTooltips =  [PacbaseEditor.VOLUME_DLINE_COLUMN_TOOL_TIP_DLINE ,
			                        PacbaseEditor.VOLUME_DLINE_COLUMN_TOOL_TIP_DESCRIPTION];
			
			// create a new grid
			var DLinesGrid = new DataGrid( {
				store : store,
				structure : layoutDLines,
				height : ( rppCurrentPaneGridHeight - 70 ) + "px",
				autoWidth : true,
				onStyleRow: function (row) {
					row.customClasses = "rppGridFixedDefaultFont";
				},
				canSort : function () {
					return false;
				},
				
				onHeaderCellMouseOver  : function (e) {
					dijit.showTooltip(hearderTooltips[e.cellIndex], e.cellNode);
				},
				onHeaderCellMouseOut : function (e) {
					dijit.hideTooltip(e.cellNode);
				} 	
				
				
			} );
						
			detailDLines.addChild(DLinesGrid);
			return detailDLines;
		
		}
	}
	
	function getDescription( item ) {
		if (item["class"] == "PacCallLine") {

			var entity = item.calledInstance[0] ;
						
			if ( entity != undefined && entity != "" ) {				
				var description = ""
				if (item.description != undefined)
					description += item.description;
				if (description != "" && entity["class"] == "pactext") {
					// section code
					var whereFields = item.whereFields
					description += " " + whereFields[0].value
				}
				return description;
												
			}
		}

		return "";
	}
	
	function getDetailDLine( item, store, streamId, project) {

		if (item["class"] == "PacTitleLine") {
			var line = "";
			if(item.level != undefined) {
				line += item.level;
			}
			if(item.text != undefined && item.text != "") {
				line += " " + item.text;
			}
			return "<span class='display_icon pactitleline'></span> " + line.replace(/ /g,"&nbsp;");
		}
		else if (item["class"] == "PacCallLine") {

			var entity = item.calledInstance[0] ;
						
			if ( entity != undefined && entity != "" ) {
				var complement = entity.name
				if (entity["class"] == "dataaggregate") {
					complement += " (" + typeDict[ entity["class"] ] + ") ";
//				} else if (entity["class"] == "pacblockbase") {
//					complement += " (type) ";
				} else if (entity["class"] == "dataelement") {
					if (entity.internalFormat != undefined && entity.internalFormat != "")
						complement += " " + entity.internalFormat;
//				} else if (entity["class"] == "pacscreen") {
//					complement += " (" + PacbaseEditor.VOLUME_PACSCREEN + ") ";
//				} else if (entity["class"] == "pacserver") {
//					complement += " (" + PacbaseEditor.VOLUME_PACSERVER + ") ";
				} else if (entity["class"] == "userentity") {
					complement += " (" + typeDict[ "metaentity" ] + ": " + entity.metaType + ") ";
				}
				
				var description = ""
				if (item.description != undefined)
					description += item.description;
				
				if (description == "" && entity["class"] != "dataaggregate" && entity["class"] != "dataelement" && entity["class"] != "userentity")
					return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", entity, true, streamId, project); 
					else {
						// complement dans l'icone cliquable 
//						if (description != "")
//							complement += " " + description
						return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", entity, true, streamId, project, complement, undefined, true); 
					}
								
			}
		}
		else if (item["class"] == "PacRequestLine") {
			var line = "<span class='display_icon pacrequestline'></span> " + PacbaseEditor.VOLUME_SEARCH;    //langue 
			var entity = item.request
			if ( entity != undefined && entity != "" ) {
				line += " '" + entity[0].expression + "'";
				if (entity[0].searchFor != undefined && entity[0].searchFor != "") {
					if (entity[0].searchFor.length != 0) {
						var components = entity[0].searchFor;
						var n = 0;
						components.forEach( function ( component ) {
							if (n == 0) line += PacbaseEditor.VOLUME_FOR + "'"
							else line += ", ";
							line += typeDict[ component ];
							n += 1
							if (n == components.length) line += "'"
						} );
					}
					line += PacbaseEditor.VOLUME_LIMIT_TO + "'" 
					 	+ PacEnumerationLabel[ entity[0].limitTo[0][ "class" ] + entity[0].limitTo[0][ "value" ] ] + "'"
				}
			}
			return line;
		}

		return "";
	}
				
	function getValue ( value ) {		
		return value.toString().replace("_","").replace("None","");
	}
	
	function getValueS ( value ) {		
		return value.toString().replace("_","").replace("STAR","*").replace("None","");
	}	
	} );