/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2017, 2018. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.OrionViewerWidget");


dojo.require( "dijit._Widget" );
dojo.require( "dijit._WidgetBase" );
dojo.require( "dijit._TemplatedMixin" );
dojo.require( "dijit._WidgetsInTemplateMixin" );
dojo.require( "dijit.registry" );
dojo.require( "dojo.io.script");
dojo.require( "com.ibm.teampdp.query.engine.web/ui/internal/page/jsModules/cobol/styleParser");
dojo.require( "com.ibm.teampdp.query.engine.web/ui/internal/page/jsModules/cobol/cobolStyler");
dojo.require( "com.ibm.teampdp.query.engine.web/ui/internal/page/jsModules/cobol/textStyler");

var codeEdit = null;

dojo.addOnLoad(function() {
	if (rppUseInlineCodeEdit == true ) {
	    var deferred = dojo.io.script.get({url : net.jazz.ajax.contextRoot()  + "/web/com.ibm.teampdp.query.engine.web/ui/built-codeEdit/code_edit/built-codeEdit.js"});
	    deferred.then(function() {
	       codeEdit = new orion.codeEdit();
	       
	       var fileref=document.createElement("link")
	       fileref.setAttribute("rel", "stylesheet")
	       fileref.setAttribute("type", "text/css")
	       fileref.setAttribute("href", net.jazz.ajax.contextRoot()  + "/web/com.ibm.teampdp.query.engine.web/ui/built-codeEdit/code_edit/built-codeEdit.css")
	       document.getElementsByTagName("head")[0].appendChild(fileref)
	       
	       fileref=document.createElement("link")
	       fileref.setAttribute("rel", "stylesheet")
	       fileref.setAttribute("type", "text/css")
	       fileref.setAttribute("href", net.jazz.ajax.contextRoot()  + "/web/com.ibm.teampdp.query.engine.web/ui/internal/page/jsModules/cobol/textStyler.css")
	       document.getElementsByTagName("head")[0].appendChild(fileref)
	       
	       var cto = {
	    	   id: "text/cobol",
	    	   extension: ["cbl"],
	    	   name: "COBOL",
	    	   'extends': 'text/plain'
	       };
	
	       codeEdit.serviceRegistry.registerService('orion.core.contenttype', {}, {contentTypes: [cto]});
				     
	       console.debug('Inline Orion Code Edit is loaded!!!');
	    }, function() {
	    	console.debug('Cannot load Inline Orion Code Edit!!!');
	    });
	} else {
		console.debug('External Orion Code Edit is loaded!!!');
	}
});

define("com.ibm.teampdp.query.engine.web/ui/internal/page/jsModules/widgets/OrionViewerWidget", 
		["../cobol/styleParser", "../cobol/cobolStyler", "../cobol/textStyler"],
function (mStyleParser, mCobolStyler, mTextStyler) {
	var OrionViewerWidget = dojo.declare( "com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.OrionViewerWidget", [
		dijit._Widget, dijit._WidgetBase, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin], {
		
		templatePath : dojo.moduleUrl( "com.ibm.teampdp.query.engine.web", "ui/internal/page/jsModules/widgets/OrionViewerWidget.html" ),
		
		constructor : function (widgetLabel, content, type, project, streamId, compentId, fileItemId, generated, size  ) {
			this.label = widgetLabel;
			this.content = content;
			this.streamId = streamId;
			this.type = type;
			this.project = project;
			this.fileItemId = fileItemId;
			this.compentId = compentId;
			if ( generated )
				this.generated = generated;
			if ( size )
				this.size = size;			
			else
				this.generated = false;
			if (content.indexOf('\r') !== -1) {
				this.content = this.content.replace(/\r/g,'');
			}
		},

		postCreate : function () {
			if (rppUseInlineCodeEdit == true ) {				
				var aDiv = document.createElement('div');
				aDiv.id = 'embeddedEditor.'+this.streamId+'.'+this.fileItemId;
				aDiv.className = 'orionBody';
				this.embeddedEditor.appendChild(aDiv);
				codeEdit.create({parent: aDiv.id, contentType: "text/cobol", contents: this.content}).then(function(editorViewer) {
					editorViewer.readonly = true;
					
					var editor = editorViewer.editor;
					
					if (editorViewer.settings) {
						editorViewer.settings.contentAssistAutoTrigger = false;
						editorViewer.settings.showOccurrences = false;
					}
					//You can call APIs from editorViewer.editor  for further actions.
					var cobolStyler = new mCobolStyler.CobolStyler("FIXME");
					var styleParser = new mStyleParser.StyleParser(cobolStyler, null);
					this.styler = new mTextStyler.TextStyler(editor, styleParser );
					this.styler.initialize();
				});
			} else {
				var _streamId = this.streamId;
				var _fileItemId = this.fileItemId;
				var _type = this.type;
				var _project = this.project;
				var _compentId = this.compentId;
				var _generated = this.generated;
				var _that = this.embeddedEditor;
				var _size = rppCurrentPaneHeight;
				
				if ( this.size != undefined ) {
					_size = this.size;
				}
							
				require(["dojo/dom-construct"], function(domConst) {
					domConst.create('iframe', {
						title : "Source",
						src: net.jazz.ajax.contextRoot()  + "/web/com.ibm.teampdp.query.engine.web/ui/built-codeEdit/orion-viewer-amd-rtc.html"
				    		+ "?streamId="+_streamId+"&fileItemId="+_fileItemId+"&type="+_type+"&project="+_project+"&compentId="+_compentId+"&generated="+_generated+"&rppDebug="+rppDebug+"&size="+(_size-10)+'px',
				    		"style": "border: 0; width: 100%; height: " + _size+'px',
				    		scrolling:"no"
					}, _that);
				});
			}			
		},
		
		destroy:  function () {
			//this.inherited();
			if ( this.styler ) {
				this.styler.destroy();
			}
		},
		
	} );
	return OrionViewerWidget;
});
