/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2016, 2018. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget");

dojo.require( "dojo/on" );
dojo.require( "dijit._Widget" );
dojo.require( "dijit._WidgetBase" );
dojo.require( "dijit._TemplatedMixin" );
dojo.require( "dijit._WidgetsInTemplateMixin" );
dojo.require( "dijit.registry" );

define("com.ibm.teampdp.query.engine.web/ui/internal/page/jsModules/widgets/RadicalEntityWidget", [],
function () {
	var RadicalEntityWidget = dojo.declare( "com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget", [
		dijit._Widget, dijit._WidgetBase, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin], {
		
		templatePath : dojo.moduleUrl( "com.ibm.teampdp.query.engine.web", "ui/internal/page/jsModules/widgets/RadicalEntityWidget.html" ),
		
		// Display result :   
		// 		icon + radical entity name 
		// OR if showLabel is true : 
		// 		icon + radical entity code + radical entity label
		// label is used for adding this widget in dojo table.
		constructor : function (widgetLabel, radicalEntity , showReLabel, streamId, workingProject, extraLabel, specificIcon,showItem) {
			this.init (widgetLabel, radicalEntity , showReLabel, streamId, workingProject, extraLabel, specificIcon,showItem);
		},
		
		// Display result :   
		// 		icon + radical entity name 
		// OR if showLabel is true : 
		// 		icon + radical entity code + radical entity label
		// label is used for adding this widget in dojo table.
		// to take account param 'showItem', specificIcon must be defined ("", ..) 
		// if showItem == false --> no icon before radical entity code
		// (showItem no defined or equal true --> icon before radical entity code)
		init : function (widgetLabel, radicalEntity , showReLabel, streamId, workingProject, extraLabel, specificIcon,showItem) {
			this.label = widgetLabel;
			this.reName = radicalEntity.name;
			this.reProject = radicalEntity.project;
			this.reLabel = radicalEntity.label;
			this.reClass = radicalEntity["class"];
			if ( radicalEntity.metaEntity != undefined )
				this.reMetaType = radicalEntity.metaEntity.name;
			else
				this.reMetaType = null;
			this.reText = this.reName;
			this.specificIcon = specificIcon;
			this.showItem = showItem;
			
			if ( radicalEntity.fileItemId != undefined ) {
				if ( radicalEntity.componentID != undefined ) {
					this.componentID = radicalEntity.componentID;
					this.fileItemId = radicalEntity.fileItemId;
				}
			}
			if ( this.fileItemId == undefined ) {
				console.debug ("No fileItemId or componentID for this radical entity -> " + this.reName );
			}
			
			
			if ( streamId != undefined )
				this.streamId = streamId;
			else {
				this.streamId = dijit.registry.byId( "streamId" ).getValue();
				console.debug("No stream id for : " + this.reName);
			}
			
			if ( workingProject != undefined )
				this.workingProject = workingProject;
			else {
				this.workingProject =  this.reProject == undefined ? "ERROR" :  this.reProject;
				console.debug("No working project for : " + this.reName);
			}
			
			if ( this.reProject == undefined || this.reProject.length == 0) {
				this.reProject = this.workingProject;
				console.debug("No project for : " + this.reName);
			}
			
			if ( showReLabel == true ) {
				if (extraLabel != undefined && extraLabel.length > 0 )
					this.reText = extraLabel;
				else 
					if (this.reLabel != undefined && this.reLabel.length > 0 )
						this.reText = this.reName + " - " + this.reLabel;
					else {
						this.reText = "";
					}
			}			
		},

		postCreate : function () {
			this.domNode.className = "";
			if ( this.reClass != undefined) {
				if (this.specificIcon != undefined && this.specificIcon.length > 0) {
					this.reIcon.className = this.specificIcon;
				}
				else {
					if (this.showItem == undefined || this.showItem.length == 0 || this.showItem ) {
						if (this.reClass.toLowerCase != undefined ) {
							this.reIcon.className = "display_icon " + this.reClass.toLowerCase();
						} else {
							this.reIcon.className = "display_icon " + this.reClass;
						}
					}
				}
			}
			this.displayText.innerHTML = this.reText;
			if ( this.reName ) {
				var _that = this;
				require(["dojo/on", "dijit/registry"], function(on, registry){
					on(_that.displayText, "click", function(){
						_that.openTheRadicalElement();
					});
				});
			}
		},
		
		openTheRadicalElement: function(){
			if ( this.reClass != undefined && this.reClass.toLowerCase != undefined ) {
				if ( this.fileItemId == undefined )
					openRadicalElement ( this.streamId, this.workingProject, this.reProject, this.reClass.toLowerCase(), this.reName);
				else
					openArtifactFile ( this.streamId, this.componentID, this.fileItemId, this.workingProject, this.reProject, this.reClass.toLowerCase(), this.reMetaType, this.reName);
			}
			else {
				if ( this.fileItemId == undefined )
					openRadicalElement ( this.streamId, this.workingProject, this.reProject, this.reClass, this.reName);
				else
					openArtifactFile ( this.streamId, this.componentID, this.fileItemId, this.workingProject, this.reProject, this.reClass.toString(), this.reMetaType, this.reName);
			}
        }	
	} );
	return RadicalEntityWidget;
});
